@@ -45,30 +45,22 @@ class FaultWrapper(wsgi.Middleware):
 
     error_map = {
         'ActionInProgress': webob.exc.HTTPConflict,
-        'ActionNotFound': webob.exc.HTTPNotFound,
-        'ClusterNotFound': webob.exc.HTTPNotFound,
-        'EventNotFound': webob.exc.HTTPNotFound,
+        'BadRequest': webob.exc.HTTPBadRequest,
         'FeatureNotSupported': webob.exc.HTTPConflict,
         'Forbidden': webob.exc.HTTPForbidden,
         'InternalError': webob.exc.HTTPInternalServerError,
+        'InvalidGlobalAPIVersion': webob.exc.HTTPNotAcceptable,
         'InvalidParameter': webob.exc.HTTPBadRequest,
         'InvalidSchemaError': webob.exc.HTTPBadRequest,
+        'MethodVersionNotFound': webob.exc.HTTPBadRequest,
         'MultipleChoices': webob.exc.HTTPBadRequest,
-        'NodeNotFound': webob.exc.HTTPNotFound,
         'NodeNotOrphan': webob.exc.HTTPConflict,
         'PolicyBindingNotFound': webob.exc.HTTPNotFound,
-        'PolicyNotFound': webob.exc.HTTPNotFound,
-        'PolicyTypeNotFound': webob.exc.HTTPNotFound,
-        'ProfileInUse': webob.exc.HTTPConflict,
-        'ProfileNotFound': webob.exc.HTTPNotFound,
-        'ProfileTypeNotFound': webob.exc.HTTPNotFound,
         'ProfileTypeNotMatch': webob.exc.HTTPBadRequest,
-        'ReceiverNotFound': webob.exc.HTTPNotFound,
         'RequestLimitExceeded': webob.exc.HTTPBadRequest,
         'ResourceInUse': webob.exc.HTTPConflict,
-        'BadRequest': webob.exc.HTTPBadRequest,
+        'ResourceNotFound': webob.exc.HTTPNotFound,
         'SpecValidationFailed': webob.exc.HTTPBadRequest,
-        'WebhookNotFound': webob.exc.HTTPNotFound,
     }
 
     def _map_exception_to_error(self, class_exception):
