@@ -22,17 +22,17 @@ from oslo_service import service
 
 from senlin.common import consts
 from senlin.common import messaging
+from senlin import objects
 
 _lazy.enable_lazy()
 
-LOG = logging.getLogger('senlin.engine')
-
 
 def main():
     logging.register_options(cfg.CONF)
     cfg.CONF(project='senlin', prog='senlin-engine')
     logging.setup(cfg.CONF, 'senlin-engine')
     logging.set_defaults()
+    objects.register_all()
     messaging.setup()
 
     from senlin.engine import service as engine
