@@ -10,8 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import logging
-
+from oslo_log import log as logging
 
 RPC_ATTRS = (
     ENGINE_TOPIC,
@@ -22,7 +21,7 @@ RPC_ATTRS = (
     'senlin-engine',
     'engine-dispatcher',
     'engine-health-mgr',
-    '1.0',
+    '1.1',
 )
 
 RPC_PARAMS = (
@@ -36,7 +35,7 @@ RPC_PARAMS = (
 ACTION_NAMES = (
     CLUSTER_CREATE, CLUSTER_DELETE, CLUSTER_UPDATE,
     CLUSTER_ADD_NODES, CLUSTER_DEL_NODES, CLUSTER_RESIZE,
-    CLUSTER_CHECK, CLUSTER_RECOVER,
+    CLUSTER_CHECK, CLUSTER_RECOVER, CLUSTER_REPLACE_NODES,
     CLUSTER_SCALE_OUT, CLUSTER_SCALE_IN,
     CLUSTER_ATTACH_POLICY, CLUSTER_DETACH_POLICY, CLUSTER_UPDATE_POLICY,
 
@@ -46,7 +45,7 @@ ACTION_NAMES = (
 ) = (
     'CLUSTER_CREATE', 'CLUSTER_DELETE', 'CLUSTER_UPDATE',
     'CLUSTER_ADD_NODES', 'CLUSTER_DEL_NODES', 'CLUSTER_RESIZE',
-    'CLUSTER_CHECK', 'CLUSTER_RECOVER',
+    'CLUSTER_CHECK', 'CLUSTER_RECOVER', 'CLUSTER_REPLACE_NODES',
     'CLUSTER_SCALE_OUT', 'CLUSTER_SCALE_IN',
     'CLUSTER_ATTACH_POLICY', 'CLUSTER_DETACH_POLICY', 'CLUSTER_UPDATE_POLICY',
 
@@ -153,7 +152,7 @@ EVENT_ATTRS = (
     EVENT_USER, EVENT_ACTION, EVENT_STATUS, EVENT_STATUS_REASON,
     EVENT_LEVEL, EVENT_CLUSTER_ID,
 ) = (
-    'timestamp', 'obj_id', 'obj_name', 'obj_type',
+    'timestamp', 'oid', 'oname', 'otype',
     'user', 'action', 'status', 'status_reason',
     'level', 'cluster_id',
 )
@@ -183,9 +182,9 @@ ACTION_SORT_KEYS = [
 ]
 
 RECEIVER_TYPES = (
-    RECEIVER_WEBHOOK,
+    RECEIVER_WEBHOOK, RECEIVER_MESSAGE,
 ) = (
-    'webhook',
+    'webhook', 'message',
 )
 
 RECEIVER_ATTRS = (
@@ -226,3 +225,9 @@ EVENT_LEVELS = {
     'INFO': logging.INFO,
     'DEBUG': logging.DEBUG,
 }
+
+RECOVER_OPERATIONS = (
+    RECOVER_RECREATE, RECOVER_REBUILD,
+) = (
+    'RECREATE', 'REBUILD',
+)
