@@ -31,13 +31,12 @@ LOG = logging.getLogger(__name__)
 
 
 class SenlinException(Exception):
-    '''Base Senlin Exception.
-
-    To correctly use this class, inherit from it and define
-    a 'msg_fmt' property. That msg_fmt will get printf'd
-    with the keyword arguments provided to the constructor.
-    '''
+    """Base Senlin Exception.
 
+    To correctly use this class, inherit from it and define a 'msg_fmt'
+    property. That msg_fmt will get printed with the keyword arguments
+    provided to the constructor.
+    """
     message = _("An unknown exception occurred.")
 
     def __init__(self, **kwargs):
@@ -45,12 +44,15 @@ class SenlinException(Exception):
 
         try:
             self.message = self.msg_fmt % kwargs
+            # if last char is '.', wipe out redundant '.'
+            if self.message[-1] == '.':
+                self.message = self.message.rstrip('.') + '.'
         except KeyError:
             # exc_info = sys.exc_info()
             # if kwargs doesn't match a variable in the message
             # log the issue and the kwargs
             LOG.exception(_LE('Exception in string format operation'))
-            for name, value in six.iteritems(kwargs):
+            for name, value in kwargs.items():
                 LOG.error("%s: %s" % (name, value))  # noqa
 
             if _FATAL_EXCEPTION_FORMAT_ERRORS:
@@ -80,7 +82,21 @@ class Forbidden(SenlinException):
 
 
 class BadRequest(SenlinException):
-    msg_fmt = _("The request is malformed: %(msg)s")
+    msg_fmt = _("The request is malformed: %(msg)s.")
+
+
+class InvalidAPIVersionString(SenlinException):
+    msg_fmt = _("API Version String (%(version)s) is of invalid format. It "
+                "must be of format 'major.minor'.")
+
+
+class MethodVersionNotFound(SenlinException):
+    msg_fmt = _("API version %(version)s is not supported on this method.")
+
+
+class InvalidGlobalAPIVersion(SenlinException):
+    msg_fmt = _("Version %(req_ver)s is not supported by the API. Minimum is "
+                "%(min_ver)s and maximum is %(max_ver)s.")
 
 
 class MultipleChoices(SenlinException):
@@ -92,26 +108,35 @@ class InvalidParameter(SenlinException):
     msg_fmt = _("Invalid value '%(value)s' specified for '%(name)s'")
 
 
-class ClusterNotFound(SenlinException):
-    msg_fmt = _("The cluster (%(cluster)s) could not be found.")
+class ResourceNotFound(SenlinException):
+    """Generic exception for resource not found.
+
+    The resource type here can be 'cluster', 'node', 'profile',
+    'policy', 'receiver', 'webhook', 'profile_type', 'policy_type',
+    'action', 'event' and so on.
+    """
+    msg_fmt = _("The %(type)s (%(id)s) could not be found.")
+
+    @staticmethod
+    def enhance_msg(enhance, ex):
+        enhance_msg = ex.message[:4] + enhance + ' ' + ex.message[4:]
+        return enhance_msg
 
 
-class NodeNotFound(SenlinException):
-    msg_fmt = _("The node (%(node)s) could not be found.")
+class ResourceInUse(SenlinException):
+    """Generic exception for resource in use.
 
-
-class ProfileTypeNotFound(SenlinException):
-    msg_fmt = _("Profile type (%(profile_type)s) is not found.")
+    The resource type here can be 'cluster', 'node', 'profile',
+    'policy', 'receiver', 'webhook', 'profile_type', 'policy_type',
+    'action', 'event' and so on.
+    """
+    msg_fmt = _("The %(type)s %(id)s cannot be deleted: %(reason)s.")
 
 
 class ProfileTypeNotMatch(SenlinException):
     msg_fmt = _("%(message)s")
 
 
-class ProfileNotFound(SenlinException):
-    msg_fmt = _("The profile (%(profile)s) could not be found.")
-
-
 class ProfileNotSpecified(SenlinException):
     msg_fmt = _("Profile not specified.")
 
@@ -128,14 +153,6 @@ class PolicyNotSpecified(SenlinException):
     msg_fmt = _("Policy not specified.")
 
 
-class PolicyTypeNotFound(SenlinException):
-    msg_fmt = _("Policy type (%(policy_type)s) is not found.")
-
-
-class PolicyNotFound(SenlinException):
-    msg_fmt = _("The policy (%(policy)s) could not be found.")
-
-
 class PolicyBindingNotFound(SenlinException):
     msg_fmt = _("The policy (%(policy)s) is not found attached to the "
                 "specified cluster (%(identity)s).")
@@ -164,10 +181,6 @@ class Error(SenlinException):
         super(Error, self).__init__(message=msg)
 
 
-class ResourceInUse(SenlinException):
-    msg_fmt = _("The %(resource_type)s (%(resource_id)s) is still in use.")
-
-
 class InvalidContentType(SenlinException):
     msg_fmt = _("Invalid content type %(content_type)s")
 
@@ -176,25 +189,8 @@ class RequestLimitExceeded(SenlinException):
     msg_fmt = _('Request limit exceeded: %(message)s')
 
 
-class WebhookNotFound(SenlinException):
-    msg_fmt = _("The webhook (%(webhook)s) could not be found.")
-
-
-class ReceiverNotFound(SenlinException):
-    msg_fmt = _("The receiver (%(receiver)s) could not be found.")
-
-
-class ActionNotFound(SenlinException):
-    msg_fmt = _("The action (%(action)s) could not be found.")
-
-
 class ActionInProgress(SenlinException):
-    msg_fmt = _("Cluster %(cluster_name)s already has an action (%(action)s) "
-                "in progress.")
-
-
-class EventNotFound(SenlinException):
-    msg_fmt = _("The event (%(event)s) could not be found.")
+    msg_fmt = _("The %(type)s %(id)s is in status %(status)s.")
 
 
 class NodeNotOrphan(SenlinException):
@@ -202,24 +198,25 @@ class NodeNotOrphan(SenlinException):
 
 
 class InternalError(SenlinException):
-    '''A base class for internal exceptions in senlin.
+    """A base class for internal exceptions in senlin.
 
     The internal exception classes which inherit from :class:`InternalError`
     class should be translated to a user facing exception type if need to be
     made user visible.
-    '''
-    msg_fmt = _('ERROR %(code)s happens for %(message)s.')
-    message = _('Internal error happens')
+    """
+    msg_fmt = _("%(message)s")
+    message = _('Internal error happened')
 
     def __init__(self, **kwargs):
-        super(InternalError, self).__init__(**kwargs)
-        if 'code' in kwargs:
-            self.code = kwargs.get('code', 500)
-            self.message = kwargs.get('message', self.message)
+        self.code = kwargs.pop('code', 500)
+        self.message = kwargs.pop('message', self.message)
+        super(InternalError, self).__init__(
+            code=self.code, message=self.message, **kwargs)
 
 
-class ResourceBusyError(InternalError):
-    msg_fmt = _("The %(resource_type)s (%(resource_id)s) is busy now.")
+class EResourceBusy(InternalError):
+    # Internal exception, not to be exposed to end user.
+    msg_fmt = _("The %(type)s (%(id)s) is busy now.")
 
 
 class TrustNotFound(InternalError):
@@ -227,29 +224,36 @@ class TrustNotFound(InternalError):
     msg_fmt = _("The trust for trustor (%(trustor)s) could not be found.")
 
 
-class ResourceCreationFailure(InternalError):
+class EResourceCreation(InternalError):
     # Used when creating resources in other services
-    msg_fmt = _("Failed in creating %(rtype)s.")
+    msg_fmt = _("Failed in creating %(type)s: %(message)s.")
 
 
-class ResourceUpdateFailure(InternalError):
+class EResourceUpdate(InternalError):
     # Used when updating resources from other services
-    msg_fmt = _("Failed in updating %(resource)s.")
+    msg_fmt = _("Failed in updating %(type)s %(id)s: %(message)s.")
 
 
-class ResourceDeletionFailure(InternalError):
+class EResourceDeletion(InternalError):
     # Used when deleting resources from other services
-    msg_fmt = _("Failed in deleting %(resource)s.")
+    msg_fmt = _("Failed in deleting %(type)s %(id)s: %(message)s.")
 
 
-class ResourceNotFound(InternalError):
-    # Used when retrieving resources from other services
-    msg_fmt = _("The resource (%(resource)s) could not be found.")
+class EResourceOperation(InternalError):
+    """Generic exception for resource fail operation.
 
-
-class ResourceStatusError(InternalError):
-    msg_fmt = _("The resource %(resource_id)s is in error status "
-                "- '%(status)s' due to '%(reason)s'.")
+    The op here can be 'recovering','rebuilding', 'checking' and
+    so on. And the op 'creating', 'updating' and 'deleting' we can
+    use separately class `EResourceCreation`,`EResourceUpdate` and
+    `EResourceDeletion`.
+    The type here is resource's driver type.It can be 'server',
+    'stack', 'container' and so on.
+    The id is resource's id.
+    The message here can be message from class 'ResourceNotFound',
+    'ResourceInUse' and so on, or developer can specified message.
+    """
+    # Used when operating resources from other services
+    msg_fmt = _("Failed in %(op)s %(type)s %(id)s: %(message)s.")
 
 
 class InvalidPlugin(InternalError):
