@@ -14,12 +14,15 @@ import collections
 import numbers
 import six
 
+from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import strutils
 
 from senlin.common import exception
 from senlin.common.i18n import _
 
+LOG = logging.getLogger(__name__)
+
 
 class AnyIndexDict(collections.Mapping):
     '''Convenience schema for a list.'''
@@ -39,15 +42,15 @@ class AnyIndexDict(collections.Mapping):
         return 1
 
 
-class Schema(collections.Mapping):
-    '''Class for validating profile and policy specifications.'''
+class SchemaBase(collections.Mapping):
+    """Class for validating property or operation schemas."""
 
     KEYS = (
-        TYPE, DESCRIPTION, DEFAULT, REQUIRED, SCHEMA, UPDATABLE,
-        CONSTRAINTS, READONLY,
+        TYPE, DESCRIPTION, DEFAULT, REQUIRED, SCHEMA, CONSTRAINTS,
+        MIN_VERSION, MAX_VERSION,
     ) = (
-        'type', 'description', 'default', 'required', 'schema', 'updatable',
-        'constraints', 'readonly',
+        'type', 'description', 'default', 'required', 'schema', 'constraints',
+        'min_version', 'max_version',
     )
 
     TYPES = (
@@ -56,12 +59,11 @@ class Schema(collections.Mapping):
         'Integer', 'String', 'Number', 'Boolean', 'Map', 'List',
     )
 
-    def __init__(self, description=None, default=None,
-                 required=False, schema=None, updatable=False,
-                 readonly=False, constraints=None):
-
+    def __init__(self, description=None, default=None, required=False,
+                 schema=None, constraints=None, min_version=None,
+                 max_version=None):
         if schema is not None:
-            if type(self) not in (List, Map):
+            if type(self) not in (List, Map, Operation):
                 msg = _('Schema valid only for List or Map, not '
                         '"%s"') % self[self.TYPE]
                 raise exception.InvalidSchemaError(message=msg)
@@ -74,10 +76,10 @@ class Schema(collections.Mapping):
         self.description = description
         self.default = default
         self.required = required
-        self.updatable = updatable
         self.constraints = constraints or []
-        self.readonly = readonly
         self._len = None
+        self.min_version = min_version
+        self.max_version = max_version
 
     def has_default(self):
         return self.default is not None
@@ -97,10 +99,10 @@ class Schema(collections.Mapping):
                 dict(default=self.default, exc=exc))
 
     def validate(self, context=None):
-        '''Validates the schema.
+        """Validates the schema.
 
-        This method checks if the schema itself is valid.
-        '''
+        :param context: A RequestContext instance for deep validation.
+        """
         self._validate_default(context)
 
         # validated nested schema: List or Map
@@ -122,6 +124,26 @@ class Schema(collections.Mapping):
         except ValueError as ex:
             raise exception.SpecValidationFailed(message=six.text_type(ex))
 
+    def _validate_version(self, version, key):
+        if self.min_version and self.min_version > version:
+            msg = _('%(key)s(min_version=%(min)s) is not supported by '
+                    'spec version %(version)s.'
+                    ) % {'key': key, 'min': self.min_version,
+                         'version': version}
+            raise exception.SpecValidationFailed(message=msg)
+        if self.max_version:
+            if version > self.max_version:
+                msg = _('%(key)s(max_version=%(max)s) is not supported '
+                        'by spec version %(version)s.'
+                        ) % {'version': version, 'max': self.max_version,
+                             'key': key}
+                raise exception.SpecValidationFailed(message=msg)
+            else:
+                msg = _('Warning: %(key)s will be deprecated after version '
+                        '%(version)s!') % {'key': key,
+                                           'version': self.max_version}
+                LOG.warning(msg)
+
     def __getitem__(self, key):
         if key == self.DESCRIPTION:
             if self.description is not None:
@@ -134,10 +156,6 @@ class Schema(collections.Mapping):
                 return dict((n, dict(s)) for n, s in self.schema.items())
         elif key == self.REQUIRED:
             return self.required
-        elif key == self.READONLY:
-            return self.readonly
-        elif key == self.UPDATABLE:
-            return self.updatable
         elif key == self.CONSTRAINTS:
             if self.constraints:
                 return [dict(c) for c in self.constraints]
@@ -159,12 +177,40 @@ class Schema(collections.Mapping):
         return self._len
 
 
-class Boolean(Schema):
+class PropertySchema(SchemaBase):
+    """Class for validating profile and policy specifications."""
+
+    KEYS = (
+        TYPE, DESCRIPTION, DEFAULT, REQUIRED, SCHEMA, UPDATABLE,
+        CONSTRAINTS, MIN_VERSION, MAX_VERSION,
+    ) = (
+        'type', 'description', 'default', 'required', 'schema', 'updatable',
+        'constraints', 'min_version', 'max_version',
+    )
+
+    def __init__(self, description=None, default=None, required=False,
+                 schema=None, updatable=False, constraints=None,
+                 min_version=None, max_version=None):
+        super(PropertySchema, self).__init__(description=description,
+                                             default=default,
+                                             required=required, schema=schema,
+                                             constraints=constraints,
+                                             min_version=min_version,
+                                             max_version=max_version)
+        self.updatable = updatable
+
+    def __getitem__(self, key):
+        if key == self.UPDATABLE:
+            return self.updatable
+
+        return super(PropertySchema, self).__getitem__(key)
+
+
+class Boolean(PropertySchema):
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.BOOLEAN
-        else:
-            return super(Boolean, self).__getitem__(key)
+        return super(Boolean, self).__getitem__(key)
 
     def to_schema_type(self, value):
         return strutils.bool_from_string(str(value), strict=True)
@@ -183,12 +229,11 @@ class Boolean(Schema):
         self.resolve(value)
 
 
-class Integer(Schema):
+class Integer(PropertySchema):
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.INTEGER
-        else:
-            return super(Integer, self).__getitem__(key)
+        return super(Integer, self).__getitem__(key)
 
     def to_schema_type(self, value):
         if isinstance(value, six.integer_types):
@@ -196,7 +241,7 @@ class Integer(Schema):
         try:
             num = int(value)
         except ValueError:
-            raise ValueError(_('%s is not an intger.') % num)
+            raise ValueError(_('%s is not an integer.') % num)
 
         return num
 
@@ -215,12 +260,11 @@ class Integer(Schema):
         self.validate_constraints(value, self, context)
 
 
-class String(Schema):
+class String(PropertySchema):
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.STRING
-        else:
-            return super(String, self).__getitem__(key)
+        return super(String, self).__getitem__(key)
 
     def to_schema_type(self, value):
         return str(value)
@@ -228,8 +272,8 @@ class String(Schema):
     def resolve(self, value):
         try:
             return str(value)
-        except (TypeError, ValueError) as ex:
-            raise ex
+        except (TypeError, ValueError):
+            raise
 
     def validate(self, value, context=None):
         if not isinstance(value, six.string_types):
@@ -241,12 +285,11 @@ class String(Schema):
         self.validate_constraints(value, self, context)
 
 
-class Number(Schema):
+class Number(PropertySchema):
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.NUMBER
-        else:
-            return super(Number, self).__getitem__(key)
+        return super(Number, self).__getitem__(key)
 
     def to_schema_type(self, value):
         if isinstance(value, numbers.Number):
@@ -274,12 +317,11 @@ class Number(Schema):
         self.resolve_constraints(value, self, context)
 
 
-class List(Schema):
+class List(PropertySchema):
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.LIST
-        else:
-            return super(List, self).__getitem__(key)
+        return super(List, self).__getitem__(key)
 
     def _get_children(self, values, keys, context):
         sub_schema = self.schema
@@ -317,12 +359,11 @@ class List(Schema):
             child.validate(item_value, context)
 
 
-class Map(Schema):
+class Map(PropertySchema):
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.MAP
-        else:
-            return super(Map, self).__getitem__(key)
+        return super(Map, self).__getitem__(key)
 
     def _get_children(self, values, context=None):
         # There are cases where the Map is not specified to the very detailed
@@ -332,7 +373,7 @@ class Map(Schema):
 
         sub_schema = self.schema
         if sub_schema is not None:
-            # sub_schema shoud be a dict here
+            # sub_schema should be a dict here.
             subspec = Spec(sub_schema, dict(values))
             subspec.validate()
 
@@ -356,7 +397,7 @@ class Map(Schema):
         if not isinstance(value, collections.Mapping):
             raise TypeError(_('"%s" is not a Map') % value)
 
-        return dict(self._get_children(six.iteritems(value), context))
+        return dict(self._get_children(value.items(), context))
 
     def validate(self, value, context=None):
         if not isinstance(value, collections.Mapping):
@@ -367,18 +408,33 @@ class Map(Schema):
             child.validate(item_value, context)
 
 
+class StringParam(SchemaBase):
+    def _getitem__(self, key):
+        if key == self.TYPE:
+            return self.STRING
+
+
+class Operation(Map):
+    """Class for specifying operations on profiles."""
+    pass
+
+
 class Spec(collections.Mapping):
     '''A class that contains all spec items.'''
-    def __init__(self, schema, data):
+    def __init__(self, schema, data, version=None):
         self._schema = schema
         self._data = data
+        self._version = version
 
     def validate(self):
         '''Validate the schema.'''
         for (k, s) in self._schema.items():
             try:
-                # validate through resolve
+                # Validate through resolve
                 self.resolve_value(k)
+                # Validate schema for versioned spec
+                if self._version:
+                    self._schema[k]._validate_version(self._version, k)
             except (TypeError, ValueError) as err:
                 msg = _('Spec validation error (%(key)s): %(err)s') % dict(
                     key=k, err=six.text_type(err))
