@@ -10,28 +10,33 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-'''
-Utilities module.
-'''
+"""
+Common utilities module.
+"""
 
 import random
 import string
 
-import requests
-from requests import exceptions
-from six.moves import urllib
-
+from jsonpath_rw import parse
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import strutils
+from oslo_utils import timeutils
+import requests
+import six
+from six.moves import urllib
 
 from senlin.common import consts
 from senlin.common import exception
 from senlin.common.i18n import _
 from senlin.common.i18n import _LI
+from senlin.objects import service as service_obj
 
 cfg.CONF.import_opt('max_response_size', 'senlin.common.config')
+cfg.CONF.import_opt('periodic_interval', 'senlin.common.config')
+
 LOG = logging.getLogger(__name__)
+_ISO8601_TIME_FORMAT = '%Y-%m-%dT%H:%M:%S'
 
 
 class URLFetchError(exception.Error, IOError):
@@ -153,7 +158,7 @@ def url_fetch(url, allowed_schemes=('http', 'https')):
                                     " bytes)" % cfg.CONF.max_response_size)
         return result
 
-    except exceptions.RequestException as ex:
+    except requests.exceptions.RequestException as ex:
         raise URLFetchError(_('Failed to retrieve data: %s') % ex)
 
 
@@ -167,9 +172,54 @@ def random_name(length=8):
     return lead + tail
 
 
-def format_time(value):
-    """Cut microsecond and format to isoformat string."""
-    if value:
-        value = value.replace(microsecond=0)
-        value = value.isoformat()
-    return value
+def isotime(at):
+    """Stringify time in ISO 8601 format.
+
+    oslo.versionedobject is using this function for datetime formatting.
+    """
+    if at is None:
+        return None
+
+    st = at.strftime(_ISO8601_TIME_FORMAT)
+    tz = at.tzinfo.tzname(None) if at.tzinfo else 'UTC'
+    st += ('Z' if tz == 'UTC' else tz)
+    return st
+
+
+def get_path_parser(path):
+    """Get a JsonPath parser based on a path string.
+
+    :param path: A string containing a JsonPath.
+    :returns: A parser used for path matching.
+    :raises: An exception of `BadRequest` if the path fails validation.
+    """
+    try:
+        expr = parse(path)
+    except Exception as ex:
+        error_text = six.text_type(ex)
+        error_msg = error_text.split(':', 1)[1]
+        raise exception.BadRequest(
+            msg=_("Invalid attribute path - %s") % error_msg.strip())
+
+    return expr
+
+
+def is_engine_dead(ctx, engine_id, duration=None):
+    """Check if an engine is dead.
+
+    If engine hasn't reported its status for the given duration, it is treated
+    as a dead engine.
+
+    :param ctx: A request context.
+    :param engine_id: The ID of the engine to test.
+    :param duration: The time duration in seconds.
+    """
+    if not duration:
+        duration = 2 * cfg.CONF.periodic_interval
+
+    eng = service_obj.Service.get(ctx, engine_id)
+    if not eng:
+        return True
+    if timeutils.is_older_than(eng.updated_at, duration):
+        return True
+    return False
