@@ -16,11 +16,13 @@ from senlin.drivers.openstack import keystone_v3
 from senlin.drivers.openstack import lbaas
 from senlin.drivers.openstack import neutron_v2
 from senlin.drivers.openstack import nova_v2
+from senlin.drivers.openstack import zaqar_v2
 
 
 compute = nova_v2.NovaClient
 identity = keystone_v3.KeystoneClient
 loadbalancing = lbaas.LoadBalancerDriver
+message = zaqar_v2.ZaqarClient
 network = neutron_v2.NeutronClient
 orchestration = heat_v1.HeatClient
 telemetry = ceilometer_v2.CeilometerClient
