@@ -14,6 +14,7 @@ import copy
 import eventlet
 
 from oslo_log import log as logging
+from oslo_utils import timeutils
 
 from senlin.common import consts
 from senlin.common import exception
@@ -21,7 +22,6 @@ from senlin.common.i18n import _
 from senlin.common.i18n import _LI
 from senlin.common import scaleutils
 from senlin.common import utils
-from senlin.db import api as db_api
 from senlin.engine.actions import base
 from senlin.engine import cluster as cluster_mod
 from senlin.engine import dispatcher
@@ -29,6 +29,10 @@ from senlin.engine import event as EVENT
 from senlin.engine import node as node_mod
 from senlin.engine import scheduler
 from senlin.engine import senlin_lock
+from senlin.objects import action as ao
+from senlin.objects import cluster as co
+from senlin.objects import dependency as dobj
+from senlin.objects import node as no
 from senlin.policies import base as policy_mod
 
 LOG = logging.getLogger(__name__)
@@ -42,14 +46,15 @@ class ClusterAction(base.Action):
         CLUSTER_ADD_NODES, CLUSTER_DEL_NODES,
         CLUSTER_RESIZE, CLUSTER_CHECK, CLUSTER_RECOVER,
         CLUSTER_SCALE_OUT, CLUSTER_SCALE_IN,
-        CLUSTER_ATTACH_POLICY, CLUSTER_DETACH_POLICY, CLUSTER_UPDATE_POLICY
+        CLUSTER_ATTACH_POLICY, CLUSTER_DETACH_POLICY, CLUSTER_UPDATE_POLICY,
+        CLUSTER_REPLACE_NODES
     ) = (
         consts.CLUSTER_CREATE, consts.CLUSTER_DELETE, consts.CLUSTER_UPDATE,
         consts.CLUSTER_ADD_NODES, consts.CLUSTER_DEL_NODES,
         consts.CLUSTER_RESIZE, consts.CLUSTER_CHECK, consts.CLUSTER_RECOVER,
         consts.CLUSTER_SCALE_OUT, consts.CLUSTER_SCALE_IN,
         consts.CLUSTER_ATTACH_POLICY, consts.CLUSTER_DETACH_POLICY,
-        consts.CLUSTER_UPDATE_POLICY,
+        consts.CLUSTER_UPDATE_POLICY, consts.CLUSTER_REPLACE_NODES
     )
 
     def __init__(self, target, action, context, **kwargs):
@@ -67,6 +72,10 @@ class ClusterAction(base.Action):
         except Exception:
             self.cluster = None
 
+    def _sleep(self, period):
+        if period:
+            eventlet.sleep(period)
+
     def _wait_for_dependents(self):
         """Wait for dependent actions to complete.
 
@@ -117,7 +126,7 @@ class ClusterAction(base.Action):
         nodes = []
         child = []
         for m in range(count):
-            index = db_api.cluster_next_index(self.context, self.cluster.id)
+            index = co.Cluster.get_next_index(self.context, self.cluster.id)
             kwargs = {
                 'index': index,
                 'metadata': {},
@@ -148,11 +157,11 @@ class ClusterAction(base.Action):
 
         if child:
             # Build dependency and make the new action ready
-            db_api.dependency_add(self.context, [a for a in child], self.id)
+            dobj.Dependency.create(self.context, [a for a in child], self.id)
             for cid in child:
-                db_api.action_update(self.context, cid,
-                                     {'status': base.Action.READY})
-                dispatcher.start_action(action_id=cid)
+                ao.Action.update(self.context, cid,
+                                 {'status': base.Action.READY})
+            dispatcher.start_action()
 
             # Wait for cluster creation to complete
             res, reason = self._wait_for_dependents()
@@ -185,14 +194,11 @@ class ClusterAction(base.Action):
 
         result, reason = self._create_nodes(self.cluster.desired_capacity)
 
+        params = {}
         if result == self.RES_OK:
             reason = _('Cluster creation succeeded.')
-            self.cluster.set_status(self.context, self.cluster.ACTIVE, reason)
-        elif result in [self.RES_CANCEL, self.RES_TIMEOUT, self.RES_ERROR]:
-            self.cluster.set_status(self.context, self.cluster.ERROR, reason)
-        else:
-            # in case of RES_RETRY, need to reset cluster status
-            self.cluster.set_status(self.context, self.cluster.INIT)
+            params = {'created_at': timeutils.utcnow(True)}
+        self.cluster.eval_status(self.context, self.CLUSTER_CREATE, **params)
 
         return result, reason
 
@@ -222,7 +228,8 @@ class ClusterAction(base.Action):
 
         reason = _('Cluster update completed.')
         if profile_id is None:
-            self.cluster.set_status(self.context, self.cluster.ACTIVE, reason)
+            self.cluster.eval_status(self.context, self.CLUSTER_UPDATE,
+                                     updated_at=timeutils.utcnow(True))
             return self.RES_OK, reason
 
         fmt = _LI("Updating cluster '%(cluster)s': profile='%(profile)s'.")
@@ -241,21 +248,21 @@ class ClusterAction(base.Action):
             child.append(action_id)
 
         if child:
-            db_api.dependency_add(self.context, [c for c in child], self.id)
+            dobj.Dependency.create(self.context, [c for c in child], self.id)
             for cid in child:
-                db_api.action_update(self.context, cid,
-                                     {'status': base.Action.READY})
-                dispatcher.start_action(action_id=cid)
+                ao.Action.update(self.context, cid,
+                                 {'status': base.Action.READY})
+            dispatcher.start_action()
 
             result, new_reason = self._wait_for_dependents()
             if result != self.RES_OK:
-                new_reason = _('Failed in updating nodes.')
-                self.cluster.set_status(self.context, self.cluster.WARNING,
-                                        new_reason)
-                return result, new_reason
+                self.cluster.eval_status(self.context, self.CLUSTER_UPDATE)
+                return result, _('Failed in updating nodes.')
 
-        self.cluster.set_status(self.context, self.cluster.ACTIVE,
-                                reason, profile_id=profile_id)
+        self.cluster.profile_id = profile_id
+        self.cluster.eval_status(self.context, self.CLUSTER_UPDATE,
+                                 profile_id=profile_id,
+                                 updated_at=timeutils.utcnow(True))
         return self.RES_OK, reason
 
     def _delete_nodes(self, node_ids):
@@ -278,18 +285,15 @@ class ClusterAction(base.Action):
             child.append(action_id)
 
         if child:
-            db_api.dependency_add(self.context, [c for c in child], self.id)
+            dobj.Dependency.create(self.context, [c for c in child], self.id)
             for cid in child:
                 # Build dependency and make the new action ready
-                db_api.action_update(self.context, cid,
-                                     {'status': base.Action.READY})
-                dispatcher.start_action(action_id=cid)
+                ao.Action.update(self.context, cid,
+                                 {'status': base.Action.READY})
+            dispatcher.start_action()
 
             res, reason = self._wait_for_dependents()
             if res == self.RES_OK:
-                self.cluster.desired_capacity -= len(node_ids)
-                self.cluster.store(self.context)
-
                 self.outputs['nodes_removed'] = node_ids
                 for node_id in node_ids:
                     self.cluster.remove_node(node_id)
@@ -300,9 +304,6 @@ class ClusterAction(base.Action):
 
         return self.RES_OK, ''
 
-    def _wait_before_deletion(self, period):
-        eventlet.sleep(period)
-
     def do_delete(self):
         """Handler for the CLUSTER_DELETE action.
 
@@ -324,81 +325,79 @@ class ClusterAction(base.Action):
         if result == self.RES_OK:
             res = self.cluster.do_delete(self.context)
             if not res:
+                self.cluster.eval_status(self.context, self.CLUSTER_DELETE)
                 return self.RES_ERROR, _('Cannot delete cluster object.')
-        elif result == self.RES_CANCEL:
-            self.cluster.set_status(self.context, self.cluster.ACTIVE, reason)
-        elif result in [self.RES_TIMEOUT, self.RES_ERROR]:
-            self.cluster.set_status(self.context, self.cluster.WARNING, reason)
         else:
-            # RETRY
-            pass
+            self.cluster.eval_status(self.context, self.CLUSTER_DELETE)
 
         return result, reason
 
     def do_add_nodes(self):
         """Handler for the CLUSTER_ADD_NODES action.
 
+        TODO(anyone): handle placement data
+
         :returns: A tuple containing the result and the corresponding reason.
         """
         node_ids = self.inputs.get('nodes')
-        # TODO(anyone): handle placement data
-
         errors = []
         nodes = []
-        for node_id in node_ids:
-            try:
-                node = node_mod.Node.load(self.context, node_id)
-            except exception.NodeNotFound:
-                errors.append(_('Node [%s] is not found.') % node_id)
+        for nid in node_ids:
+            node = no.Node.get(self.context, nid)
+            if not node:
+                errors.append(_('Node %s is not found.') % nid)
                 continue
+
             if node.cluster_id:
-                errors.append(_('Node [%(n)s] is already owned by cluster '
-                                '[%(c)s].') % {'n': node_id,
-                                               'c': node.cluster_id})
+                errors.append(_('Node %(n)s is already owned by cluster %(c)s.'
+                                ) % {'n': nid, 'c': node.cluster_id})
                 continue
-            if node.status != node.ACTIVE:
-                errors.append(_('Node [%s] is not in ACTIVE status.'
-                                ) % node_id)
+
+            if node.status != "ACTIVE":
+                errors.append(_('Node %s is not in ACTIVE status.') % nid)
                 continue
+
             nodes.append(node)
 
         if len(errors) > 0:
-            return self.RES_ERROR, ''.join(errors)
+            return self.RES_ERROR, '\n'.join(errors)
 
         reason = _('Completed adding nodes.')
-
+        current = no.Node.count_by_cluster(self.context, self.target)
         child = []
         for node in nodes:
+            nid = node.id
             kwargs = {
-                'name': 'node_join_%s' % node.id[:8],
+                'name': 'node_join_%s' % nid[:8],
                 'cause': base.CAUSE_DERIVED,
                 'inputs': {'cluster_id': self.target},
             }
-            action_id = base.Action.create(self.context, node.id,
-                                           consts.NODE_JOIN, **kwargs)
+            action_id = base.Action.create(self.context, nid, consts.NODE_JOIN,
+                                           **kwargs)
             child.append(action_id)
 
         if child:
-            db_api.dependency_add(self.context, [c for c in child], self.id)
+            dobj.Dependency.create(self.context, [c for c in child], self.id)
             for cid in child:
-                db_api.action_update(self.context, cid,
-                                     {'status': base.Action.READY})
-                dispatcher.start_action(action_id=cid)
+                ao.Action.update(self.context, cid,
+                                 {'status': base.Action.READY})
+            dispatcher.start_action()
 
         # Wait for dependent action if any
         result, new_reason = self._wait_for_dependents()
         if result != self.RES_OK:
             reason = new_reason
         else:
-            self.cluster.desired_capacity += len(nodes)
-            self.cluster.store(self.context)
-            nodes_added = [n.id for n in nodes]
-            self.outputs['nodes_added'] = nodes_added
+            desired = current + len(node_ids)
+            self.cluster.eval_status(self.context, self.CLUSTER_ADD_NODES,
+                                     desired_capacity=desired)
+            self.outputs['nodes_added'] = node_ids
             creation = self.data.get('creation', {})
-            creation['nodes'] = nodes_added
+            creation['nodes'] = node_ids
             self.data['creation'] = creation
             for node in nodes:
-                self.cluster.add_node(node)
+                obj = node_mod.Node.load(self.context, db_node=node)
+                self.cluster.add_node(obj)
 
         return result, reason
 
@@ -407,31 +406,37 @@ class ClusterAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        # Check if deletion policy is attached to the action, if is,
-        # get grace_period value
+        # Use policy decision if any, or fall back to defaults
+        destroy_after_deletion = False
+        grace_period = 0
+        reduce_desired_capacity = True
         pd = self.data.get('deletion', None)
-        grace_period = None
-        if pd is not None:
-            grace_period = self.data['deletion'].get('grace_period')
-        else:  # if not, deleting nodes from cluster, don't destroy them
-            data = {
-                'deletion': {
-                    'destroy_after_deletion': False,
-                }
+        if pd:
+            destroy_after_deletion = pd.get('destroy_after_deletion', False)
+            grace_period = pd.get('grace_period', 0)
+            reduce_desired_capacity = pd.get('reduce_desired_capacity', True)
+
+        data = {
+            'deletion': {
+                'destroy_after_deletion': destroy_after_deletion,
+                'grace_period': grace_period,
+                'reduce_desired_capacity': reduce_desired_capacity,
             }
-            self.data.update(data)
+        }
+        self.data.update(data)
         nodes = self.inputs.get('candidates', [])
 
         node_ids = copy.deepcopy(nodes)
         errors = []
         for node_id in node_ids:
-            try:
-                node = db_api.node_get(self.context, node_id)
-            except exception.NodeNotFound:
-                errors.append(_('Node [%s] is not found.') % node_id)
+            node = no.Node.get(self.context, node_id)
+
+            # The return value is None if node not found
+            if not node:
+                errors.append(_('Node %s is not found.') % node_id)
                 continue
-            if ((not node.cluster_id) or
-                    (node.cluster_id != self.cluster.id)):
+
+            if ((not node.cluster_id) or (node.cluster_id != self.target)):
                 nodes.remove(node_id)
 
         if len(errors) > 0:
@@ -441,39 +446,119 @@ class ClusterAction(base.Action):
         if len(nodes) == 0:
             return self.RES_OK, reason
 
-        if grace_period is not None:
-            self._wait_before_deletion(grace_period)
+        # sleep period
+        self._sleep(grace_period)
+
+        current = no.Node.count_by_cluster(self.context, self.target)
         result, new_reason = self._delete_nodes(nodes)
 
+        params = {}
         if result != self.RES_OK:
             reason = new_reason
+        if reduce_desired_capacity:
+            params['desired_capacity'] = current - len(nodes)
+
+        self.cluster.eval_status(self.context,
+                                 self.CLUSTER_DEL_NODES, **params)
+
         return result, reason
 
-    def _get_action_data(self, current_size):
-        if 'deletion' in self.data:
-            count = self.data['deletion']['count']
-            desired = current_size - count
-            candidates = self.data['deletion'].get('candidates', [])
-        elif 'creation' in self.data:
-            count = self.data['creation']['count']
-            desired = current_size + count
-            candidates = None
-        else:
-            return 0, 0, None
-        return count, desired, candidates
+    def do_replace_nodes(self):
+        """Handler for the CLUSTER_REPLACE_NODES action.
+
+        :returns: A tuple containing the result and the corresponding reason.
+        """
+        node_dict = self.inputs
+
+        errors = []
+        original_nodes = []
+        replacement_nodes = []
+        for (original, replacement) in node_dict.items():
+            original_node = no.Node.get(self.context, original)
+            replacement_node = no.Node.get(self.context, replacement)
+
+            # The return value is None if node not found
+            if not original_node:
+                errors.append(_('Original node %s not found.'
+                                ) % original)
+                continue
+            if not replacement_node:
+                errors.append(_('Replacement node %s not found.'
+                                ) % replacement)
+                continue
+            if original_node.cluster_id != self.target:
+                errors.append(_('Node %(o)s is not a member of the '
+                                'cluster %(c)s.') % {'o': original,
+                                                     'c': self.target})
+                continue
+            if replacement_node.cluster_id:
+                errors.append(_('Node %(r)s is already owned by cluster %(c)s.'
+                                ) % {'r': replacement,
+                                     'c': replacement_node.cluster_id})
+                continue
+            if replacement_node.status != node_mod.Node.ACTIVE:
+                errors.append(_('Node %s is not in ACTIVE status.'
+                                ) % replacement)
+                continue
+            original_nodes.append(original_node)
+            replacement_nodes.append(replacement_node)
+
+        if len(errors) > 0:
+            return self.RES_ERROR, '\n'.join(errors)
+
+        result = self.RES_OK
+        reason = _('Completed replacing nodes.')
+
+        children = []
+        for (original, replacement) in node_dict.items():
+            kwargs = {
+                'cause': base.CAUSE_DERIVED,
+            }
+
+            # node_leave action
+            kwargs['name'] = 'node_leave_%s' % original[:8]
+            leave_action_id = base.Action.create(self.context, original,
+                                                 consts.NODE_LEAVE, **kwargs)
+            # node_join action
+            kwargs['name'] = 'node_join_%s' % replacement[:8]
+            kwargs['inputs'] = {'cluster_id': self.target}
+            join_action_id = base.Action.create(self.context, replacement,
+                                                consts.NODE_JOIN, **kwargs)
+
+            children.append((join_action_id, leave_action_id))
+
+        if children:
+            dobj.Dependency.create(self.context, [c[0] for c in children],
+                                   self.id)
+            for child in children:
+                join_id = child[0]
+                leave_id = child[1]
+                ao.Action.update(self.context, join_id,
+                                 {'status': base.Action.READY})
+
+                dobj.Dependency.create(self.context, [join_id], leave_id)
+                ao.Action.update(self.context, leave_id,
+                                 {'status': base.Action.READY})
+
+                dispatcher.start_action()
+
+            result, new_reason = self._wait_for_dependents()
+            if result != self.RES_OK:
+                reason = new_reason
+            else:
+                for n in range(len(original_nodes)):
+                    self.cluster.remove_node(original_nodes[n])
+                    self.cluster.add_node(replacement_nodes[n])
+
+        self.cluster.eval_status(self.context, self.CLUSTER_REPLACE_NODES)
+        return result, reason
 
     def do_check(self):
         """Handler for CLUSTER_CHECK action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        saved_status = self.cluster.status
-        saved_reason = self.cluster.status_reason
-        res = self.cluster.do_check(self.context)
-        if not res:
-            reason = _('Cluster checking failed.')
-            self.cluster.set_status(self.context, saved_status, saved_reason)
-            return self.RES_ERROR, reason
+        self.cluster.do_check(self.context)
 
         child = []
         res = self.RES_OK
@@ -488,19 +573,18 @@ class ClusterAction(base.Action):
             child.append(action_id)
 
         if child:
-            db_api.dependency_add(self.context, [c for c in child], self.id)
+            dobj.Dependency.create(self.context, [c for c in child], self.id)
             for cid in child:
-                db_api.action_update(self.context, cid,
-                                     {'status': base.Action.READY})
-                dispatcher.start_action(action_id=cid)
+                ao.Action.update(self.context, cid,
+                                 {'status': base.Action.READY})
+            dispatcher.start_action()
 
             # Wait for dependent action if any
             res, new_reason = self._wait_for_dependents()
             if res != self.RES_OK:
                 reason = new_reason
 
-        self.cluster.set_status(self.context, saved_status, saved_reason)
-
+        self.cluster.eval_status(self.context, self.CLUSTER_CHECK)
         return res, reason
 
     def do_recover(self):
@@ -508,24 +592,18 @@ class ClusterAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        res = self.cluster.do_recover(self.context)
-        if not res:
-            reason = _('Cluster recovery failed.')
-            self.cluster.set_status(self.context, self.cluster.ERROR, reason)
-            return self.RES_ERROR, reason
+        self.cluster.do_recover(self.context)
 
         # process data from health_policy
         pd = self.data.get('health', None)
-        if pd is None:
-            pd = {
-                'health': {
-                    'recover_action': 'RECREATE',
-                }
-            }
-            self.data.update(pd)
-        recover_action = pd.get('recover_action', 'RECREATE')
-
-        reason = _('Cluster recovery succeeded.')
+        inputs = {}
+        if pd:
+            recover_action = pd.get('recover_action', None)
+            fencing = pd.get('fencing', None)
+            if recover_action is not None:
+                inputs['operation'] = recover_action
+            if fencing is not None and 'COMPUTE' in fencing:
+                inputs['force'] = True
 
         children = []
         for node in self.cluster.nodes:
@@ -535,70 +613,29 @@ class ClusterAction(base.Action):
             action_id = base.Action.create(
                 self.context, node_id, consts.NODE_RECOVER,
                 name='node_recover_%s' % node_id[:8],
-                cause=base.CAUSE_DERIVED,
-                inputs={'operation': recover_action}
+                cause=base.CAUSE_DERIVED, inputs=inputs,
             )
             children.append(action_id)
 
+        res = self.RES_OK
+        reason = _('Cluster recovery succeeded.')
         if children:
-            db_api.dependency_add(self.context, [c for c in children], self.id)
+            dobj.Dependency.create(self.context, [c for c in children],
+                                   self.id)
             for cid in children:
-                db_api.action_update(self.context, cid, {'status': 'READY'})
-                dispatcher.start_action(action_id=cid)
+                ao.Action.update(self.context, cid, {'status': 'READY'})
+            dispatcher.start_action()
 
             # Wait for dependent action if any
-            res, reason = self._wait_for_dependents()
-
+            res, new_reason = self._wait_for_dependents()
             if res != self.RES_OK:
-                self.cluster.set_status(self.context, self.cluster.ERROR,
-                                        reason)
-                return res, reason
+                reason = new_reason
 
-        self.cluster.set_status(self.context, self.cluster.ACTIVE, reason)
+        self.cluster.eval_status(self.context, self.CLUSTER_RECOVER)
+        return res, reason
 
-        return self.RES_OK, reason
-
-    def do_resize(self):
-        """Handler for the CLUSTER_RESIZE action.
-
-        :returns: A tuple containing the result and the corresponding reason.
-        """
-        self.cluster.set_status(self.context, self.cluster.RESIZING,
-                                'Cluster resize started.')
-        node_list = self.cluster.nodes
-        current_size = len(node_list)
-        count, desired, candidates = self._get_action_data(current_size)
-        grace_period = None
-        # if policy is attached to the cluster, use policy data directly,
-        # or parse resize params to get action data.
-        if count == 0:
-            result, reason = scaleutils.parse_resize_params(self, self.cluster)
-            if result != self.RES_OK:
-                status_reason = _('Cluster resizing failed: %s') % reason
-                self.cluster.set_status(self.context, self.cluster.ACTIVE,
-                                        status_reason)
-                return result, reason
-            count, desired, candidates = self._get_action_data(current_size)
-        elif 'deletion' in self.data:
-            grace_period = self.data['deletion'].get('grace_period', None)
-        if candidates is not None and len(candidates) == 0:
-            # Choose victims randomly
-            candidates = scaleutils.nodes_by_random(self.cluster.nodes, count)
-
-        # delete nodes if necessary
-        if desired < current_size:
-            if grace_period is not None:
-                self._wait_before_deletion(grace_period)
-            result, reason = self._delete_nodes(candidates)
-        # Create new nodes if desired_capacity increased
-        else:
-            result, reason = self._create_nodes(count)
-
-        if result != self.RES_OK:
-            self.cluster.set_status(self.context, self.cluster.WARNING, reason)
-            return result, reason
-
-        reason = _('Cluster resize succeeded.')
+    def _update_cluster_size(self, desired):
+        """Private function for updating cluster properties."""
         kwargs = {'desired_capacity': desired}
         min_size = self.inputs.get(consts.ADJUSTMENT_MIN_SIZE, None)
         max_size = self.inputs.get(consts.ADJUSTMENT_MAX_SIZE, None)
@@ -606,59 +643,89 @@ class ClusterAction(base.Action):
             kwargs['min_size'] = min_size
         if max_size is not None:
             kwargs['max_size'] = max_size
-        self.cluster.set_status(self.context, self.cluster.ACTIVE, reason,
-                                **kwargs)
-        return self.RES_OK, reason
+        self.cluster.set_status(self.context, self.cluster.RESIZING,
+                                _('Cluster resize started.'), **kwargs)
+
+    def do_resize(self):
+        """Handler for the CLUSTER_RESIZE action.
+
+        :returns: A tuple containing the result and the corresponding reason.
+        """
+        # if no policy decision(s) found, use policy inputs directly,
+        # Note the 'parse_resize_params' function is capable of calculating
+        # desired capacity and handling best effort scaling. It also verifies
+        # that the inputs are valid
+        curr_capacity = no.Node.count_by_cluster(self.context, self.cluster.id)
+        if 'creation' not in self.data and 'deletion' not in self.data:
+            result, reason = scaleutils.parse_resize_params(self, self.cluster,
+                                                            curr_capacity)
+            if result != self.RES_OK:
+                return result, reason
+
+        # action input consolidated to action data now
+        reason = _('Cluster resize succeeded.')
+        if 'deletion' in self.data:
+            count = self.data['deletion']['count']
+            candidates = self.data['deletion'].get('candidates', [])
+
+            # Choose victims randomly if not already picked
+            if not candidates:
+                node_list = self.cluster.nodes
+                candidates = scaleutils.nodes_by_random(node_list, count)
+
+            self._update_cluster_size(curr_capacity - count)
+
+            grace_period = self.data['deletion'].get('grace_period', 0)
+            self._sleep(grace_period)
+            result, new_reason = self._delete_nodes(candidates)
+        else:
+            # 'creation' in self.data:
+            count = self.data['creation']['count']
+            self._update_cluster_size(curr_capacity + count)
+            result, new_reason = self._create_nodes(count)
+
+        if result != self.RES_OK:
+            reason = new_reason
+
+        self.cluster.eval_status(self.context, self.CLUSTER_RESIZE)
+        return result, reason
 
     def do_scale_out(self):
         """Handler for the CLUSTER_SCALE_OUT action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        self.cluster.set_status(self.context, self.cluster.RESIZING,
-                                'Cluster scale out started.')
         # We use policy output if any, or else the count is
         # set to 1 as default.
         pd = self.data.get('creation', None)
         if pd is not None:
             count = pd.get('count', 1)
         else:
-            # If no scaling policy is attached, use the
-            # input count directly
+            # If no scaling policy is attached, use the input count directly
             count = self.inputs.get('count', 1)
             try:
-                count = utils.parse_int_param('count', count,
-                                              allow_zero=False)
+                count = utils.parse_int_param('count', count, allow_zero=False)
             except exception.InvalidParameter:
                 reason = _('Invalid count (%s) for scaling out.') % count
-                status_reason = _('Cluster scaling failed: %s') % reason
-                self.cluster.set_status(self.context, self.cluster.ACTIVE,
-                                        status_reason)
                 return self.RES_ERROR, reason
 
         # check provided params against current properties
         # desired is checked when strict is True
-        curr_size = len(self.cluster.nodes)
+        curr_size = no.Node.count_by_cluster(self.context, self.target)
         new_size = curr_size + count
-
         result = scaleutils.check_size_params(self.cluster, new_size,
                                               None, None, True)
         if result:
-            status_reason = _('Cluster scaling failed: %s') % result
-            self.cluster.set_status(self.context, self.cluster.ACTIVE,
-                                    status_reason)
             return self.RES_ERROR, result
 
+        self.cluster.set_status(self.context, self.cluster.RESIZING,
+                                _('Cluster scale out started.'),
+                                desired_capacity=new_size)
+
         result, reason = self._create_nodes(count)
         if result == self.RES_OK:
             reason = _('Cluster scaling succeeded.')
-            # TODO(anyone): make update to desired_capacity customizable
-            self.cluster.set_status(self.context, self.cluster.ACTIVE, reason,
-                                    desired_capacity=new_size)
-        elif result in [self.RES_CANCEL, self.RES_TIMEOUT, self.RES_ERROR]:
-            self.cluster.set_status(self.context, self.cluster.ERROR, reason)
-        else:  # RES_RETRY
-            pass
+        self.cluster.eval_status(self.context, self.CLUSTER_SCALE_OUT)
 
         return result, reason
 
@@ -667,13 +734,11 @@ class ClusterAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        self.cluster.set_status(self.context, self.cluster.RESIZING,
-                                'Cluster scale in started.')
         # We use policy data if any, deletion policy and scaling policy might
         # be attached.
         pd = self.data.get('deletion', None)
-        grace_period = None
-        if pd is not None:
+        grace_period = 0
+        if pd:
             grace_period = pd.get('grace_period', 0)
             candidates = pd.get('candidates', [])
             # if scaling policy is attached, get 'count' from action data
@@ -687,48 +752,40 @@ class ClusterAction(base.Action):
                                               allow_zero=False)
             except exception.InvalidParameter:
                 reason = _('Invalid count (%s) for scaling in.') % count
-                status_reason = _('Cluster scaling failed: %s') % reason
-                self.cluster.set_status(self.context, self.cluster.ACTIVE,
-                                        status_reason)
                 return self.RES_ERROR, reason
 
         # check provided params against current properties
         # desired is checked when strict is True
-        curr_size = len(self.cluster.nodes)
+        curr_size = no.Node.count_by_cluster(self.context, self.target)
         if count > curr_size:
-            LOG.warning(_('Triming count (%(count)s) to current cluster size '
-                          '(%(curr)s) for scaling in'),
-                        {'count': count, 'curr': curr_size})
+            msg = _("Triming count (%(count)s) to current "
+                    "cluster size (%(curr)s) for scaling in")
+            LOG.warning(msg, {'count': count, 'curr': curr_size})
             count = curr_size
         new_size = curr_size - count
 
         result = scaleutils.check_size_params(self.cluster, new_size,
                                               None, None, True)
         if result:
-            status_reason = _('Cluster scaling failed: %s') % result
-            self.cluster.set_status(self.context, self.cluster.ACTIVE,
-                                    status_reason)
             return self.RES_ERROR, result
 
+        self.cluster.set_status(self.context, self.cluster.RESIZING,
+                                _('Cluster scale in started.'),
+                                desired_capacity=new_size)
+
         # Choose victims randomly
         if len(candidates) == 0:
             candidates = scaleutils.nodes_by_random(self.cluster.nodes, count)
 
-        if grace_period is not None:
-            self._wait_before_deletion(grace_period)
-        # The policy data may contain destroy flag and grace period option
+        #
+        self._sleep(grace_period)
+
         result, reason = self._delete_nodes(candidates)
 
         if result == self.RES_OK:
             reason = _('Cluster scaling succeeded.')
-            # TODO(anyone): make update to desired capacity customizable
-            self.cluster.set_status(self.context, self.cluster.ACTIVE, reason,
-                                    desired_capacity=new_size)
-        elif result in [self.RES_CANCEL, self.RES_TIMEOUT, self.RES_ERROR]:
-            self.cluster.set_status(self.context, self.cluster.ERROR, reason)
-        else:
-            # RES_RETRY
-            pass
+
+        self.cluster.eval_status(self.context, self.CLUSTER_SCALE_IN)
 
         return result, reason
 
