@@ -10,14 +10,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import eventlet
+
 from senlin.common.i18n import _
-from senlin.common import scaleutils
+from senlin.common import scaleutils as su
 from senlin.engine.actions import base
-from senlin.engine import cluster as cluster_mod
+from senlin.engine import cluster as cm
 from senlin.engine import event as EVENT
 from senlin.engine import node as node_mod
 from senlin.engine import senlin_lock
-from senlin.policies import base as policy_mod
+from senlin.objects import node as no
+from senlin.policies import base as pb
 
 
 class NodeAction(base.Action):
@@ -55,62 +58,69 @@ class NodeAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        if self.node.cluster_id and self.cause == base.CAUSE_RPC:
-            # If node is created with target cluster specified,
-            # check cluster size constraint
-            cluster = cluster_mod.Cluster.load(self.context,
-                                               self.node.cluster_id)
-            result = scaleutils.check_size_params(
-                cluster, cluster.desired_capacity + 1, None, None, True)
-
+        cluster_id = self.node.cluster_id
+        if cluster_id and self.cause == base.CAUSE_RPC:
+            # Check cluster size constraint if target cluster is specified
+            cluster = cm.Cluster.load(self.context, cluster_id)
+            current = no.Node.count_by_cluster(self.context, cluster_id)
+            desired = current + 1
+            result = su.check_size_params(cluster, desired, None, None, True)
             if result:
+                # cannot place node into the cluster
+                no.Node.update(self.context, self.node.id, {'cluster_id': ''})
                 return self.RES_ERROR, result
 
         res = self.node.do_create(self.context)
+
+        if cluster_id and self.cause == base.CAUSE_RPC:
+            # Update cluster's desired_capacity and re-evaluate its status no
+            # matter the creation is a success or not because the node object
+            # is # already treated as member of the cluster and the node
+            # creation may have changed the cluster's status
+            cluster.eval_status(self.context, self.NODE_CREATE,
+                                desired_capacity=desired)
         if res:
-            if self.node.cluster_id and self.cause == base.CAUSE_RPC:
-                # Update cluster desired_capacity if node creation succeeded
-                cluster.desired_capacity += 1
-                cluster.store(self.context)
-                cluster.add_node(self.node)
             return self.RES_OK, _('Node created successfully.')
         else:
             return self.RES_ERROR, _('Node creation failed.')
 
-    def do_check(self):
-        """Handler for the NODE_check action.
-
-        :returns: A tuple containing the result and the corresponding reason.
-        """
-        res = self.node.do_check(self.context)
-        if res:
-            return self.RES_OK, _('Node status is ACTIVE.')
-        else:
-            return self.RES_ERROR, _('Node status is not ACTIVE.')
-
     def do_delete(self):
         """Handler for the NODE_DELETE action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        if self.node.cluster_id and self.cause == base.CAUSE_RPC:
+        cluster_id = self.node.cluster_id
+        if cluster_id and self.cause == base.CAUSE_RPC:
             # If node belongs to a cluster, check size constraint
             # before deleting it
-            cluster = cluster_mod.Cluster.load(self.context,
-                                               self.node.cluster_id)
-            result = scaleutils.check_size_params(cluster,
-                                                  cluster.desired_capacity - 1,
-                                                  None, None, True)
+            cluster = cm.Cluster.load(self.context, cluster_id)
+            current = no.Node.count_by_cluster(self.context, cluster_id)
+            desired = current - 1
+            result = su.check_size_params(cluster, desired, None, None, True)
             if result:
                 return self.RES_ERROR, result
 
+            # handle grace_period
+            pd = self.data.get('deletion', None)
+            if pd:
+                grace_period = pd.get('grace_period', 0)
+                if grace_period:
+                    eventlet.sleep(grace_period)
+
         res = self.node.do_delete(self.context)
+
+        if cluster_id and self.cause == base.CAUSE_RPC:
+            # check if desired_capacity should be changed
+            do_reduce = True
+            params = {}
+            pd = self.data.get('deletion', None)
+            if pd:
+                do_reduce = pd.get('reduce_desired_capacity', True)
+            if do_reduce and res:
+                params = {'desired_capacity': desired}
+            cluster.eval_status(self.context, self.NODE_DELETE, **params)
+
         if res:
-            if self.node.cluster_id and self.cause == base.CAUSE_RPC:
-                # Update cluster desired_capacity if node deletion succeeded
-                cluster.desired_capacity -= 1
-                cluster.store(self.context)
-                cluster.remove_node(self.node.id)
             return self.RES_OK, _('Node deleted successfully.')
         else:
             return self.RES_ERROR, _('Node deletion failed.')
@@ -130,20 +140,23 @@ class NodeAction(base.Action):
     def do_join(self):
         """Handler for the NODE_JOIN action.
 
+        Note that we don't manipulate the cluster's status after this
+        operation. This is because a NODE_JOIN is always an internal action,
+        i.e. derived from a cluster action. The cluster's status is supposed
+        to be checked and set in the outer cluster action rather than here.
+
         :returns: A tuple containing the result and the corresponding reason.
         """
         cluster_id = self.inputs.get('cluster_id')
         # Check the size constraint of parent cluster
-        cluster = cluster_mod.Cluster.load(self.context, cluster_id)
-        new_capacity = cluster.desired_capacity + 1
-        result = scaleutils.check_size_params(cluster, new_capacity,
-                                              None, None, True)
+        cluster = cm.Cluster.load(self.context, cluster_id)
+        current = no.Node.count_by_cluster(self.context, cluster_id)
+        result = su.check_size_params(cluster, current + 1, None, None, True)
         if result:
             return self.RES_ERROR, result
 
         result = self.node.do_join(self.context, cluster_id)
         if result:
-            cluster.add_node(self.node)
             return self.RES_OK, _('Node successfully joined cluster.')
         else:
             return self.RES_ERROR, _('Node failed in joining cluster.')
@@ -151,24 +164,37 @@ class NodeAction(base.Action):
     def do_leave(self):
         """Handler for the NODE_LEAVE action.
 
+        Note that we don't manipulate the cluster's status after this
+        operation. This is because a NODE_JOIN is always an internal action,
+        i.e. derived from a cluster action. The cluster's status is supposed
+        to be checked and set in the outer cluster action rather than here.
+
         :returns: A tuple containing the result and the corresponding reason.
         """
         # Check the size constraint of parent cluster
-        cluster = cluster_mod.Cluster.load(self.context,
-                                           self.node.cluster_id)
-        new_capacity = cluster.desired_capacity - 1
-        result = scaleutils.check_size_params(cluster, new_capacity,
-                                              None, None, True)
+        cluster = cm.Cluster.load(self.context, self.node.cluster_id)
+        current = no.Node.count_by_cluster(self.context, self.node.cluster_id)
+        result = su.check_size_params(cluster, current - 1, None, None, True)
         if result:
             return self.RES_ERROR, result
 
         res = self.node.do_leave(self.context)
         if res:
-            cluster.remove_node(self.node.id)
             return self.RES_OK, _('Node successfully left cluster.')
         else:
             return self.RES_ERROR, _('Node failed in leaving cluster.')
 
+    def do_check(self):
+        """Handler for the NODE_check action.
+
+        :returns: A tuple containing the result and the corresponding reason.
+        """
+        res = self.node.do_check(self.context)
+        if res:
+            return self.RES_OK, _('Node status is ACTIVE.')
+        else:
+            return self.RES_ERROR, _('Node status is not ACTIVE.')
+
     def do_recover(self):
         """Handler for the NODE_RECOVER action.
 
@@ -200,26 +226,22 @@ class NodeAction(base.Action):
         :param dict kwargs: Parameters provided to the action, if any.
         :returns: A tuple containing the result and the related reason.
         """
-        # Since node.cluster_id could be reset to None in _execute progress,
+        # Since node.cluster_id could be reset to '' during action execution,
         # we record it here for policy check and cluster lock release.
         saved_cluster_id = self.node.cluster_id
-        if self.node.cluster_id:
-            if self.cause == base.CAUSE_RPC:
-                res = senlin_lock.cluster_lock_acquire(
-                    self.context,
-                    self.node.cluster_id, self.id, self.owner,
-                    senlin_lock.NODE_SCOPE, False)
-                if not res:
-                    return self.RES_RETRY, _('Failed in locking cluster')
+        if (saved_cluster_id and self.cause == base.CAUSE_RPC):
+            res = senlin_lock.cluster_lock_acquire(
+                self.context, self.node.cluster_id, self.id, self.owner,
+                senlin_lock.NODE_SCOPE, False)
+
+            if not res:
+                return self.RES_RETRY, _('Failed in locking cluster')
 
             self.policy_check(self.node.cluster_id, 'BEFORE')
-            if self.data['status'] != policy_mod.CHECK_OK:
-                # Don't emit message here since policy_check should have
-                # done it
-                if self.cause == base.CAUSE_RPC:
-                    senlin_lock.cluster_lock_release(
-                        self.node.cluster_id, self.id, senlin_lock.NODE_SCOPE)
-
+            if self.data['status'] != pb.CHECK_OK:
+                # Don't emit message since policy_check should have done it
+                senlin_lock.cluster_lock_release(saved_cluster_id, self.id,
+                                                 senlin_lock.NODE_SCOPE)
                 return self.RES_ERROR, 'Policy check: ' + self.data['reason']
 
         reason = ''
@@ -231,16 +253,17 @@ class NodeAction(base.Action):
                 reason = _('Failed in locking node')
             else:
                 res, reason = self._execute()
-                if res == self.RES_OK and saved_cluster_id is not None:
+                if (res == self.RES_OK and saved_cluster_id and
+                        self.cause == base.CAUSE_RPC):
                     self.policy_check(saved_cluster_id, 'AFTER')
-                    if self.data['status'] != policy_mod.CHECK_OK:
+                    if self.data['status'] != pb.CHECK_OK:
                         res = self.RES_ERROR
                         reason = 'Policy check: ' + self.data['reason']
                     else:
                         res = self.RES_OK
         finally:
             senlin_lock.node_lock_release(self.node.id, self.id)
-            if saved_cluster_id is not None and self.cause == base.CAUSE_RPC:
+            if saved_cluster_id and self.cause == base.CAUSE_RPC:
                 senlin_lock.cluster_lock_release(saved_cluster_id, self.id,
                                                  senlin_lock.NODE_SCOPE)
         return res, reason
