@@ -19,11 +19,12 @@ from senlin.common import exception
 from senlin.common.i18n import _
 from senlin.common.i18n import _LE
 from senlin.common import utils
-from senlin.db import api as db_api
-from senlin.engine import cluster_policy as cp_mod
+from senlin.engine import cluster_policy as cpm
 from senlin.engine import node as node_mod
-from senlin.policies import base as policy_base
-from senlin.profiles import base as profile_base
+from senlin.objects import cluster as co
+from senlin.objects import cluster_policy as cpo
+from senlin.policies import base as pcb
+from senlin.profiles import base as pfb
 
 LOG = logging.getLogger(__name__)
 
@@ -35,7 +36,7 @@ class Cluster(object):
 
     All operations are performed without further checking because the
     checkings are supposed to be done before/after/during an action is
-    excuted.
+    executed.
     """
 
     STATUSES = (
@@ -48,7 +49,7 @@ class Cluster(object):
 
     def __init__(self, name, desired_capacity, profile_id,
                  context=None, **kwargs):
-        '''Intialize a cluster object.
+        '''Initialize a cluster object.
 
         The cluster defaults to have 0 node with no profile assigned.
         '''
@@ -79,6 +80,7 @@ class Cluster(object):
         self.status_reason = kwargs.get('status_reason', 'Initializing')
         self.data = kwargs.get('data', {})
         self.metadata = kwargs.get('metadata') or {}
+        self.dependents = kwargs.get('dependents') or {}
 
         # rt is a dict for runtime data
         self.rt = {
@@ -95,16 +97,15 @@ class Cluster(object):
             return
 
         policies = []
-        bindings = db_api.cluster_policy_get_all(context, self.id)
+        bindings = cpo.ClusterPolicy.get_all(context, self.id)
         for b in bindings:
             # Detect policy type conflicts
-            policy = policy_base.Policy.load(context, b.policy_id)
+            policy = pcb.Policy.load(context, b.policy_id)
             policies.append(policy)
 
         self.rt = {
-            'profile': profile_base.Profile.load(context,
-                                                 profile_id=self.profile_id,
-                                                 project_safe=False),
+            'profile': pfb.Profile.load(context, profile_id=self.profile_id,
+                                        project_safe=False),
             'nodes': node_mod.Node.load_all(context, cluster_id=self.id),
             'policies': policies
         }
@@ -133,71 +134,73 @@ class Cluster(object):
             'status_reason': self.status_reason,
             'meta_data': self.metadata,
             'data': self.data,
+            'dependents': self.dependents,
         }
 
-        timestamp = timeutils.utcnow()
+        timestamp = timeutils.utcnow(True)
         if self.id:
             values['updated_at'] = timestamp
-            db_api.cluster_update(context, self.id, values)
+            co.Cluster.update(context, self.id, values)
         else:
             self.init_at = timestamp
             values['init_at'] = timestamp
-            cluster = db_api.cluster_create(context, values)
+            cluster = co.Cluster.create(context, values)
             self.id = cluster.id
 
         self._load_runtime_data(context)
         return self.id
 
     @classmethod
-    def _from_db_record(cls, context, record):
-        '''Construct a cluster object from database record.
+    def _from_object(cls, context, obj):
+        """Construct a cluster from database object.
 
         :param context: the context used for DB operations;
-        :param record: a DB cluster object that will receive all fields;
-        '''
+        :param obj: a DB cluster object that will receive all fields;
+        """
         kwargs = {
-            'id': record.id,
-            'user': record.user,
-            'project': record.project,
-            'domain': record.domain,
-            'init_at': record.init_at,
-            'created_at': record.created_at,
-            'updated_at': record.updated_at,
-            'min_size': record.min_size,
-            'max_size': record.max_size,
-            'next_index': record.next_index,
-            'timeout': record.timeout,
-            'status': record.status,
-            'status_reason': record.status_reason,
-            'data': record.data,
-            'metadata': record.meta_data,
+            'id': obj.id,
+            'user': obj.user,
+            'project': obj.project,
+            'domain': obj.domain,
+            'init_at': obj.init_at,
+            'created_at': obj.created_at,
+            'updated_at': obj.updated_at,
+            'min_size': obj.min_size,
+            'max_size': obj.max_size,
+            'next_index': obj.next_index,
+            'timeout': obj.timeout,
+            'status': obj.status,
+            'status_reason': obj.status_reason,
+            'data': obj.data,
+            'metadata': obj.metadata,
+            'dependents': obj.dependents,
         }
 
-        return cls(record.name, record.desired_capacity, record.profile_id,
+        return cls(obj.name, obj.desired_capacity, obj.profile_id,
                    context=context, **kwargs)
 
     @classmethod
-    def load(cls, context, cluster_id=None, cluster=None, project_safe=True):
+    def load(cls, context, cluster_id=None, dbcluster=None, project_safe=True):
         '''Retrieve a cluster from database.'''
-        if cluster is None:
-            cluster = db_api.cluster_get(context, cluster_id,
-                                         project_safe=project_safe)
-            if cluster is None:
-                raise exception.ClusterNotFound(cluster=cluster_id)
+        if dbcluster is None:
+            dbcluster = co.Cluster.get(context, cluster_id,
+                                       project_safe=project_safe)
+            if dbcluster is None:
+                raise exception.ResourceNotFound(type='cluster', id=cluster_id)
 
-        return cls._from_db_record(context, cluster)
+        return cls._from_object(context, dbcluster)
 
     @classmethod
     def load_all(cls, context, limit=None, marker=None, sort=None,
                  filters=None, project_safe=True):
-        '''Retrieve all clusters from database.'''
+        """Retrieve all clusters from database."""
 
-        records = db_api.cluster_get_all(context, limit=limit, marker=marker,
-                                         sort=sort, filters=filters,
-                                         project_safe=project_safe)
+        objs = co.Cluster.get_all(context, limit=limit, marker=marker,
+                                  sort=sort, filters=filters,
+                                  project_safe=project_safe)
 
-        for record in records:
-            cluster = cls._from_db_record(context, record)
+        for obj in objs:
+            cluster = cls._from_object(context, obj)
             yield cluster
 
     def to_dict(self):
@@ -208,9 +211,9 @@ class Cluster(object):
             'user': self.user,
             'project': self.project,
             'domain': self.domain,
-            'init_at': utils.format_time(self.init_at),
-            'created_at': utils.format_time(self.created_at),
-            'updated_at': utils.format_time(self.updated_at),
+            'init_at': utils.isotime(self.init_at),
+            'created_at': utils.isotime(self.created_at),
+            'updated_at': utils.isotime(self.updated_at),
             'min_size': self.min_size,
             'max_size': self.max_size,
             'desired_capacity': self.desired_capacity,
@@ -219,6 +222,7 @@ class Cluster(object):
             'status_reason': self.status_reason,
             'metadata': self.metadata,
             'data': self.data,
+            'dependents': self.dependents,
             'nodes': [node.id for node in self.rt['nodes']],
             'policies': [policy.id for policy in self.rt['policies']],
         }
@@ -241,7 +245,7 @@ class Cluster(object):
         """
 
         values = {}
-        now = timeutils.utcnow()
+        now = timeutils.utcnow(True)
         if status == self.ACTIVE and self.status == self.CREATING:
             self.created_at = now
             values['created_at'] = now
@@ -263,12 +267,21 @@ class Cluster(object):
 
         # There is a possibility that the profile id is changed
         if 'profile_id' in values:
-            profile = profile_base.Profile.load(context,
-                                                profile_id=self.profile_id)
+            profile = pfb.Profile.load(context, profile_id=self.profile_id)
             self.rt['profile'] = profile
-        db_api.cluster_update(context, self.id, values)
+        co.Cluster.update(context, self.id, values)
         return
 
+    def update_dependents(self, context, dependents):
+        """Update dependency information of cluster's property.
+
+        :param context: An instance of request context.
+        :param dependents: The dependency information.
+        """
+
+        values = {'dependents': dependents}
+        co.Cluster.update(context, self.id, values)
+
     def do_create(self, context, **kwargs):
         '''Additional logic at the beginning of cluster creation process.
 
@@ -278,13 +291,13 @@ class Cluster(object):
             LOG.error(_LE('Cluster is in status "%s"'), self.status)
             return False
 
-        self.set_status(context, self.CREATING, reason='Creation in progress')
+        self.set_status(context, self.CREATING, _('Creation in progress'))
         return True
 
     def do_delete(self, context, **kwargs):
         """Additional logic at the end of cluster deletion process."""
 
-        db_api.cluster_delete(context, self.id)
+        co.Cluster.delete(context, self.id)
         return True
 
     def do_update(self, context, **kwargs):
@@ -292,15 +305,15 @@ class Cluster(object):
 
         This method is intended to be called only from an action.
         '''
-        self.set_status(context, self.UPDATING, reason='Update in progress')
+        self.set_status(context, self.UPDATING, _('Update in progress'))
         return True
 
     def do_check(self, context, **kwargs):
-        '''Additional logic at the beginning of cluster checking process.
+        """Additional logic at the beginning of cluster checking process.
 
         Set cluster status to CHECKING.
-        '''
-        self.set_status(context, self.CHECKING, reason=_('Check in progress'))
+        """
+        self.set_status(context, self.CHECKING, _('Check in progress'))
         return True
 
     def do_recover(self, context, **kwargs):
@@ -308,8 +321,7 @@ class Cluster(object):
 
         Set cluster status to RECOVERING.
         '''
-        self.set_status(context, self.RECOVERING,
-                        reason=_('Recovery in progress'))
+        self.set_status(context, self.RECOVERING, _('Recovery in progress'))
         return True
 
     def attach_policy(self, ctx, policy_id, values):
@@ -324,7 +336,7 @@ class Cluster(object):
         :returns: A tuple containing a boolean result and a reason string.
         """
 
-        policy = policy_base.Policy.load(ctx, policy_id)
+        policy = pcb.Policy.load(ctx, policy_id)
         # Check if policy has already been attached
         for existing in self.rt['policies']:
             # Policy already attached
@@ -353,7 +365,7 @@ class Cluster(object):
             'priority': policy.PRIORITY
         }
 
-        cp = cp_mod.ClusterPolicy(self.id, policy_id, **kwargs)
+        cp = cpm.ClusterPolicy(self.id, policy_id, **kwargs)
         cp.store(ctx)
 
         # refresh cached runtime
@@ -386,7 +398,7 @@ class Cluster(object):
 
         params = {'enabled': bool(enabled)}
 
-        db_api.cluster_policy_update(ctx, self.id, policy_id, params)
+        cpo.ClusterPolicy.update(ctx, self.id, policy_id, params)
         return True, _('Policy updated.')
 
     def detach_policy(self, ctx, policy_id):
@@ -408,12 +420,12 @@ class Cluster(object):
         if found is None:
             return False, _('Policy not attached.')
 
-        policy = policy_base.Policy.load(ctx, policy_id)
+        policy = pcb.Policy.load(ctx, policy_id)
         res, reason = policy.detach(self)
         if not res:
             return res, reason
 
-        db_api.cluster_policy_detach(ctx, self.id, policy_id)
+        cpo.ClusterPolicy.delete(ctx, self.id, policy_id)
         self.rt['policies'].remove(found)
 
         return True, _('Policy detached.')
@@ -511,3 +523,47 @@ class Cluster(object):
                 if zone == placement['zone']:
                     result.append(node)
         return result
+
+    def eval_status(self, ctx, operation, **params):
+        """Re-evaluate cluster's health status.
+
+        :param ctx: The requesting context.
+        :param operation: The operation that triggers this status evaluation.
+        :returns: ``None``.
+        """
+        self.rt['nodes'] = node_mod.Node.load_all(ctx, cluster_id=self.id)
+        active_count = 0
+        for node in self.nodes:
+            if node.status == 'ACTIVE':
+                active_count += 1
+
+        # get provided desired_capacity/min_size/max_size
+        desired = params.get('desired_capacity', None)
+        desired = desired or self.desired_capacity
+        min_size = params.get('min_size', None)
+        min_size = min_size or self.min_size
+        max_size = params.get('max_size', None)
+        max_size = max_size or self.max_size
+
+        values = params or {}
+        if active_count < min_size:
+            status = self.ERROR
+            reason = _("%(o)s: number of active nodes is below min_size "
+                       "(%(n)d).") % {'o': operation, 'n': min_size}
+        elif active_count < desired:
+            status = self.WARNING
+            reason = _("%(o)s: number of active nodes is below "
+                       "desired_capacity "
+                       "(%(n)d).") % {'o': operation, 'n': desired}
+        elif max_size < 0 or active_count <= max_size:
+            status = self.ACTIVE
+            reason = _("%(o)s: number of active nodes is above "
+                       "desired_capacity "
+                       "(%(n)d).") % {'o': operation, 'n': desired}
+        else:
+            status = self.WARNING
+            reason = _("%(o)s: number of active nodes is above max_size "
+                       "(%(n)d).") % {'o': operation, 'n': max_size}
+
+        values.update({'status': status, 'status_reason': reason})
+        co.Cluster.update(ctx, self.id, values)
