@@ -10,28 +10,27 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import six
-
 from oslo_log import log as logging
 from oslo_utils import timeutils
+import six
 
-from senlin.common import exception
+from senlin.common import exception as exc
 from senlin.common.i18n import _
 from senlin.common.i18n import _LE
 from senlin.common import utils
-from senlin.db import api as db_api
-from senlin.profiles import base as profile_base
+from senlin.objects import node as no
+from senlin.profiles import base as pb
 
 LOG = logging.getLogger(__name__)
 
 
 class Node(object):
-    '''A node is an object that can belong to at most one single cluster.
+    """A node is an object that can belong to at most one single cluster.
 
     All operations are performed without further checking because the
     checkings are supposed to be done before/after/during an action is
-    excuted.
-    '''
+    executed.
+    """
 
     statuses = (
         INIT, ACTIVE, ERROR, WARNING, CREATING, UPDATING, DELETING,
@@ -54,7 +53,7 @@ class Node(object):
         if cluster_id is None:
             cluster_id = ''
 
-        self.physical_id = kwargs.get('physical_id', '')
+        self.physical_id = kwargs.get('physical_id', None)
         self.profile_id = profile_id
         self.user = kwargs.get('user', '')
         self.project = kwargs.get('project', '')
@@ -71,6 +70,7 @@ class Node(object):
         self.status_reason = kwargs.get('status_reason', 'Initializing')
         self.data = kwargs.get('data', {})
         self.metadata = kwargs.get('metadata', {})
+        self.dependents = kwargs.get('dependents', {})
         self.rt = {}
 
         if context is not None:
@@ -85,21 +85,22 @@ class Node(object):
     def _load_runtime_data(self, context):
         profile = None
         try:
-            profile = profile_base.Profile.load(context,
-                                                profile_id=self.profile_id,
-                                                project_safe=False)
-        except exception.ProfileNotFound:
+            profile = pb.Profile.load(context, profile_id=self.profile_id,
+                                      project_safe=False)
+        except exc.ResourceNotFound:
             LOG.debug(_('Profile not found: %s'), self.profile_id)
 
         self.rt = {'profile': profile}
 
     def store(self, context):
-        '''Store the node record into database table.
+        """Store the node into database table.
 
-        The invocation of DB API could be a node_create or a node_update,
+        The invocation of object API could be a node_create or a node_update,
         depending on whether node has an ID assigned.
-        '''
 
+        @param context: Request context for node creation.
+        @return: UUID of node created.
+        """
         values = {
             'name': self.name,
             'physical_id': self.physical_id,
@@ -117,67 +118,69 @@ class Node(object):
             'status_reason': self.status_reason,
             'meta_data': self.metadata,
             'data': self.data,
+            'dependents': self.dependents,
         }
 
         if self.id:
-            db_api.node_update(context, self.id, values)
+            no.Node.update(context, self.id, values)
         else:
-            init_at = timeutils.utcnow()
+            init_at = timeutils.utcnow(True)
             self.init_at = init_at
             values['init_at'] = init_at
-            node = db_api.node_create(context, values)
+            node = no.Node.create(context, values)
             self.id = node.id
 
         self._load_runtime_data(context)
         return self.id
 
     @classmethod
-    def _from_db_record(cls, context, record):
-        '''Construct a node object from database record.
+    def _from_object(cls, context, obj):
+        """Construct a node from node object.
 
-        :param context: the context used for DB operations;
-        :param record: a DB node object that contains all fields;
-        '''
+        @param context: the context used for DB operations;
+        @param node: a node object that contains all fields;
+        """
         kwargs = {
-            'id': record.id,
-            'physical_id': record.physical_id,
-            'user': record.user,
-            'project': record.project,
-            'domain': record.domain,
-            'index': record.index,
-            'role': record.role,
-            'init_at': record.init_at,
-            'created_at': record.created_at,
-            'updated_at': record.updated_at,
-            'status': record.status,
-            'status_reason': record.status_reason,
-            'data': record.data,
-            'metadata': record.meta_data,
+            'id': obj.id,
+            'physical_id': obj.physical_id,
+            'user': obj.user,
+            'project': obj.project,
+            'domain': obj.domain,
+            'index': obj.index,
+            'role': obj.role,
+            'init_at': obj.init_at,
+            'created_at': obj.created_at,
+            'updated_at': obj.updated_at,
+            'status': obj.status,
+            'status_reason': obj.status_reason,
+            'data': obj.data,
+            'metadata': obj.metadata,
+            'dependents': obj.dependents,
         }
 
-        return cls(record.name, record.profile_id, record.cluster_id,
+        return cls(obj.name, obj.profile_id, obj.cluster_id,
                    context=context, **kwargs)
 
     @classmethod
-    def load(cls, context, node_id=None, node=None, project_safe=True):
+    def load(cls, context, node_id=None, db_node=None, project_safe=True):
         '''Retrieve a node from database.'''
-        if node is None:
-            node = db_api.node_get(context, node_id, project_safe=project_safe)
-            if node is None:
-                raise exception.NodeNotFound(node=node_id)
+        if db_node is None:
+            db_node = no.Node.get(context, node_id, project_safe=project_safe)
+            if db_node is None:
+                raise exc.ResourceNotFound(type='node', id=node_id)
 
-        return cls._from_db_record(context, node)
+        return cls._from_object(context, db_node)
 
     @classmethod
     def load_all(cls, context, cluster_id=None, limit=None, marker=None,
                  sort=None, filters=None, project_safe=True):
         '''Retrieve all nodes of from database.'''
-        records = db_api.node_get_all(context, cluster_id=cluster_id,
-                                      filters=filters, sort=sort,
-                                      limit=limit, marker=marker,
-                                      project_safe=project_safe)
+        objs = no.Node.get_all(context, cluster_id=cluster_id,
+                               filters=filters, sort=sort,
+                               limit=limit, marker=marker,
+                               project_safe=project_safe)
 
-        return [cls._from_db_record(context, record) for record in records]
+        return [cls._from_object(context, obj) for obj in objs]
 
     def to_dict(self):
         if self.rt['profile']:
@@ -195,25 +198,33 @@ class Node(object):
             'domain': self.domain,
             'index': self.index,
             'role': self.role,
-            'init_at': utils.format_time(self.init_at),
-            'created_at': utils.format_time(self.created_at),
-            'updated_at': utils.format_time(self.updated_at),
+            'init_at': utils.isotime(self.init_at),
+            'created_at': utils.isotime(self.created_at),
+            'updated_at': utils.isotime(self.updated_at),
             'status': self.status,
             'status_reason': self.status_reason,
             'data': self.data,
             'metadata': self.metadata,
+            'dependents': self.dependents,
             'profile_name': profile_name,
         }
         return node_dict
 
-    def set_status(self, context, status, reason=None):
-        '''Set status of the node.'''
+    def set_status(self, context, status, reason=None, **params):
+        """Set status of the node.
 
+        :param context: The request context.
+        :param status: New status for the node.
+        :param reason: The reason that leads the node to its current status.
+        :param kwargs params: Other properties that need an update.
+        :returns: ``None``.
+        """
         values = {}
-        now = timeutils.utcnow()
+        now = timeutils.utcnow(True)
         if status == self.ACTIVE and self.status == self.CREATING:
             self.created_at = values['created_at'] = now
         elif status == self.ACTIVE and self.status == self.UPDATING:
+            # TODO(Qiming): update_at should be changed unconditionally
             self.updated_at = values['updated_at'] = now
 
         self.status = status
@@ -221,66 +232,57 @@ class Node(object):
         if reason:
             self.status_reason = reason
             values['status_reason'] = reason
-        db_api.node_update(context, self.id, values)
+        for p, v in params.items():
+            setattr(self, p, v)
+            values[p] = v
+        no.Node.update(context, self.id, values)
 
     def get_details(self, context):
         if not self.physical_id:
             return {}
-        return profile_base.Profile.get_details(context, self)
+        return pb.Profile.get_details(context, self)
 
-    def _handle_exception(self, context, action, status, exception):
-        msg = six.text_type(exception)
-        self.physical_id = exception.kwargs.get('resource_id', None)
-        if self.physical_id:
-            reason = _('Profile failed in %(action)s resource (%(id)s) due '
-                       'to: %(msg)s') % {'action': action[:-1] + 'ing',
-                                         'id': self.physical_id, 'msg': msg}
-        else:
-            # Exception happens before physical node creatin started.
-            reason = _('Profile failed in creating node due to: %(msg)s') % {
-                'msg': msg}
-        self.set_status(context, self.ERROR, reason)
-        self.store(context)
+    def update_dependents(self, context, dependents):
+        """Update dependency information of node's property.
+
+        :param context: An instance of request context.
+        :param dependents: The dependency information.
+        """
+
+        values = {'dependents': dependents}
+        no.Node.update(context, self.id, values)
 
     def do_create(self, context):
         if self.status != self.INIT:
             LOG.error(_LE('Node is in status "%s"'), self.status)
             return False
-        self.set_status(context, self.CREATING, reason='Creation in progress')
+
+        self.set_status(context, self.CREATING, _('Creation in progress'))
         try:
-            physical_id = profile_base.Profile.create_object(context, self)
-        except exception.InternalError as ex:
-            LOG.exception(_('Failed in creating server: %s'),
-                          six.text_type(ex))
-            self._handle_exception(context, 'create', self.ERROR, ex)
-            return False
-        if not physical_id:
+            physical_id = pb.Profile.create_object(context, self)
+        except exc.EResourceCreation as ex:
+            self.set_status(context, self.ERROR, six.text_type(ex))
             return False
 
-        status_reason = 'Creation succeeded'
-        self.set_status(context, self.ACTIVE, status_reason)
-        self.physical_id = physical_id
-        self.store(context)
+        self.set_status(context, self.ACTIVE, _('Creation succeeded'),
+                        physical_id=physical_id)
         return True
 
     def do_delete(self, context):
         if not self.physical_id:
-            db_api.node_delete(context, self.id)
+            no.Node.delete(context, self.id)
             return True
 
         # TODO(Qiming): check if actions are working on it and can be canceled
-        self.set_status(context, self.DELETING, reason='Deletion in progress')
+        self.set_status(context, self.DELETING, 'Deletion in progress')
         try:
-            res = profile_base.Profile.delete_object(context, self)
-        except exception.ResourceStatusError as ex:
-            self._handle_exception(context, 'delete', self.ERROR, ex)
-            res = False
-
-        if res:
-            db_api.node_delete(context, self.id)
+            # The following operation always return True unless exception
+            # is thrown
+            pb.Profile.delete_object(context, self)
+            no.Node.delete(context, self.id)
             return True
-        else:
-            self.set_status(context, self.ERROR, reason='Deletion failed')
+        except exc.EResourceDeletion as ex:
+            self.set_status(context, self.ERROR, six.text_type(ex))
             return False
 
     def do_update(self, context, params):
@@ -293,58 +295,63 @@ class Node(object):
         if not self.physical_id:
             return False
 
-        self.set_status(context, self.UPDATING,
-                        reason='Update in progress')
+        self.set_status(context, self.UPDATING, 'Update in progress')
 
         new_profile_id = params.pop('new_profile_id', None)
         res = True
         if new_profile_id:
             try:
-                res = profile_base.Profile.update_object(
-                    context, self, new_profile_id, **params)
-            except exception.ResourceStatusError as ex:
-                self._handle_exception(context, 'update', self.ERROR, ex)
-                res = False
+                res = pb.Profile.update_object(context, self, new_profile_id,
+                                               **params)
+            except exc.EResourceUpdate as ex:
+                self.set_status(context, self.ERROR, six.text_type(ex))
+                return False
 
-        if res:
-            if 'name' in params:
-                self.name = params['name']
-            if 'role' in params:
-                self.role = params['role']
-            if 'metadata' in params:
-                self.metadata = params['metadata']
+        # update was not successful
+        if not res:
+            return False
 
-            if new_profile_id:
-                self.profile_id = new_profile_id
-            self.store(context)
+        props = dict([(k, v) for k, v in params.items()
+                      if k in ('name', 'role', 'metadata')])
+        if new_profile_id:
+            props['profile_id'] = new_profile_id
+            self.rt['profile'] = pb.Profile.load(context,
+                                                 profile_id=new_profile_id)
 
-            self.set_status(context, self.ACTIVE, reason='Update succeeded')
+        self.set_status(context, self.ACTIVE, 'Update succeeded', **props)
 
-        return res
+        return True
 
     def do_join(self, context, cluster_id):
         if self.cluster_id == cluster_id:
             return True
-        res = profile_base.Profile.join_cluster(context, self, cluster_id)
+
+        timestamp = timeutils.utcnow(True)
+        db_node = no.Node.migrate(context, self.id, cluster_id, timestamp)
+        self.cluster_id = cluster_id
+        self.updated_at = timestamp
+        self.index = db_node.index
+
+        res = pb.Profile.join_cluster(context, self, cluster_id)
         if res:
-            timestamp = timeutils.utcnow()
-            db_node = db_api.node_migrate(context, self.id, cluster_id,
-                                          timestamp)
-            self.cluster_id = cluster_id
-            self.updated_at = timestamp
-            self.index = db_node.index
             return True
-        else:
-            return False
+
+        # rollback
+        db_node = no.Node.migrate(context, self.id, None, timestamp)
+        self.cluster_id = ''
+        self.updated_at = timestamp
+        self.index = -1
+
+        return False
 
     def do_leave(self, context):
         if self.cluster_id == '':
             return True
 
-        res = profile_base.Profile.leave_cluster(context, self)
+        res = pb.Profile.leave_cluster(context, self)
         if res:
-            timestamp = timeutils.utcnow()
-            db_api.node_migrate(context, self.id, None, timestamp)
+            timestamp = timeutils.utcnow(True)
+            no.Node.migrate(context, self.id, None, timestamp)
             self.cluster_id = ''
             self.updated_at = timestamp
             self.index = -1
@@ -356,12 +363,19 @@ class Node(object):
         if not self.physical_id:
             return False
 
-        res = profile_base.Profile.check_object(context, self)
+        res = True
+        try:
+            res = pb.Profile.check_object(context, self)
+        except exc.EResourceOperation as ex:
+            self.set_status(context, self.ERROR, six.text_type(ex))
+            return False
 
-        if not res:
-            self.status = 'ERROR'
-            self.status_reason = _('Physical node is not ACTIVE!')
-            self.store(context)
+        if res:
+            self.set_status(context, self.ACTIVE,
+                            _("Check: Node is ACTIVE."))
+        else:
+            self.set_status(context, self.ERROR,
+                            _("Check: Node is not ACTIVE."))
 
         return res
 
@@ -377,19 +391,19 @@ class Node(object):
                         reason=_('Recover in progress'))
 
         try:
-            physical_id = profile_base.Profile.recover_object(context, self,
-                                                              **options)
-        except exception.ResourceStatusError as ex:
-            self._handle_exception(context, 'recover', self.ERROR, ex)
+            physical_id = pb.Profile.recover_object(context, self, **options)
+        except exc.EResourceOperation as ex:
+            self.set_status(context, self.ERROR, reason=six.text_type(ex))
             return False
 
         if not physical_id:
             self.set_status(context, self.ERROR, reason=_('Recover failed'))
             return False
 
-        self.set_status(context, self.ACTIVE, reason=_('Recover succeeded'))
+        params = {}
         if self.physical_id != physical_id:
-            self.physical_id = physical_id
-            self.store(context)
+            params['physical_id'] = physical_id
+        self.set_status(context, self.ACTIVE, reason=_('Recover succeeded'),
+                        **params)
 
         return True
