@@ -12,14 +12,14 @@
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import timeutils
 import time
 
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
-from senlin.common.i18n import _LI
-from senlin.db import api as db_api
+from senlin.common.i18n import _, _LE, _LI
+from senlin.common import utils
 from senlin.engine import scheduler
+from senlin.objects import action as ao
+from senlin.objects import cluster_lock as cl_obj
+from senlin.objects import node_lock as nl_obj
 
 CONF = cfg.CONF
 
@@ -35,19 +35,6 @@ LOCK_SCOPES = (
 )
 
 
-def is_engine_dead(ctx, engine_id, period_time=None):
-    # if engine didn't report its status for peirod_time, will consider it
-    # as a dead engine.
-    if period_time is None:
-        period_time = 2 * CONF.periodic_interval
-    eng = db_api.service_get(ctx, engine_id)
-    if not eng:
-        return True
-    if (timeutils.utcnow() - eng.updated_at).total_seconds() > period_time:
-        return True
-    return False
-
-
 def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
                          scope=CLUSTER_SCOPE, forced=False):
     """Try to lock the specified cluster.
@@ -64,7 +51,7 @@ def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
 
     # Step 1: try lock the cluster - if the returned owner_id is the
     #         action id, it was a success
-    owners = db_api.cluster_lock_acquire(cluster_id, action_id, scope)
+    owners = cl_obj.ClusterLock.acquire(cluster_id, action_id, scope)
     if action_id in owners:
         return True
 
@@ -75,29 +62,28 @@ def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
     while retries > 0:
         scheduler.sleep(retry_interval)
         LOG.debug('Acquire lock for cluster %s again' % cluster_id)
-        owners = db_api.cluster_lock_acquire(cluster_id, action_id, scope)
+        owners = cl_obj.ClusterLock.acquire(cluster_id, action_id, scope)
         if action_id in owners:
             return True
         retries = retries - 1
 
     # Step 3: Last resort is 'forced locking', only needed when retry failed
     if forced:
-        owners = db_api.cluster_lock_steal(cluster_id, action_id)
+        owners = cl_obj.ClusterLock.steal(cluster_id, action_id)
         return action_id in owners
 
     # Will reach here only because scope == CLUSTER_SCOPE
-    action = db_api.action_get(context, owners[0])
+    action = ao.Action.get(context, owners[0])
     if (action and action.owner and action.owner != engine and
-            is_engine_dead(context, action.owner)):
+            utils.is_engine_dead(context, action.owner)):
         LOG.info(_LI('The cluster %(c)s is locked by dead action %(a)s, '
                      'try to steal the lock.'), {
             'c': cluster_id,
             'a': owners[0]
         })
         reason = _('Engine died when executing this action.')
-        db_api.action_mark_failed(context, action.id, time.time(),
-                                  reason=reason)
-        owners = db_api.cluster_lock_steal(cluster_id, action_id)
+        ao.Action.mark_failed(context, action.id, time.time(), reason)
+        owners = cl_obj.ClusterLock.steal(cluster_id, action_id)
         return action_id in owners
 
     LOG.error(_LE('Cluster is already locked by action %(old)s, '
@@ -114,7 +100,7 @@ def cluster_lock_release(cluster_id, action_id, scope):
     :param action_id: ID of the action that attempts to release the node.
     :param scope: The scope of the lock to be released.
     """
-    return db_api.cluster_lock_release(cluster_id, action_id, scope)
+    return cl_obj.ClusterLock.release(cluster_id, action_id, scope)
 
 
 def node_lock_acquire(context, node_id, action_id, engine=None,
@@ -131,7 +117,7 @@ def node_lock_acquire(context, node_id, action_id, engine=None,
     """
     # Step 1: try lock the node - if the returned owner_id is the
     #         action id, it was a success
-    owner = db_api.node_lock_acquire(node_id, action_id)
+    owner = nl_obj.NodeLock.acquire(node_id, action_id)
     if action_id == owner:
         return True
 
@@ -142,29 +128,28 @@ def node_lock_acquire(context, node_id, action_id, engine=None,
     while retries > 0:
         scheduler.sleep(retry_interval)
         LOG.debug('Acquire lock for node %s again' % node_id)
-        owner = db_api.node_lock_acquire(node_id, action_id)
+        owner = nl_obj.NodeLock.acquire(node_id, action_id)
         if action_id == owner:
             return True
         retries = retries - 1
 
     # Step 3: Last resort is 'forced locking', only needed when retry failed
     if forced:
-        owner = db_api.node_lock_steal(node_id, action_id)
+        owner = nl_obj.NodeLock.steal(node_id, action_id)
         return action_id == owner
 
     # if this node lock by dead engine
-    action = db_api.action_get(context, owner)
+    action = ao.Action.get(context, owner)
     if (action and action.owner and action.owner != engine and
-            is_engine_dead(context, action.owner)):
+            utils.is_engine_dead(context, action.owner)):
         LOG.info(_LI('The node %(n)s is locked by dead action %(a)s, '
                      'try to steal the lock.'), {
             'n': node_id,
             'a': owner
         })
         reason = _('Engine died when executing this action.')
-        db_api.action_mark_failed(context, action.id, time.time(),
-                                  reason=reason)
-        db_api.node_lock_steal(node_id, action_id)
+        ao.Action.mark_failed(context, action.id, time.time(), reason)
+        nl_obj.NodeLock.steal(node_id, action_id)
         return True
 
     LOG.error(_LE('Node is already locked by action %(old)s, '
@@ -180,4 +165,4 @@ def node_lock_release(node_id, action_id):
     :param node_id: ID of the node to be released.
     :param action_id: ID of the action that attempts to release the node.
     """
-    return db_api.node_lock_release(node_id, action_id)
+    return nl_obj.NodeLock.release(node_id, action_id)
