@@ -19,8 +19,10 @@ from senlin.common import exception
 from senlin.common.i18n import _
 from senlin.common import schema
 from senlin.common import utils
-from senlin.db import api as db_api
+from senlin.drivers import base as driver
 from senlin.engine import environment
+from senlin.objects import credential as co
+from senlin.objects import policy as po
 
 CHECK_RESULTS = (
     CHECK_OK, CHECK_ERROR,
@@ -103,26 +105,37 @@ class Policy(object):
         self.updated_at = kwargs.get('updated_at', None)
 
         self.spec_data = schema.Spec(self.spec_schema, spec)
-        self.properties = schema.Spec(self.properties_schema,
-                                      self.spec.get(self.PROPERTIES, {}))
+        self.properties = schema.Spec(
+            self.properties_schema,
+            self.spec.get(self.PROPERTIES, {}),
+            version)
+
         self.singleton = True
+        self._novaclient = None
+        self._keystoneclient = None
+        self._networkclient = None
+        self._lbaasclient = None
 
     @classmethod
-    def _from_db_record(cls, record):
-        '''Construct a policy object from a database record.'''
+    def _from_object(cls, policy):
+        """Construct a policy from a Policy object.
+
+        @param cls: The target class.
+        @param policy: A policy object.
+        """
 
         kwargs = {
-            'id': record.id,
-            'type': record.type,
-            'user': record.user,
-            'project': record.project,
-            'domain': record.domain,
-            'created_at': record.created_at,
-            'updated_at': record.updated_at,
-            'data': record.data,
+            'id': policy.id,
+            'type': policy.type,
+            'user': policy.user,
+            'project': policy.project,
+            'domain': policy.domain,
+            'created_at': policy.created_at,
+            'updated_at': policy.updated_at,
+            'data': policy.data,
         }
 
-        return cls(record.name, record.spec, **kwargs)
+        return cls(policy.name, policy.spec, **kwargs)
 
     @classmethod
     def load(cls, context, policy_id=None, db_policy=None, project_safe=True):
@@ -137,32 +150,32 @@ class Policy(object):
         :returns: An object of the proper policy class.
         """
         if db_policy is None:
-            db_policy = db_api.policy_get(context, policy_id,
-                                          project_safe=project_safe)
+            db_policy = po.Policy.get(context, policy_id,
+                                      project_safe=project_safe)
             if db_policy is None:
-                raise exception.PolicyNotFound(policy=policy_id)
+                raise exception.ResourceNotFound(type='policy', id=policy_id)
 
-        return cls._from_db_record(db_policy)
+        return cls._from_object(db_policy)
 
     @classmethod
     def load_all(cls, context, limit=None, marker=None, sort=None,
                  filters=None, project_safe=True):
         """Retrieve all policies from database."""
 
-        records = db_api.policy_get_all(context, limit=limit, marker=marker,
-                                        sort=sort, filters=filters,
-                                        project_safe=project_safe)
+        objs = po.Policy.get_all(context, limit=limit, marker=marker,
+                                 sort=sort, filters=filters,
+                                 project_safe=project_safe)
 
-        for record in records:
-            yield cls._from_db_record(record)
+        for obj in objs:
+            yield cls._from_object(obj)
 
     @classmethod
     def delete(cls, context, policy_id):
-        db_api.policy_delete(context, policy_id)
+        po.Policy.delete(context, policy_id)
 
     def store(self, context):
         '''Store the policy object into database table.'''
-        timestamp = timeutils.utcnow()
+        timestamp = timeutils.utcnow(True)
 
         values = {
             'name': self.name,
@@ -177,16 +190,16 @@ class Policy(object):
         if self.id is not None:
             self.updated_at = timestamp
             values['updated_at'] = timestamp
-            db_api.policy_update(context, self.id, values)
+            po.Policy.update(context, self.id, values)
         else:
             self.created_at = timestamp
             values['created_at'] = timestamp
-            policy = db_api.policy_create(context, values)
+            policy = po.Policy.create(context, values)
             self.id = policy.id
 
         return self.id
 
-    def validate(self):
+    def validate(self, context, validate_props=False):
         '''Validate the schema and the data provided.'''
         self.spec_data.validate()
         self.properties.validate()
@@ -217,6 +230,83 @@ class Policy(object):
 
         return data.get('data', None)
 
+    def _build_conn_params(self, user, project):
+        """Build trust-based connection parameters.
+
+        :param user: the user for which the trust will be checked.
+        :param object: the user for which the trust will be checked.
+        """
+        service_creds = senlin_context.get_service_context()
+        params = {
+            'username': service_creds.get('username'),
+            'password': service_creds.get('password'),
+            'auth_url': service_creds.get('auth_url'),
+            'user_domain_name': service_creds.get('user_domain_name')
+        }
+
+        cred = co.Credential.get(oslo_context.get_current(), user, project)
+        if cred is None:
+            raise exception.TrustNotFound(trustor=user)
+        params['trust_id'] = cred.cred['openstack']['trust']
+
+        return params
+
+    def keystone(self, user, project):
+        """Construct keystone client based on object.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the keystone client.
+        """
+        if self._keystoneclient is not None:
+            return self._keystoneclient
+        params = self._build_conn_params(user, project)
+        self._keystoneclient = driver.SenlinDriver().identity(params)
+        return self._keystoneclient
+
+    def nova(self, user, project):
+        """Construct nova client based on user and project.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the nova client.
+        """
+        if self._novaclient is not None:
+            return self._novaclient
+
+        params = self._build_conn_params(user, project)
+        self._novaclient = driver.SenlinDriver().compute(params)
+        return self._novaclient
+
+    def network(self, user, project):
+        """Construct network client based on user and project.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the network client.
+        """
+        if self._networkclient is not None:
+            return self._networkclient
+
+        params = self._build_conn_params(user, project)
+        self._networkclient = driver.SenlinDriver().network(params)
+        return self._networkclient
+
+    def lbaas(self, user, project):
+        """Construct LB service client based on user and project.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the LB service client.
+        """
+        if self._lbaasclient is not None:
+            return self._lbaasclient
+
+        params = self._build_conn_params(user, project)
+
+        self._lbaasclient = driver.SenlinDriver().loadbalancing(params)
+        return self._lbaasclient
+
     def attach(self, cluster):
         '''Method to be invoked before policy is attached to a cluster.
 
@@ -265,29 +355,8 @@ class Policy(object):
             'project': self.project,
             'domain': self.domain,
             'spec': self.spec,
-            'created_at': utils.format_time(self.created_at),
-            'updated_at': utils.format_time(self.updated_at),
+            'created_at': utils.isotime(self.created_at),
+            'updated_at': utils.isotime(self.updated_at),
             'data': self.data,
         }
         return pb_dict
-
-    def _build_conn_params(self, cluster):
-        """Build trust-based connection parameters.
-
-        :param cluster: the cluste for which the trust will be checked.
-        """
-        service_creds = senlin_context.get_service_context()
-        params = {
-            'username': service_creds.get('username'),
-            'password': service_creds.get('password'),
-            'auth_url': service_creds.get('auth_url'),
-            'user_domain_name': service_creds.get('user_domain_name')
-        }
-
-        cred = db_api.cred_get(oslo_context.get_current(),
-                               cluster.user, cluster.project)
-        if cred is None:
-            raise exception.TrustNotFound(trustor=cluster.user)
-        params['trust_id'] = cred.cred['openstack']['trust']
-
-        return params
