@@ -23,8 +23,9 @@ from senlin.common import consts
 from senlin.common.i18n import _
 from senlin.common import scaleutils
 from senlin.common import schema
-from senlin.db import api as db_api
-from senlin.engine import cluster as cluster_mod
+from senlin.engine import cluster as cm
+from senlin.objects import cluster as co
+from senlin.objects import node as no
 from senlin.policies import base
 
 LOG = logging.getLogger(__name__)
@@ -60,6 +61,7 @@ class DeletionPolicy(base.Policy):
         ('BEFORE', consts.CLUSTER_SCALE_IN),
         ('BEFORE', consts.CLUSTER_DEL_NODES),
         ('BEFORE', consts.CLUSTER_RESIZE),
+        ('BEFORE', consts.NODE_DELETE),
     ]
 
     PROFILE_TYPE = [
@@ -86,7 +88,7 @@ class DeletionPolicy(base.Policy):
         REDUCE_DESIRED_CAPACITY: schema.Boolean(
             _('Whether the desired capacity of the cluster should be '
               'reduced along the deletion. Default to False.'),
-            default=False,
+            default=True,
         )
     }
 
@@ -142,6 +144,7 @@ class DeletionPolicy(base.Policy):
         pd['candidates'] = victims
         pd['destroy_after_deletion'] = self.destroy_after_deletion
         pd['grace_period'] = self.grace_period
+        pd['reduce_desired_capacity'] = self.reduce_desired_capacity
         action.data.update({
             'status': base.CHECK_OK,
             'reason': _('Candidates generated'),
@@ -161,6 +164,10 @@ class DeletionPolicy(base.Policy):
             self._update_action(action, victims)
             return
 
+        if action.action == consts.NODE_DELETE:
+            self._update_action(action, [action.node.id])
+            return
+
         db_cluster = None
         regions = None
         zones = None
@@ -177,21 +184,22 @@ class DeletionPolicy(base.Policy):
 
         # No policy decision, check action itself: RESIZE
         else:
-            db_cluster = db_api.cluster_get(action.context, cluster_id)
-            res = scaleutils.parse_resize_params(action, db_cluster)
-            if res[0] == base.CHECK_ERROR:
+            db_cluster = co.Cluster.get(action.context, cluster_id)
+            current = no.Node.count_by_cluster(action.context, cluster_id)
+            res, reason = scaleutils.parse_resize_params(action, db_cluster,
+                                                         current)
+            if res == base.CHECK_ERROR:
                 action.data['status'] = base.CHECK_ERROR
-                action.data['reason'] = res[1]
-                LOG.error(res[1])
+                action.data['reason'] = reason
+                LOG.error(reason)
                 return
 
             if 'deletion' not in action.data:
                 return
             count = action.data['deletion']['count']
 
-        cluster = cluster_mod.Cluster.load(action.context,
-                                           cluster=db_cluster,
-                                           cluster_id=cluster_id)
+        cluster = cm.Cluster.load(action.context, dbcluster=db_cluster,
+                                  cluster_id=cluster_id)
         # Cross-region
         if regions:
             victims = self._victims_by_regions(cluster, regions)
