@@ -23,14 +23,14 @@ from oslo_log import log as logging
 
 from senlin.common import constraints
 from senlin.common import consts
-from senlin.common.i18n import _
-from senlin.common.i18n import _LW
+from senlin.common import exception as exc
+from senlin.common.i18n import _, _LW
 from senlin.common import scaleutils
 from senlin.common import schema
-from senlin.db import api as db_api
-from senlin.drivers import base as driver_base
 from senlin.engine import cluster_policy
-from senlin.engine import node as node_mod
+from senlin.engine import node as nm
+from senlin.objects import cluster as co
+from senlin.objects import node as no
 from senlin.policies import base
 
 LOG = logging.getLogger(__name__)
@@ -44,7 +44,7 @@ class LoadBalancingPolicy(base.Policy):
     the cluster (which could be created by the policy) when these actions are
     performed.
     """
-    VERSION = '1.0'
+    VERSION = '1.1'
 
     PRIORITY = 500
 
@@ -52,9 +52,11 @@ class LoadBalancingPolicy(base.Policy):
         ('AFTER', consts.CLUSTER_ADD_NODES),
         ('AFTER', consts.CLUSTER_SCALE_OUT),
         ('AFTER', consts.CLUSTER_RESIZE),
+        ('AFTER', consts.NODE_CREATE),
         ('BEFORE', consts.CLUSTER_DEL_NODES),
         ('BEFORE', consts.CLUSTER_SCALE_IN),
         ('BEFORE', consts.CLUSTER_RESIZE),
+        ('BEFORE', consts.NODE_DELETE),
     ]
 
     PROFILE_TYPE = [
@@ -62,9 +64,9 @@ class LoadBalancingPolicy(base.Policy):
     ]
 
     KEYS = (
-        POOL, VIP, HEALTH_MONITOR,
+        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT
     ) = (
-        'pool', 'vip', 'health_monitor',
+        'pool', 'vip', 'health_monitor', 'lb_status_timeout'
     )
 
     _POOL_KEYS = (
@@ -159,7 +161,7 @@ class LoadBalancingPolicy(base.Policy):
                     default=True,
                 ),
                 POOL_SESSION_PERSISTENCE: schema.Map(
-                    _('Session pesistence configuration.'),
+                    _('Session persistence configuration.'),
                     schema={
                         PERSISTENCE_TYPE: schema.String(
                             _('Type of session persistence implementation.'),
@@ -213,7 +215,7 @@ class LoadBalancingPolicy(base.Policy):
             _('Health monitor for loadbalancer.'),
             schema={
                 HM_TYPE: schema.String(
-                    _('The type of probe sent by the load balancer to verify '
+                    _('The type of probe sent by the loadbalancer to verify '
                       'the member state.'),
                     constraints=[
                         constraints.AllowedValues(HEALTH_MONITOR_TYPES),
@@ -221,13 +223,13 @@ class LoadBalancingPolicy(base.Policy):
                     default=PING,
                 ),
                 HM_DELAY: schema.Integer(
-                    _('The amount of time in seconds between sending '
+                    _('The amount of time in milliseconds between sending '
                       'probes to members.'),
                     default=10,
                 ),
                 HM_TIMEOUT: schema.Integer(
-                    _('The maximum time in seconds that a monitor waits to '
-                      'connect before it times out.'),
+                    _('The maximum time in milliseconds that a monitor waits '
+                      'to connect before it times out.'),
                     default=5,
                 ),
                 HM_MAX_RETRIES: schema.Integer(
@@ -254,6 +256,13 @@ class LoadBalancingPolicy(base.Policy):
                 ),
             },
         ),
+        LB_STATUS_TIMEOUT: schema.Integer(
+            _('Time in second to wait for loadbalancer to be ready'
+              '(provisioning_status is ACTIVE and operating_status is '
+              'ONLINE) before and after senlin requests lbaas V2 service '
+              'for lb operations. '),
+            default=600,
+        )
     }
 
     def __init__(self, name, spec, **kwargs):
@@ -262,14 +271,34 @@ class LoadBalancingPolicy(base.Policy):
         self.pool_spec = self.properties.get(self.POOL, {})
         self.vip_spec = self.properties.get(self.VIP, {})
         self.hm_spec = self.properties.get(self.HEALTH_MONITOR, None)
-        self.validate()
+        self.lb_status_timeout = self.properties.get(self.LB_STATUS_TIMEOUT)
         self.lb = None
 
-    def validate(self):
-        super(LoadBalancingPolicy, self).validate()
+    def validate(self, context, validate_props=False):
+        super(LoadBalancingPolicy, self).validate(context, validate_props)
 
-        # validate subnet's exists
-        # subnet = self.nc.subnet_get(vip[self.VIP_SUBNET])
+        if not validate_props:
+            return True
+
+        nc = self.network(context.user, context.project)
+
+        # validate pool subnet
+        name_or_id = self.pool_spec.get(self.POOL_SUBNET)
+        try:
+            nc.subnet_get(name_or_id)
+        except exc.InternalError:
+            msg = _("The specified %(key)s '%(value)s' could not be found."
+                    ) % {'key': self.POOL_SUBNET, 'value': name_or_id}
+            raise exc.InvalidSpec(message=msg)
+
+        # validate VIP subnet
+        name_or_id = self.vip_spec.get(self.VIP_SUBNET)
+        try:
+            nc.subnet_get(name_or_id)
+        except exc.InternalError:
+            msg = _("The specified %(key)s '%(value)s' could not be found."
+                    ) % {'key': self.VIP_SUBNET, 'value': name_or_id}
+            raise exc.InvalidSpec(message=msg)
 
     def attach(self, cluster):
         """Routine to be invoked when policy is to be attached to a cluster.
@@ -282,11 +311,11 @@ class LoadBalancingPolicy(base.Policy):
         if res is False:
             return False, data
 
-        nodes = node_mod.Node.load_all(oslo_context.get_current(),
-                                       cluster_id=cluster.id)
+        nodes = nm.Node.load_all(oslo_context.get_current(),
+                                 cluster_id=cluster.id)
 
-        params = self._build_conn_params(cluster)
-        lb_driver = driver_base.SenlinDriver().loadbalancing(params)
+        lb_driver = self.lbaas(cluster.user, cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
 
         res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
                                         self.hm_spec)
@@ -328,8 +357,8 @@ class LoadBalancingPolicy(base.Policy):
             contains a error message.
         """
         reason = _('LB resources deletion succeeded.')
-        params = self._build_conn_params(cluster)
-        lb_driver = driver_base.SenlinDriver().loadbalancing(params)
+        lb_driver = self.lbaas(cluster.user, cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
 
         cp = cluster_policy.ClusterPolicy.load(oslo_context.get_current(),
                                                cluster.id, self.id)
@@ -342,8 +371,8 @@ class LoadBalancingPolicy(base.Policy):
         if res is False:
             return False, reason
 
-        nodes = node_mod.Node.load_all(oslo_context.get_current(),
-                                       cluster_id=cluster.id)
+        nodes = nm.Node.load_all(oslo_context.get_current(),
+                                 cluster_id=cluster.id, project_safe=False)
         for node in nodes:
             if 'lb_member' in node.data:
                 node.data.pop('lb_member')
@@ -365,14 +394,18 @@ class LoadBalancingPolicy(base.Policy):
         # policy or deletion policy is attached.
         candidates = None
         if deletion is None:
-            if action.action == consts.CLUSTER_DEL_NODES:
+            if action.action == consts.NODE_DELETE:
+                candidates = [action.node.id]
+                count = 1
+            elif action.action == consts.CLUSTER_DEL_NODES:
                 # Get candidates from action.input
                 candidates = action.inputs.get('candidates', [])
                 count = len(candidates)
             elif action.action == consts.CLUSTER_RESIZE:
                 # Calculate deletion count based on action input
-                db_cluster = db_api.cluster_get(action.context, cluster_id)
-                scaleutils.parse_resize_params(action, db_cluster)
+                db_cluster = co.Cluster.get(action.context, cluster_id)
+                current = no.Node.count_by_cluster(action.context, cluster_id)
+                scaleutils.parse_resize_params(action, db_cluster, current)
                 if 'deletion' not in action.data:
                     return []
                 else:
@@ -387,8 +420,7 @@ class LoadBalancingPolicy(base.Policy):
         if candidates is None:
             if count == 0:
                 return []
-            nodes = db_api.node_get_all_by_cluster(action.context,
-                                                   cluster_id=cluster_id)
+            nodes = no.Node.get_all_by_cluster(action.context, cluster_id)
             if count > len(nodes):
                 count = len(nodes)
             candidates = scaleutils.nodes_by_random(nodes, count)
@@ -417,9 +449,9 @@ class LoadBalancingPolicy(base.Policy):
         if len(candidates) == 0:
             return
 
-        db_cluster = db_api.cluster_get(action.context, cluster_id)
-        params = self._build_conn_params(db_cluster)
-        lb_driver = driver_base.SenlinDriver().loadbalancing(params)
+        db_cluster = co.Cluster.get(action.context, cluster_id)
+        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
         cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
                                                self.id)
         policy_data = self._extract_policy_data(cp.data)
@@ -428,7 +460,7 @@ class LoadBalancingPolicy(base.Policy):
 
         # Remove nodes that will be deleted from lb pool
         for node_id in candidates:
-            node = node_mod.Node.load(action.context, node_id=node_id)
+            node = nm.Node.load(action.context, node_id=node_id)
             member_id = node.data.get('lb_member', None)
             if member_id is None:
                 LOG.warning(_LW('Node %(n)s not found in lb pool %(p)s.'),
@@ -455,17 +487,19 @@ class LoadBalancingPolicy(base.Policy):
         :param action: The action object that triggered this operation.
         :returns: Nothing.
         """
-
         # TODO(Yanyanhu): Need special handling for cross-az scenario
         # which is supported by Neutron lbaas.
-        creation = action.data.get('creation', None)
-        nodes_added = creation.get('nodes', []) if creation else []
-        if len(nodes_added) == 0:
-            return
+        if action.action == consts.NODE_CREATE:
+            nodes_added = [action.node.id]
+        else:
+            creation = action.data.get('creation', None)
+            nodes_added = creation.get('nodes', []) if creation else []
+            if len(nodes_added) == 0:
+                return
 
-        db_cluster = db_api.cluster_get(action.context, cluster_id)
-        params = self._build_conn_params(db_cluster)
-        lb_driver = driver_base.SenlinDriver().loadbalancing(params)
+        db_cluster = co.Cluster.get(action.context, cluster_id)
+        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        lb_driver.lb_status_timeout = self.lb_status_timeout
         cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
                                                self.id)
         policy_data = self._extract_policy_data(cp.data)
@@ -476,7 +510,7 @@ class LoadBalancingPolicy(base.Policy):
 
         # Add new nodes to lb pool
         for node_id in nodes_added:
-            node = node_mod.Node.load(action.context, node_id=node_id)
+            node = nm.Node.load(action.context, node_id=node_id)
             member_id = node.data.get('lb_member', None)
             if member_id:
                 LOG.warning(_LW('Node %(n)s already in lb pool %(p)s.'),
