@@ -21,13 +21,13 @@ import math
 from oslo_log import log as logging
 
 from senlin.common import consts
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
+from senlin.common import exception as exc
+from senlin.common.i18n import _, _LE
 from senlin.common import scaleutils
 from senlin.common import schema
-from senlin.db import api as db_api
-from senlin.drivers import base as driver_base
-from senlin.engine import cluster as cluster_mod
+from senlin.engine import cluster as cm
+from senlin.objects import cluster as co
+from senlin.objects import node as no
 from senlin.policies import base
 
 LOG = logging.getLogger(__name__)
@@ -44,6 +44,7 @@ class RegionPlacementPolicy(base.Policy):
         ('BEFORE', consts.CLUSTER_SCALE_OUT),
         ('BEFORE', consts.CLUSTER_SCALE_IN),
         ('BEFORE', consts.CLUSTER_RESIZE),
+        ('BEFORE', consts.NODE_CREATE),
     ]
 
     PROFILE_TYPE = [
@@ -88,7 +89,6 @@ class RegionPlacementPolicy(base.Policy):
     def __init__(self, name, spec, **kwargs):
         super(RegionPlacementPolicy, self).__init__(name, spec, **kwargs)
 
-        self._keystoneclient = None
         regions = {}
         for r in self.properties.get(self.REGIONS):
             regions[r[self.REGION_NAME]] = {
@@ -97,18 +97,22 @@ class RegionPlacementPolicy(base.Policy):
             }
         self.regions = regions
 
-    def _keystone(self, obj):
-        """Construct keystone client based on object.
+    def validate(self, context, validate_props=False):
+        super(RegionPlacementPolicy, self).validate(context, validate_props)
 
-        :param obj: Object for which the client is created. It is expected to
-                    be None when retrieving an existing client. When creating
-                    a client, it contains the user and project to be used.
-        """
-        if self._keystoneclient is not None:
-            return self._keystoneclient
-        params = self._build_conn_params(obj)
-        self._keystoneclient = driver_base.SenlinDriver().identity(params)
-        return self._keystoneclient
+        if not validate_props:
+            return True
+
+        kc = self.keystone(context.user, context.project)
+        input_regions = sorted(self.regions.keys())
+        valid_regions = kc.validate_regions(input_regions)
+        invalid_regions = sorted(set(input_regions) - set(valid_regions))
+        if invalid_regions:
+            msg = _("The specified regions '%(value)s' could not be "
+                    "found.") % {'value': invalid_regions}
+            raise exc.InvalidSpec(message=msg)
+
+        return True
 
     def _create_plan(self, current, regions, count, expand):
         """Compute a placement plan based on the weights of regions.
@@ -181,14 +185,23 @@ class RegionPlacementPolicy(base.Policy):
                  to create; 2) negative - number of nodes to delete; 3) 0 -
                  something wrong happened, and the policy check failed.
         """
+        if action.action == consts.NODE_CREATE:
+            # skip node if the context already contains a region_name
+            profile = action.node.rt['profile']
+            if 'region_name' in profile.properties[profile.CONTEXT]:
+                return 0
+            else:
+                return 1
+
         if action.action == consts.CLUSTER_RESIZE:
             if action.data.get('deletion', None):
                 return -action.data['deletion']['count']
             elif action.data.get('creation', None):
                 return action.data['creation']['count']
 
-            db_cluster = db_api.cluster_get(action.context, cluster_id)
-            res = scaleutils.parse_resize_params(action, db_cluster)
+            db_cluster = co.Cluster.get(action.context, cluster_id)
+            curr = no.Node.count_by_cluster(action.context, cluster_id)
+            res = scaleutils.parse_resize_params(action, db_cluster, curr)
             if res[0] == base.CHECK_ERROR:
                 action.data['status'] = base.CHECK_ERROR
                 action.data['reason'] = res[1]
@@ -230,9 +243,8 @@ class RegionPlacementPolicy(base.Policy):
             expand = False
             count = -count
 
-        cluster = cluster_mod.Cluster.load(action.context, cluster_id)
-
-        kc = self._keystone(cluster)
+        cluster = cm.Cluster.load(action.context, cluster_id)
+        kc = self.keystone(cluster.user, cluster.project)
 
         regions_good = kc.validate_regions(self.regions.keys())
         if len(regions_good) == 0:
