@@ -17,7 +17,8 @@ from senlin.common.i18n import _
 from senlin.common import scaleutils as su
 from senlin.common import schema
 from senlin.common import utils
-from senlin.db import api as db_api
+from senlin.objects import cluster as co
+from senlin.objects import node as no
 from senlin.policies import base
 
 
@@ -106,7 +107,7 @@ class ScalingPolicy(base.Policy):
     }
 
     def __init__(self, name, spec, **kwargs):
-        """Intialize a scaling policy object.
+        """Initialize a scaling policy object.
 
         :param name: Name for the policy object.
         :param spec: A dictionary containing the detailed specification for
@@ -164,7 +165,7 @@ class ScalingPolicy(base.Policy):
 
         # Use action input if count is provided
         count = action.inputs.get('count', None)
-        current = db_api.node_count_by_cluster(action.context, cluster_id)
+        current = no.Node.count_by_cluster(action.context, cluster_id)
         if count is None:
             # count not specified, calculate it
             count = self._calculate_adjustment_count(current)
@@ -182,7 +183,7 @@ class ScalingPolicy(base.Policy):
             return
 
         # Check size constraints
-        cluster = db_api.cluster_get(action.context, cluster_id)
+        cluster = co.Cluster.get(action.context, cluster_id)
         if action.action == consts.CLUSTER_SCALE_IN:
             if self.best_effort:
                 count = min(count, current - cluster.min_size)
