@@ -19,16 +19,17 @@ http://docs.openstack.org/developer/senlin/developer/policies/zone_v1.html
 
 import math
 import operator
+
 from oslo_log import log as logging
 
 from senlin.common import consts
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
+from senlin.common import exception as exc
+from senlin.common.i18n import _, _LE
 from senlin.common import scaleutils
 from senlin.common import schema
-from senlin.db import api as db_api
-from senlin.drivers import base as driver_base
-from senlin.engine import cluster as cluster_mod
+from senlin.engine import cluster as cm
+from senlin.objects import cluster as co
+from senlin.objects import node as no
 from senlin.policies import base
 
 LOG = logging.getLogger(__name__)
@@ -45,6 +46,7 @@ class ZonePlacementPolicy(base.Policy):
         ('BEFORE', consts.CLUSTER_SCALE_OUT),
         ('BEFORE', consts.CLUSTER_SCALE_IN),
         ('BEFORE', consts.CLUSTER_RESIZE),
+        ('BEFORE', consts.NODE_CREATE),
     ]
 
     PROFILE_TYPE = [
@@ -85,23 +87,26 @@ class ZonePlacementPolicy(base.Policy):
     def __init__(self, name, spec, **kwargs):
         super(ZonePlacementPolicy, self).__init__(name, spec, **kwargs)
 
-        self._novaclient = None
         self.zones = dict((z[self.ZONE_NAME], z[self.ZONE_WEIGHT])
                           for z in self.properties.get(self.ZONES))
 
-    def _nova(self, obj):
-        """Construct nova client based on object.
+    def validate(self, context, validate_props=False):
+        super(ZonePlacementPolicy, self).validate(context, validate_props)
 
-        :param obj: Object for which the client is created. It is expected to
-                    be None when retrieving an existing client. When creating
-                    a client, it contains the user and project to be used.
-        """
-        if self._novaclient is not None:
-            return self._novaclient
+        if not validate_props:
+            return True
 
-        params = self._build_conn_params(obj)
-        self._novaclient = driver_base.SenlinDriver().compute(params)
-        return self._novaclient
+        nc = self.nova(context.user, context.project)
+        input_azs = sorted(self.zones.keys())
+        valid_azs = nc.validate_azs(input_azs)
+        invalid_azs = sorted(set(input_azs) - set(valid_azs))
+        if invalid_azs:
+            msg = _("The specified %(key)s '%(value)s' could not be "
+                    "found.") % {'key': self.ZONE_NAME,
+                                 'value': list(invalid_azs)}
+            raise exc.InvalidSpec(message=msg)
+
+        return True
 
     def _create_plan(self, current, zones, count, expand):
         """Compute a placement plan based on the weights of AZs.
@@ -164,14 +169,22 @@ class ZonePlacementPolicy(base.Policy):
                  to create; 2) negative - number of nodes to delete; 3) 0 -
                  something wrong happened, and the policy check failed.
         """
+        if action.action == consts.NODE_CREATE:
+            # skip the policy if availability zone is specified in profile
+            profile = action.node.rt['profile']
+            if profile.properties[profile.AVAILABILITY_ZONE]:
+                return 0
+            return 1
+
         if action.action == consts.CLUSTER_RESIZE:
             if action.data.get('deletion', None):
                 return -action.data['deletion']['count']
             elif action.data.get('creation', None):
                 return action.data['creation']['count']
 
-            db_cluster = db_api.cluster_get(action.context, cluster_id)
-            res = scaleutils.parse_resize_params(action, db_cluster)
+            db_cluster = co.Cluster.get(action.context, cluster_id)
+            current = no.Node.count_by_cluster(action.context, cluster_id)
+            res = scaleutils.parse_resize_params(action, db_cluster, current)
             if res[0] == base.CHECK_ERROR:
                 action.data['status'] = base.CHECK_ERROR
                 action.data['reason'] = res[1]
@@ -212,9 +225,9 @@ class ZonePlacementPolicy(base.Policy):
             expand = False
             count = -count
 
-        cluster = cluster_mod.Cluster.load(action.context, cluster_id)
+        cluster = cm.Cluster.load(action.context, cluster_id)
 
-        nc = self._nova(cluster)
+        nc = self.nova(cluster.user, cluster.project)
         zones_good = nc.validate_azs(self.zones.keys())
         if len(zones_good) == 0:
             action.data['status'] = base.CHECK_ERROR
