@@ -17,14 +17,16 @@ from oslo_log import log as logging
 from oslo_utils import timeutils
 import six
 
+from senlin.common import consts
 from senlin.common import context
-from senlin.common import exception
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
+from senlin.common import exception as exc
+from senlin.common.i18n import _, _LE, _LW
 from senlin.common import schema
 from senlin.common import utils
-from senlin.db import api as db_api
+from senlin.drivers import base as driver_base
 from senlin.engine import environment
+from senlin.objects import credential as co
+from senlin.objects import profile as po
 
 LOG = logging.getLogger(__name__)
 
@@ -54,6 +56,7 @@ class Profile(object):
     }
 
     properties_schema = {}
+    operations_schema = {}
 
     def __new__(cls, name, spec, **kwargs):
         """Create a new profile of the appropriate class.
@@ -83,6 +86,8 @@ class Profile(object):
         """
 
         type_name, version = schema.get_spec_version(spec)
+        self.type_name = type_name
+        self.version = version
         type_str = "-".join([type_name, version])
 
         self.name = name
@@ -101,8 +106,10 @@ class Profile(object):
         self.updated_at = kwargs.get('updated_at', None)
 
         self.spec_data = schema.Spec(self.spec_schema, self.spec)
-        self.properties = schema.Spec(self.properties_schema,
-                                      self.spec.get(self.PROPERTIES, {}))
+        self.properties = schema.Spec(
+            self.properties_schema,
+            self.spec.get(self.PROPERTIES, {}),
+            version)
 
         if not self.id:
             # new object needs a context dict
@@ -110,56 +117,61 @@ class Profile(object):
         else:
             self.context = kwargs.get('context')
 
-    @classmethod
-    def from_db_record(cls, record):
-        '''Construct a profile object from database record.
+        # initialize clients
+        self._computeclient = None
+        self._networkclient = None
+        self._orchestrationclient = None
 
-        :param record: a DB Profle object that contains all required fields.
+    @classmethod
+    def from_object(cls, profile):
+        '''Construct a profile from profile object.
+
+        :param profile: a profile object that contains all required fields.
         '''
         kwargs = {
-            'id': record.id,
-            'type': record.type,
-            'context': record.context,
-            'user': record.user,
-            'project': record.project,
-            'domain': record.domain,
-            'metadata': record.meta_data,
-            'created_at': record.created_at,
-            'updated_at': record.updated_at,
+            'id': profile.id,
+            'type': profile.type,
+            'context': profile.context,
+            'user': profile.user,
+            'project': profile.project,
+            'domain': profile.domain,
+            'metadata': profile.metadata,
+            'created_at': profile.created_at,
+            'updated_at': profile.updated_at,
         }
 
-        return cls(record.name, record.spec, **kwargs)
+        return cls(profile.name, profile.spec, **kwargs)
 
     @classmethod
     def load(cls, ctx, profile=None, profile_id=None, project_safe=True):
         '''Retrieve a profile object from database.'''
         if profile is None:
-            profile = db_api.profile_get(ctx, profile_id,
-                                         project_safe=project_safe)
+            profile = po.Profile.get(ctx, profile_id,
+                                     project_safe=project_safe)
             if profile is None:
-                raise exception.ProfileNotFound(profile=profile_id)
+                raise exc.ResourceNotFound(type='profile', id=profile_id)
 
-        return cls.from_db_record(profile)
+        return cls.from_object(profile)
 
     @classmethod
     def load_all(cls, ctx, limit=None, marker=None, sort=None, filters=None,
                  project_safe=True):
         """Retrieve all profiles from database."""
 
-        records = db_api.profile_get_all(ctx, limit=limit, marker=marker,
-                                         sort=sort, filters=filters,
-                                         project_safe=project_safe)
+        records = po.Profile.get_all(ctx, limit=limit, marker=marker,
+                                     sort=sort, filters=filters,
+                                     project_safe=project_safe)
 
         for record in records:
-            yield cls.from_db_record(record)
+            yield cls.from_object(record)
 
     @classmethod
     def delete(cls, ctx, profile_id):
-        db_api.profile_delete(ctx, profile_id)
+        po.Profile.delete(ctx, profile_id)
 
     def store(self, ctx):
         '''Store the profile into database and return its ID.'''
-        timestamp = timeutils.utcnow()
+        timestamp = timeutils.utcnow(True)
 
         values = {
             'name': self.name,
@@ -175,11 +187,11 @@ class Profile(object):
         if self.id:
             self.updated_at = timestamp
             values['updated_at'] = timestamp
-            db_api.profile_update(ctx, self.id, values)
+            po.Profile.update(ctx, self.id, values)
         else:
             self.created_at = timestamp
             values['created_at'] = timestamp
-            profile = db_api.profile_create(ctx, values)
+            profile = po.Profile.create(ctx, values)
             self.id = profile.id
 
         return self.id
@@ -190,14 +202,9 @@ class Profile(object):
         return profile.do_create(obj)
 
     @classmethod
-    def check_object(cls, ctx, obj):
+    def delete_object(cls, ctx, obj, **params):
         profile = cls.load(ctx, profile_id=obj.profile_id)
-        return profile.do_check(obj)
-
-    @classmethod
-    def delete_object(cls, ctx, obj):
-        profile = cls.load(ctx, profile_id=obj.profile_id)
-        return profile.do_delete(obj)
+        return profile.do_delete(obj, **params)
 
     @classmethod
     def update_object(cls, ctx, obj, new_profile_id=None, **params):
@@ -207,11 +214,6 @@ class Profile(object):
             new_profile = cls.load(ctx, profile_id=new_profile_id)
         return profile.do_update(obj, new_profile, **params)
 
-    @classmethod
-    def recover_object(cls, ctx, obj, **options):
-        profile = cls.load(ctx, profile_id=obj.profile_id)
-        return profile.do_recover(obj, **options)
-
     @classmethod
     def get_details(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
@@ -227,7 +229,17 @@ class Profile(object):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_leave(obj)
 
-    def validate(self):
+    @classmethod
+    def check_object(cls, ctx, obj):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        return profile.do_check(obj)
+
+    @classmethod
+    def recover_object(cls, ctx, obj, **options):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        return profile.do_recover(obj, **options)
+
+    def validate(self, validate_props=False):
         '''Validate the schema and the data provided.'''
         # general validation
         self.spec_data.validate()
@@ -235,6 +247,9 @@ class Profile(object):
 
         # TODO(Anyone): need to check the contents in self.CONTEXT
 
+        if validate_props:
+            self.do_validate(obj=self)
+
     @classmethod
     def get_schema(cls):
         return dict((name, dict(schema))
@@ -260,9 +275,9 @@ class Profile(object):
         :returns: A dict containing the required parameters for connection
                   creation.
         """
-        cred = db_api.cred_get(oslo_context.get_current(), user, project)
+        cred = co.Credential.get(oslo_context.get_current(), user, project)
         if cred is None:
-            raise exception.TrustNotFound(trustor=user)
+            raise exc.TrustNotFound(trustor=user)
 
         trust_id = cred.cred['openstack']['trust']
 
@@ -272,61 +287,118 @@ class Profile(object):
 
         return params
 
+    def compute(self, obj):
+        '''Construct compute client based on object.
+
+        :param obj: Object for which the client is created. It is expected to
+                    be None when retrieving an existing client. When creating
+                    a client, it contains the user and project to be used.
+        '''
+
+        if self._computeclient is not None:
+            return self._computeclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._computeclient = driver_base.SenlinDriver().compute(params)
+        return self._computeclient
+
+    def network(self, obj):
+        """Construct network client based on object.
+
+        :param obj: Object for which the client is created. It is expected to
+                    be None when retrieving an existing client. When creating
+                    a client, it contains the user and project to be used.
+        """
+        if self._networkclient is not None:
+            return self._networkclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._networkclient = driver_base.SenlinDriver().network(params)
+        return self._networkclient
+
+    def orchestration(self, obj):
+        """Construct orchestration client based on object.
+
+        :param obj: Object for which the client is created. It is expected to
+                    be None when retrieving an existing client. When creating
+                    a client, it contains the user and project to be used.
+        """
+        if self._orchestrationclient is not None:
+            return self._orchestrationclient
+        params = self._build_conn_params(obj.user, obj.project)
+        oc = driver_base.SenlinDriver().orchestration(params)
+        self._orchestrationclient = oc
+        return oc
+
     def do_create(self, obj):
-        '''For subclass to override.'''
+        """For subclass to override."""
+        raise NotImplementedError
 
-        return NotImplemented
-
-    def do_delete(self, obj):
-        '''For subclass to override.'''
-
-        return NotImplemented
+    def do_delete(self, obj, **params):
+        """For subclass to override."""
+        raise NotImplementedError
 
     def do_update(self, obj, new_profile, **params):
-        '''For subclass to override.'''
-
-        return NotImplemented
+        """For subclass to override."""
+        LOG.warning(_LW("Update operation not supported."))
+        return True
 
     def do_check(self, obj):
-        '''For subclass to override.'''
-        return NotImplemented
+        """For subclass to override."""
+        LOG.warning(_LW("Check operation not supported."))
+        return True
 
     def do_get_details(self, obj):
-        '''For subclass to override.'''
-        return NotImplemented
+        """For subclass to override."""
+        LOG.warning(_LW("Get_details operation not supported."))
+        return {}
 
     def do_join(self, obj, cluster_id):
-        '''For subclass to override to perform extra operations.'''
+        """For subclass to override to perform extra operations."""
+        LOG.warning(_LW("Join operation not specialized."))
         return True
 
     def do_leave(self, obj):
-        '''For subclass to override to perform extra operations.'''
+        """For subclass to override to perform extra operations."""
+        LOG.warning(_LW("Join operation not specialized."))
         return True
 
     def do_rebuild(self, obj):
-        '''For subclass to override.'''
-        return NotImplemented
+        """For subclass to override."""
+        LOG.warning(_LW("Rebuild operation not specialized."))
+        return True
 
     def do_recover(self, obj, **options):
-        '''For subclass to override.'''
+        """Default recover operation.
 
-        operation = options.get('operation', None)
-        if operation and operation != 'RECREATE':
-            return NotImplemented
+        :param obj: The node object to operate on.
+        :param options: Keyword arguments for the recover operation.
+        """
+        operation = options.pop('operation', None)
+        # TODO(Qiming): The operation input could be a list of operations.
+        if operation and not isinstance(operation, six.string_types):
+            operation = operation[0]
 
-        # NOTE: do_delete always returns a boolean
-        res = self.do_delete(obj)
-
-        if res:
-            try:
-                res = self.do_create(obj)
-            except Exception as ex:
-                LOG.exception(_('Failed at recovering obj: %s '),
-                              six.text_type(ex))
-                return False
+        if operation and operation != consts.RECOVER_RECREATE:
+            LOG.error(_LE("Recover operation not supported: %s"), operation)
+            return False
 
+        try:
+            self.do_delete(obj, **options)
+        except exc.EResourceDeletion as ex:
+            raise exc.EResourceOperation(op='recovering', type='node',
+                                         id=obj.id, message=six.text_type(ex))
+        res = None
+        try:
+            res = self.do_create(obj)
+        except exc.EResourceCreation as ex:
+            raise exc.EResourceOperation(op='recovering', type='node',
+                                         id=obj.id, message=six.text_type(ex))
         return res
 
+    def do_validate(self, obj):
+        """For subclass to override."""
+        LOG.warning(_LW("Validate operation not supported."))
+        return True
+
     def to_dict(self):
         pb_dict = {
             'id': self.id,
@@ -337,8 +409,8 @@ class Profile(object):
             'domain': self.domain,
             'spec': self.spec,
             'metadata': self.metadata,
-            'created_at': utils.format_time(self.created_at),
-            'updated_at': utils.format_time(self.updated_at),
+            'created_at': utils.isotime(self.created_at),
+            'updated_at': utils.isotime(self.updated_at),
         }
         return pb_dict
 
