@@ -49,7 +49,7 @@ class DBAPIClusterTest(base.SenlinTestCase):
         self.assertIsNone(cluster.created_at)
         self.assertIsNone(cluster.updated_at)
         self.assertIsNotNone(cluster.init_at)
-        self.assertEqual({}, cluster.metadata)
+        self.assertEqual({}, cluster.meta_data)
         self.assertIsNone(cluster.data)
 
     def test_cluster_get_returns_a_cluster(self):
@@ -222,6 +222,8 @@ class DBAPIClusterTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='another-project',
                                         is_admin=True)
         clusters = db_api.cluster_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(clusters))
+        clusters = db_api.cluster_get_all(admin_ctx, project_safe=False)
         self.assertEqual(5, len(clusters))
 
     def test_cluster_get_all_with_filters(self):
@@ -249,7 +251,7 @@ class DBAPIClusterTest(base.SenlinTestCase):
 
     def test_cluster_get_all_default_sort_dir(self):
         clusters = [shared.create_cluster(self.ctx, self.profile,
-                                          init_at=tu.utcnow())
+                                          init_at=tu.utcnow(True))
                     for x in range(3)]
 
         st_db = db_api.cluster_get_all(self.ctx)
@@ -260,7 +262,7 @@ class DBAPIClusterTest(base.SenlinTestCase):
 
     def test_cluster_get_all_str_sort_keys(self):
         clusters = [shared.create_cluster(self.ctx, self.profile,
-                                          created_at=tu.utcnow())
+                                          created_at=tu.utcnow(True))
                     for x in range(3)]
 
         st_db = db_api.cluster_get_all(self.ctx, sort='created_at')
@@ -282,7 +284,7 @@ class DBAPIClusterTest(base.SenlinTestCase):
 
     def test_cluster_get_all_marker(self):
         clusters = [shared.create_cluster(self.ctx, self.profile,
-                                          created_at=tu.utcnow())
+                                          created_at=tu.utcnow(True))
                     for x in range(3)]
         cl_db = db_api.cluster_get_all(self.ctx, marker=clusters[1].id)
         self.assertEqual(1, len(cl_db))
@@ -364,8 +366,10 @@ class DBAPIClusterTest(base.SenlinTestCase):
 
         admin_ctx = utils.dummy_context(project='another-project',
                                         is_admin=True)
-        self.assertEqual(5, db_api.cluster_count_all(admin_ctx,
+        self.assertEqual(0, db_api.cluster_count_all(admin_ctx,
                                                      project_safe=True))
+        self.assertEqual(5, db_api.cluster_count_all(admin_ctx,
+                                                     project_safe=False))
 
     def test_cluster_count_all_with_filters(self):
         shared.create_cluster(self.ctx, self.profile, name='foo')
@@ -391,7 +395,7 @@ class DBAPIClusterTest(base.SenlinTestCase):
         self.assertEqual('update failed', cluster.status_reason)
         self.assertEqual(90, cluster.timeout)
 
-        self.assertRaises(exception.ClusterNotFound,
+        self.assertRaises(exception.ResourceNotFound,
                           db_api.cluster_update, self.ctx, UUID2, values)
 
     def test_nested_cluster_get_by_name(self):
@@ -417,7 +421,7 @@ class DBAPIClusterTest(base.SenlinTestCase):
         self.assertIsNone(db_api.cluster_get(self.ctx, cluster_id))
         res = db_api.node_get(self.ctx, node.id)
         self.assertIsNone(res)
-        self.assertRaises(exception.ClusterNotFound, db_api.cluster_delete,
+        self.assertRaises(exception.ResourceNotFound, db_api.cluster_delete,
                           self.ctx, cluster_id)
 
         # Testing child nodes deletion
