@@ -49,7 +49,7 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertIsNone(node.updated_at)
         self.assertEqual('ACTIVE', node.status)
         self.assertEqual('create complete', node.status_reason)
-        self.assertEqual('{"foo": "123"}', jsonutils.dumps(node.metadata))
+        self.assertEqual('{"foo": "123"}', jsonutils.dumps(node.meta_data))
         self.assertEqual('{"key1": "value1"}', jsonutils.dumps(node.data))
         self.assertEqual(self.cluster.id, node.cluster_id)
         self.assertEqual(self.profile.id, node.profile_id)
@@ -178,7 +178,7 @@ class DBAPINodeTest(base.SenlinTestCase):
         node_ids = ['node1', 'node2', 'node3']
         for v in node_ids:
             shared.create_node(self.ctx, self.cluster, self.profile,
-                               id=v, init_at=tu.utcnow())
+                               id=v, init_at=tu.utcnow(True))
 
         nodes = db_api.node_get_all(self.ctx, limit=1)
         self.assertEqual(1, len(nodes))
@@ -247,7 +247,7 @@ class DBAPINodeTest(base.SenlinTestCase):
 
     def test_node_get_all_default_sorting(self):
         nodes = [shared.create_node(self.ctx, None, self.profile,
-                                    init_at=tu.utcnow())
+                                    init_at=tu.utcnow(True))
                  for x in range(3)]
 
         results = db_api.node_get_all(self.ctx)
@@ -300,6 +300,8 @@ class DBAPINodeTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a_different_project',
                                         is_admin=True)
         results = db_api.node_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(results))
+        results = db_api.node_get_all(admin_ctx, project_safe=False)
         self.assertEqual(2, len(results))
 
     def test_node_get_by_cluster(self):
@@ -356,6 +358,9 @@ class DBAPINodeTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a_different_project',
                                         is_admin=True)
         nodes = db_api.node_get_all_by_cluster(admin_ctx, self.cluster.id)
+        self.assertEqual(0, len(nodes))
+        nodes = db_api.node_get_all_by_cluster(admin_ctx, self.cluster.id,
+                                               project_safe=False)
         self.assertEqual(2, len(nodes))
         self.assertEqual(set([node1.id, node2.id]),
                          set([nodes[0].id, nodes[1].id]))
@@ -369,6 +374,21 @@ class DBAPINodeTest(base.SenlinTestCase):
         res = db_api.node_count_by_cluster(self.ctx, self.cluster.id)
         self.assertEqual(2, res)
 
+    def test_node_count_by_cluster_with_filters(self):
+        shared.create_cluster(self.ctx, self.profile)
+
+        shared.create_node(self.ctx, self.cluster, self.profile,
+                           status='ACTIVE')
+        shared.create_node(self.ctx, self.cluster, self.profile,
+                           status='ERROR')
+
+        res = db_api.node_count_by_cluster(self.ctx, self.cluster.id,
+                                           status='ACTIVE')
+        self.assertEqual(1, res)
+        res = db_api.node_count_by_cluster(self.ctx, self.cluster.id,
+                                           status='ERROR')
+        self.assertEqual(1, res)
+
     def test_node_count_by_cluster_diff_project(self):
         ctx_new = utils.dummy_context(project='a_different_project')
         shared.create_cluster(self.ctx, self.profile)
@@ -392,6 +412,9 @@ class DBAPINodeTest(base.SenlinTestCase):
                                         is_admin=True)
         res = db_api.node_count_by_cluster(admin_ctx, self.cluster.id,
                                            project_safe=True)
+        self.assertEqual(0, res)
+        res = db_api.node_count_by_cluster(admin_ctx, self.cluster.id,
+                                           project_safe=False)
         self.assertEqual(2, res)
 
     def test_node_update(self):
@@ -410,7 +433,7 @@ class DBAPINodeTest(base.SenlinTestCase):
 
     def test_node_update_not_found(self):
         new_attributes = {'name': 'new_name'}
-        ex = self.assertRaises(exception.NodeNotFound,
+        ex = self.assertRaises(exception.ResourceNotFound,
                                db_api.node_update,
                                self.ctx, 'BogusId', new_attributes)
         self.assertEqual('The node (BogusId) could not be found.',
@@ -442,7 +465,7 @@ class DBAPINodeTest(base.SenlinTestCase):
 
     def test_node_migrate_from_none(self):
         node_orphan = shared.create_node(self.ctx, None, self.profile)
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node = db_api.node_migrate(self.ctx, node_orphan.id, self.cluster.id,
                                    timestamp, 'NEW-ROLE')
@@ -456,7 +479,7 @@ class DBAPINodeTest(base.SenlinTestCase):
 
     def test_node_migrate_to_none(self):
         node = shared.create_node(self.ctx, self.cluster, self.profile)
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node_new = db_api.node_migrate(self.ctx, node.id, None, timestamp)
         self.assertEqual(timestamp, node_new.updated_at)
@@ -473,10 +496,13 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(1, len(nodes))
         nodes = db_api.node_get_all_by_cluster(self.ctx, cluster2.id)
         self.assertEqual(0, len(nodes))
+        # Refresh cluster1 and cluster2
+        cluster1 = db_api.cluster_get(self.ctx, cluster1.id)
+        cluster2 = db_api.cluster_get(self.ctx, cluster2.id)
         self.assertEqual(2, cluster1.next_index)
         self.assertEqual(1, cluster2.next_index)
 
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node_new = db_api.node_migrate(self.ctx, node.id, cluster2.id,
                                        timestamp)
@@ -493,7 +519,7 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(2, cluster2.next_index)
 
         # Migrate it back!
-        timestamp = tu.utcnow()
+        timestamp = tu.utcnow(True)
 
         node_new = db_api.node_migrate(self.ctx, node.id, cluster1.id,
                                        timestamp, 'FAKE-ROLE')
