@@ -227,12 +227,14 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a-different-project',
                                         is_admin=True)
         policies = db_api.policy_get_all(admin_ctx, project_safe=True)
+        self.assertEqual(0, len(policies))
+        policies = db_api.policy_get_all(admin_ctx, project_safe=False)
         self.assertEqual(2, len(policies))
 
     def test_policy_get_all_with_limit_marker(self):
         ids = ['policy1', 'policy2', 'policy3']
         for pid in ids:
-            timestamp = tu.utcnow()
+            timestamp = tu.utcnow(True)
             data = self.new_policy_data(id=pid, created_at=timestamp)
             db_api.policy_create(self.ctx, data)
 
@@ -282,7 +284,7 @@ class DBAPIPolicyTest(base.SenlinTestCase):
                   {'id': '003', 'name': 'policy2'}]
 
         for v in values:
-            v['created_at'] = tu.utcnow()
+            v['created_at'] = tu.utcnow(True)
             data = self.new_policy_data(**v)
             db_api.policy_create(self.ctx, data)
 
@@ -310,7 +312,7 @@ class DBAPIPolicyTest(base.SenlinTestCase):
     def test_policy_get_all_default_sorting(self):
         policies = []
         for x in range(3):
-            data = self.new_policy_data(created_at=tu.utcnow())
+            data = self.new_policy_data(created_at=tu.utcnow(True))
             policies.append(db_api.policy_create(self.ctx, data))
 
         results = db_api.policy_get_all(self.ctx)
@@ -363,7 +365,7 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         self.assertEqual('new_scaling_policy', new_policy.name)
 
     def test_policy_update_not_found(self):
-        self.assertRaises(exception.PolicyNotFound,
+        self.assertRaises(exception.ResourceNotFound,
                           db_api.policy_update,
                           self.ctx, 'BogusID', {})
 
@@ -390,7 +392,7 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         }
         db_api.cluster_policy_attach(self.ctx, self.cluster.id, policy.id,
                                      fields)
-        self.assertRaises(exception.ResourceBusyError,
+        self.assertRaises(exception.EResourceBusy,
                           db_api.policy_delete,
                           self.ctx, policy.id)
 
