@@ -14,23 +14,30 @@ import copy
 
 import mock
 from oslo_config import cfg
+from oslo_utils import timeutils
 import six
 
 from senlin.common import exception
-from senlin.db.sqlalchemy import api as db_api
-from senlin.engine.actions import base as action_base
+from senlin.common import utils as common_utils
+from senlin.engine.actions import base as ab
 from senlin.engine import cluster as cluster_mod
 from senlin.engine import cluster_policy as cp_mod
 from senlin.engine import environment
 from senlin.engine import event as EVENT
 from senlin.engine import node as node_mod
+from senlin.objects import action as ao
+from senlin.objects import dependency as dobj
 from senlin.policies import base as policy_mod
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 from senlin.tests.unit import fakes
 
+CLUSTER_ID = 'e1cfd82b-dc95-46ad-86e8-37864d7be1cd'
+OBJID = '571fffb8-f41c-4cbc-945c-cb2937d76f19'
+OWNER_ID = 'c7114713-ee68-409d-ba5d-0560a72a386c'
 
-class DummyAction(action_base.Action):
+
+class DummyAction(ab.Action):
 
     def __init__(self, target, action, context, **kwargs):
         super(DummyAction, self).__init__(target, action, context, **kwargs)
@@ -45,7 +52,7 @@ class ActionBaseTest(base.SenlinTestCase):
         self.action_values = {
             'name': 'FAKE_NAME',
             'cause': 'FAKE_CAUSE',
-            'owner': 'FAKE_OWNER',
+            'owner': OWNER_ID,
             'interval': 60,
             'start_time': 0,
             'end_time': 0,
@@ -54,7 +61,7 @@ class ActionBaseTest(base.SenlinTestCase):
             'status_reason': 'FAKE_STATUS_REASON',
             'inputs': {'param': 'value'},
             'outputs': {'key': 'output_value'},
-            'created_at': None,
+            'created_at': timeutils.utcnow(True),
             'updated_at': None,
             'data': {'data_key': 'data_value'},
         }
@@ -62,7 +69,6 @@ class ActionBaseTest(base.SenlinTestCase):
     def _verify_new_action(self, obj, target, action):
         self.assertIsNone(obj.id)
         self.assertEqual('', obj.name)
-        self.assertEqual('', obj.description)
         self.assertEqual(target, obj.target)
         self.assertEqual(action, obj.action)
         self.assertEqual('', obj.cause)
@@ -83,25 +89,22 @@ class ActionBaseTest(base.SenlinTestCase):
     @mock.patch.object(cluster_mod.Cluster, 'load')
     def test_action_new(self, mock_n_load, mock_c_load):
         for action in ['CLUSTER_CREATE', 'NODE_CREATE', 'WHAT_EVER']:
-            obj = action_base.Action('OBJID', action, self.ctx)
-            self._verify_new_action(obj, 'OBJID', action)
+            obj = ab.Action(OBJID, action, self.ctx)
+            self._verify_new_action(obj, OBJID, action)
 
     def test_action_init_with_values(self):
         values = copy.deepcopy(self.action_values)
         values['id'] = 'FAKE_ID'
-        values['description'] = 'FAKE_DESC'
         values['created_at'] = 'FAKE_CREATED_TIME'
         values['updated_at'] = 'FAKE_UPDATED_TIME'
 
-        obj = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                 **values)
+        obj = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
 
         self.assertEqual('FAKE_ID', obj.id)
         self.assertEqual('FAKE_NAME', obj.name)
-        self.assertEqual('FAKE_DESC', obj.description)
-        self.assertEqual('OBJID', obj.target)
+        self.assertEqual(OBJID, obj.target)
         self.assertEqual('FAKE_CAUSE', obj.cause)
-        self.assertEqual('FAKE_OWNER', obj.owner)
+        self.assertEqual(OWNER_ID, obj.owner)
         self.assertEqual(60, obj.interval)
         self.assertEqual(0, obj.start_time)
         self.assertEqual(0, obj.end_time)
@@ -116,10 +119,10 @@ class ActionBaseTest(base.SenlinTestCase):
 
     def test_action_store_for_create(self):
         values = copy.deepcopy(self.action_values)
-        obj = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                 **values)
+        obj = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
 
-        self.assertIsNone(obj.created_at)
+        self.assertEqual(common_utils.isotime(values['created_at']),
+                         common_utils.isotime(obj.created_at))
         self.assertIsNone(obj.updated_at)
 
         # store for creation
@@ -132,8 +135,7 @@ class ActionBaseTest(base.SenlinTestCase):
     def test_action_store_for_update(self):
         values = copy.deepcopy(self.action_values)
 
-        obj = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                 **values)
+        obj = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         obj_id = obj.store(self.ctx)
         self.assertIsNotNone(obj_id)
         self.assertIsNotNone(obj.created_at)
@@ -149,14 +151,13 @@ class ActionBaseTest(base.SenlinTestCase):
 
     def test_from_db_record(self):
         values = copy.deepcopy(self.action_values)
-        obj = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                 **values)
+        obj = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         obj.store(self.ctx)
 
-        record = db_api.action_get(self.ctx, obj.id)
+        record = ao.Action.get(self.ctx, obj.id)
 
-        action_obj = action_base.Action._from_db_record(record)
-        self.assertIsInstance(action_obj, action_base.Action)
+        action_obj = ab.Action._from_object(record)
+        self.assertIsInstance(action_obj, ab.Action)
         self.assertEqual(obj.id, action_obj.id)
         self.assertEqual(obj.action, action_obj.action)
         self.assertEqual(obj.name, action_obj.name)
@@ -171,7 +172,8 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertEqual(obj.status_reason, action_obj.status_reason)
         self.assertEqual(obj.inputs, action_obj.inputs)
         self.assertEqual(obj.outputs, action_obj.outputs)
-        self.assertEqual(obj.created_at, action_obj.created_at)
+        self.assertEqual(common_utils.isotime(obj.created_at),
+                         common_utils.isotime(action_obj.created_at))
         self.assertEqual(obj.updated_at, action_obj.updated_at)
         self.assertEqual(obj.data, action_obj.data)
         self.assertEqual(obj.user, action_obj.user)
@@ -182,71 +184,68 @@ class ActionBaseTest(base.SenlinTestCase):
         values = copy.deepcopy(self.action_values)
         del values['inputs']
         del values['outputs']
-        obj = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                 **values)
+        obj = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         obj.store(self.ctx)
-        record = db_api.action_get(self.ctx, obj.id)
-        action_obj = action_base.Action._from_db_record(record)
+        record = ao.Action.get(self.ctx, obj.id)
+        action_obj = ab.Action._from_object(record)
         self.assertEqual({}, action_obj.inputs)
         self.assertEqual({}, action_obj.outputs)
 
     def test_load(self):
         values = copy.deepcopy(self.action_values)
-        obj = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx, **values)
+        obj = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         obj.store(self.ctx)
 
-        result = action_base.Action.load(self.ctx, obj.id, None)
+        result = ab.Action.load(self.ctx, obj.id, None)
         # no need to do a thorough test here
         self.assertEqual(obj.id, result.id)
         self.assertEqual(obj.action, result.action)
 
-        db_action = db_api.action_get(self.ctx, obj.id)
-        result = action_base.Action.load(self.ctx, None, db_action)
+        db_action = ao.Action.get(self.ctx, obj.id)
+        result = ab.Action.load(self.ctx, None, db_action)
         # no need to do a thorough test here
         self.assertEqual(obj.id, result.id)
         self.assertEqual(obj.action, result.action)
 
     def test_load_not_found(self):
         # not found due to bad identity
-        ex = self.assertRaises(exception.ActionNotFound,
-                               action_base.Action.load,
+        ex = self.assertRaises(exception.ResourceNotFound,
+                               ab.Action.load,
                                self.ctx, 'non-existent', None)
         self.assertEqual('The action (non-existent) could not be found.',
                          six.text_type(ex))
 
         # not found due to no object
-        self.patchobject(db_api, 'action_get', return_value=None)
-        ex = self.assertRaises(exception.ActionNotFound,
-                               action_base.Action.load,
+        self.patchobject(ao.Action, 'get', return_value=None)
+        ex = self.assertRaises(exception.ResourceNotFound,
+                               ab.Action.load,
                                self.ctx, 'whatever', None)
         self.assertEqual('The action (whatever) could not be found.',
                          six.text_type(ex))
 
     def test_load_all(self):
-        result = action_base.Action.load_all(self.ctx)
+        result = ab.Action.load_all(self.ctx)
         self.assertEqual([], [c for c in result])
 
         values = copy.deepcopy(self.action_values)
-        action1 = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                     **values)
+        action1 = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         action1.store(self.ctx)
-        action2 = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                     **values)
+        action2 = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         action2.store(self.ctx)
 
         # NOTE: we don't test all other parameters because the db api tests
         #       already covered that
-        results = list(action_base.Action.load_all(self.ctx))
+        results = list(ab.Action.load_all(self.ctx))
         actions = [a.id for a in results]
         self.assertEqual(2, len(actions))
         self.assertIn(action1.id, actions)
         self.assertIn(action2.id, actions)
 
-    @mock.patch.object(db_api, 'action_get_all')
+    @mock.patch.object(ao.Action, 'get_all')
     def test_load_all_with_params(self, mock_call):
         mock_call.return_value = []
 
-        results = action_base.Action.load_all(
+        results = ab.Action.load_all(
             self.ctx, filters='FAKE_FILTER', limit='FAKE_LIMIT',
             marker='FAKE_MARKER', sort='FAKE_SORT')
 
@@ -258,51 +257,48 @@ class ActionBaseTest(base.SenlinTestCase):
             marker='FAKE_MARKER', sort='FAKE_SORT',
             project_safe=True)
 
-    @mock.patch.object(action_base.Action, 'store')
+    @mock.patch.object(ab.Action, 'store')
     def test_action_create(self, mock_store):
         mock_store.return_value = 'FAKE_ID'
 
-        result = action_base.Action.create(self.ctx, 'OBJ_ID', 'CLUSTER_DANCE',
-                                           name='test')
+        result = ab.Action.create(self.ctx, OBJID, 'CLUSTER_DANCE',
+                                  name='test')
 
         self.assertEqual('FAKE_ID', result)
         mock_store.assert_called_once_with(self.ctx)
 
     def test_action_delete(self):
-        result = action_base.Action.delete(self.ctx, 'non-existent')
+        result = ab.Action.delete(self.ctx, 'non-existent')
         self.assertIsNone(result)
 
         values = copy.deepcopy(self.action_values)
-        action1 = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                     **values)
+        action1 = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         action1.store(self.ctx)
 
-        result = action_base.Action.delete(self.ctx, action1.id)
+        result = ab.Action.delete(self.ctx, action1.id)
         self.assertIsNone(result)
 
-    @mock.patch.object(db_api, 'action_delete')
+    @mock.patch.object(ao.Action, 'delete')
     def test_action_delete_db_call(self, mock_call):
         # test db api call
-        action_base.Action.delete(self.ctx, 'FAKE_ID')
-        mock_call.assert_called_once_with(self.ctx, 'FAKE_ID', False)
+        ab.Action.delete(self.ctx, 'FAKE_ID')
+        mock_call.assert_called_once_with(self.ctx, 'FAKE_ID')
 
-    @mock.patch.object(db_api, 'action_signal')
+    @mock.patch.object(ao.Action, 'signal')
     def test_action_signal_bad_command(self, mock_call):
         values = copy.deepcopy(self.action_values)
-        action1 = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                     **values)
+        action1 = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         action1.store(self.ctx)
 
         result = action1.signal('BOGUS')
         self.assertIsNone(result)
         self.assertEqual(0, mock_call.call_count)
 
-    @mock.patch.object(db_api, 'action_signal')
+    @mock.patch.object(ao.Action, 'signal')
     @mock.patch.object(EVENT, 'error')
     def test_action_signal_cancel(self, mock_error, mock_call):
         values = copy.deepcopy(self.action_values)
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                    **values)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, **values)
         action.store(self.ctx)
 
         expected = [action.INIT, action.WAITING, action.READY, action.RUNNING]
@@ -324,10 +320,10 @@ class ActionBaseTest(base.SenlinTestCase):
             self.assertEqual(1, mock_error.call_count)
             mock_error.reset_mock()
 
-    @mock.patch.object(db_api, 'action_signal')
+    @mock.patch.object(ao.Action, 'signal')
     @mock.patch.object(EVENT, 'error')
     def test_action_signal_suspend(self, mock_error, mock_call):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
 
         expected = [action.RUNNING]
         for status in expected:
@@ -348,10 +344,10 @@ class ActionBaseTest(base.SenlinTestCase):
             self.assertEqual(1, mock_error.call_count)
             mock_error.reset_mock()
 
-    @mock.patch.object(db_api, 'action_signal')
+    @mock.patch.object(ao.Action, 'signal')
     @mock.patch.object(EVENT, 'error')
     def test_action_signal_resume(self, mock_error, mock_call):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
 
         expected = [action.SUSPENDED]
         for status in expected:
@@ -373,18 +369,20 @@ class ActionBaseTest(base.SenlinTestCase):
             mock_error.reset_mock()
 
     def test_execute_default(self):
-        action = action_base.Action.__new__(DummyAction, 'OBJID', 'BOOM',
-                                            self.ctx)
+        action = ab.Action.__new__(DummyAction, OBJID, 'BOOM', self.ctx)
         res = action.execute()
         self.assertEqual(NotImplemented, res)
 
-    @mock.patch.object(db_api, 'action_mark_succeeded')
-    @mock.patch.object(db_api, 'action_mark_failed')
-    @mock.patch.object(db_api, 'action_mark_cancelled')
-    @mock.patch.object(db_api, 'action_abandon')
+    @mock.patch.object(EVENT, 'info')
+    @mock.patch.object(EVENT, 'error')
+    @mock.patch.object(EVENT, 'warning')
+    @mock.patch.object(ao.Action, 'mark_succeeded')
+    @mock.patch.object(ao.Action, 'mark_failed')
+    @mock.patch.object(ao.Action, 'mark_cancelled')
+    @mock.patch.object(ao.Action, 'abandon')
     def test_set_status(self, mock_abandon, mark_cancel, mark_fail,
-                        mark_succeed):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+                        mark_succeed, mock_event, mock_error, mock_info):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.id = 'FAKE_ID'
 
         action.set_status(action.RES_OK, 'FAKE_REASON')
@@ -419,11 +417,11 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertEqual('BUSY', action.status_reason)
         mock_abandon.assert_called_once_with(action.context, 'FAKE_ID')
 
-    @mock.patch.object(db_api, 'action_check_status')
+    @mock.patch.object(ao.Action, 'check_status')
     def test_get_status(self, mock_get):
         mock_get.return_value = 'FAKE_STATUS'
 
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.id = 'FAKE_ID'
 
         res = action.get_status()
@@ -432,10 +430,9 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertEqual('FAKE_STATUS', action.status)
         mock_get.assert_called_once_with(action.context, 'FAKE_ID', mock.ANY)
 
-    @mock.patch.object(action_base, 'wallclock')
+    @mock.patch.object(ab, 'wallclock')
     def test_is_timeout(self, mock_time):
-        action = action_base.Action.__new__(DummyAction, 'OBJ', 'BOOM',
-                                            self.ctx)
+        action = ab.Action.__new__(DummyAction, 'OBJ', 'BOOM', self.ctx)
         action.start_time = 1
         action.timeout = 10
 
@@ -451,8 +448,9 @@ class ActionBaseTest(base.SenlinTestCase):
         mock_time.return_value = 12
         self.assertTrue(action.is_timeout())
 
-    def test_check_signal_timeout(self):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+    @mock.patch.object(EVENT, 'debug')
+    def test_check_signal_timeout(self, mock_debug):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.id = 'FAKE_ID'
         action.timeout = 10
         self.patchobject(action, 'is_timeout', return_value=True)
@@ -460,9 +458,9 @@ class ActionBaseTest(base.SenlinTestCase):
         res = action._check_signal()
         self.assertEqual(action.RES_TIMEOUT, res)
 
-    @mock.patch.object(db_api, 'action_signal_query')
+    @mock.patch.object(ao.Action, 'signal_query')
     def test_check_signal_signals_caught(self, mock_query):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.id = 'FAKE_ID'
         action.timeout = 100
         self.patchobject(action, 'is_timeout', return_value=False)
@@ -473,9 +471,9 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertEqual(sig_cmd, res)
         mock_query.assert_called_once_with(action.context, 'FAKE_ID')
 
-    @mock.patch.object(db_api, 'action_signal_query')
+    @mock.patch.object(ao.Action, 'signal_query')
     def test_is_cancelled(self, mock_query):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.id = 'FAKE_ID'
         action.timeout = 100
         self.patchobject(action, 'is_timeout', return_value=False)
@@ -491,9 +489,9 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertFalse(res)
         mock_query.assert_called_once_with(action.context, 'FAKE_ID')
 
-    @mock.patch.object(db_api, 'action_signal_query')
+    @mock.patch.object(ao.Action, 'signal_query')
     def test_is_suspended(self, mock_query):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.id = 'FAKE_ID'
         action.timeout = 100
         self.patchobject(action, 'is_timeout', return_value=False)
@@ -509,9 +507,9 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertFalse(res)
         mock_query.assert_called_once_with(action.context, 'FAKE_ID')
 
-    @mock.patch.object(db_api, 'action_signal_query')
+    @mock.patch.object(ao.Action, 'signal_query')
     def test_is_resumed(self, mock_query):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.id = 'FAKE_ID'
         action.timeout = 100
         self.patchobject(action, 'is_timeout', return_value=False)
@@ -529,14 +527,14 @@ class ActionBaseTest(base.SenlinTestCase):
 
     @mock.patch.object(cp_mod.ClusterPolicy, 'load_all')
     def test_policy_check_target_invalid(self, mock_load):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         res = action.policy_check('FAKE_CLUSTER', 'WHEN')
         self.assertIsNone(res)
         self.assertEqual(0, mock_load.call_count)
 
     @mock.patch.object(cp_mod.ClusterPolicy, 'load_all')
     def test_policy_check_no_bindings(self, mock_load):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         mock_load.return_value = []
         res = action.policy_check('FAKE_CLUSTER', 'BEFORE')
         self.assertIsNone(res)
@@ -545,21 +543,22 @@ class ActionBaseTest(base.SenlinTestCase):
                                           sort='priority',
                                           filters={'enabled': True})
 
-    @mock.patch.object(db_api, 'dependency_get_depended')
-    @mock.patch.object(db_api, 'dependency_get_dependents')
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    @mock.patch.object(dobj.Dependency, 'get_dependents')
     def test_action_to_dict(self, mock_dep_by, mock_dep_on):
         mock_dep_on.return_value = ['ACTION_1']
         mock_dep_by.return_value = ['ACTION_2']
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx,
-                                    **self.action_values)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx,
+                           **self.action_values)
         action.id = 'FAKE_ID'
+        ts = common_utils.isotime(self.action_values['created_at'])
         expected = {
             'id': 'FAKE_ID',
             'name': 'FAKE_NAME',
             'action': 'OBJECT_ACTION',
-            'target': 'OBJID',
+            'target': OBJID,
             'cause': 'FAKE_CAUSE',
-            'owner': 'FAKE_OWNER',
+            'owner': OWNER_ID,
             'interval': 60,
             'start_time': 0,
             'end_time': 0,
@@ -570,7 +569,7 @@ class ActionBaseTest(base.SenlinTestCase):
             'outputs': {'key': 'output_value'},
             'depends_on': ['ACTION_1'],
             'depended_by': ['ACTION_2'],
-            'created_at': None,
+            'created_at': ts,
             'updated_at': None,
             'data': {'data_key': 'data_value'},
         }
@@ -612,7 +611,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
     @mock.patch.object(policy_mod.Policy, 'load')
     def test_policy_check_missing_target(self, mock_load, mock_load_all,
                                          mock_pre_op, mock_post_op):
-        cluster_id = 'FAKE_CLUSTER_ID'
+        cluster_id = CLUSTER_ID
         # Note: policy is mocked
         spec = {
             'type': 'TestPolicy',
@@ -629,7 +628,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         mock_load.return_value = policy
         mock_pre_op.return_value = None
         mock_post_op.return_value = None
-        action = action_base.Action(cluster_id, 'OBJECT_ACTION_1', self.ctx)
+        action = ab.Action(cluster_id, 'OBJECT_ACTION_1', self.ctx)
 
         res = action.policy_check(cluster_id, 'AFTER')
 
@@ -645,10 +644,11 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         self.assertEqual(0, mock_pre_op.call_count)
         self.assertEqual(0, mock_post_op.call_count)
 
+    @mock.patch.object(EVENT, 'debug')
     @mock.patch.object(cp_mod.ClusterPolicy, 'load_all')
     @mock.patch.object(policy_mod.Policy, 'load')
-    def test_policy_check_pre_op(self, mock_load, mock_load_all):
-        cluster_id = 'FAKE_CLUSTER_ID'
+    def test_policy_check_pre_op(self, mock_load, mock_load_all, mock_event):
+        cluster_id = CLUSTER_ID
         # Note: policy is mocked
         spec = {
             'type': 'TestPolicy',
@@ -663,7 +663,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         self.assertIsNone(pb.last_op)
         mock_load_all.return_value = [pb]
         mock_load.return_value = policy
-        action = action_base.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
 
         res = action.policy_check(cluster_id, 'BEFORE')
 
@@ -676,10 +676,11 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         # last_op was not updated
         self.assertIsNone(pb.last_op)
 
+    @mock.patch.object(EVENT, 'debug')
     @mock.patch.object(cp_mod.ClusterPolicy, 'load_all')
     @mock.patch.object(policy_mod.Policy, 'load')
-    def test_policy_check_post_op(self, mock_load, mock_load_all):
-        cluster_id = 'FAKE_CLUSTER_ID'
+    def test_policy_check_post_op(self, mock_load, mock_load_all, mock_event):
+        cluster_id = CLUSTER_ID
         # Note: policy is mocked
         policy = mock.Mock()
         policy.id = 'FAKE_POLICY_ID'
@@ -690,9 +691,9 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         self.assertIsNone(pb.last_op)
         mock_load_all.return_value = [pb]
         mock_load.return_value = policy
-        action = action_base.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
 
-        res = action.policy_check('FAKE_CLUSTER_ID', 'AFTER')
+        res = action.policy_check(CLUSTER_ID, 'AFTER')
 
         self.assertIsNone(res)
         self.assertEqual(policy_mod.CHECK_OK, action.data['status'])
@@ -709,7 +710,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
     @mock.patch.object(cp_mod.ClusterPolicy, 'load_all')
     @mock.patch.object(policy_mod.Policy, 'load')
     def test_policy_check_cooldown_inprogress(self, mock_load, mock_load_all):
-        cluster_id = 'FAKE_CLUSTER_ID'
+        cluster_id = CLUSTER_ID
         # Note: policy is mocked
         policy = mock.Mock()
         policy.id = 'FAKE_POLICY_ID'
@@ -720,9 +721,9 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         self.assertIsNone(pb.last_op)
         mock_load_all.return_value = [pb]
         mock_load.return_value = policy
-        action = action_base.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
 
-        res = action.policy_check('FAKE_CLUSTER_ID', 'AFTER')
+        res = action.policy_check(CLUSTER_ID, 'AFTER')
 
         self.assertIsNone(res)
         self.assertEqual(policy_mod.CHECK_ERROR, action.data['status'])
@@ -740,10 +741,10 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
 
     @mock.patch.object(cp_mod.ClusterPolicy, 'load_all')
     @mock.patch.object(policy_mod.Policy, 'load')
-    @mock.patch.object(action_base.Action, '_check_result')
+    @mock.patch.object(ab.Action, '_check_result')
     def test_policy_check_abort_in_middle(self, mock_check, mock_load,
                                           mock_load_all):
-        cluster_id = 'FAKE_CLUSTER_ID'
+        cluster_id = CLUSTER_ID
         # Note: both policies are mocked
         policy1 = mock.Mock()
         policy1.id = 'FAKE_POLICY_ID_1'
@@ -756,7 +757,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         policy2.cooldown = 0
         policy2.TARGET = [('AFTER', 'OBJECT_ACTION')]
 
-        action = action_base.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
 
         # Note: policy binding is created but not stored
         pb1 = self._create_cp_binding(cluster_id, policy1.id)
@@ -789,18 +790,18 @@ class ActionProcTest(base.SenlinTestCase):
         self.ctx = utils.dummy_context()
 
     @mock.patch.object(EVENT, 'info')
-    @mock.patch.object(action_base.Action, 'load')
-    @mock.patch.object(db_api, 'action_mark_succeeded')
+    @mock.patch.object(ab.Action, 'load')
+    @mock.patch.object(ao.Action, 'mark_succeeded')
     def test_action_proc_successful(self, mock_mark, mock_load,
                                     mock_event_info):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
         action.owner = 'WORKER'
         action.start_time = 123456
         self.patchobject(action, 'execute',
                          return_value=(action.RES_OK, 'BIG SUCCESS'))
         mock_load.return_value = action
 
-        res = action_base.ActionProc(self.ctx, 'ACTION')
+        res = ab.ActionProc(self.ctx, 'ACTION')
         self.assertTrue(res)
 
         mock_load.assert_called_once_with(self.ctx, action_id='ACTION')
@@ -810,17 +811,20 @@ class ActionProcTest(base.SenlinTestCase):
         self.assertEqual('BIG SUCCESS', action.status_reason)
 
     @mock.patch.object(EVENT, 'info')
-    @mock.patch.object(action_base.Action, 'load')
-    @mock.patch.object(db_api, 'action_mark_failed')
+    @mock.patch.object(ab.Action, 'load')
+    @mock.patch.object(ao.Action, 'mark_failed')
     def test_action_proc_failed_error(self, mock_mark, mock_load,
                                       mock_event_info):
-        action = action_base.Action('OBJID', 'OBJECT_ACTION', self.ctx)
-        action.owner = 'WORKER'
+        action = ab.Action(OBJID, 'CLUSTER_ACTION', self.ctx)
+        action.id = '5eb0c9a5-149a-4cd7-875f-0351c3cc69d5'
+        action.owner = OWNER_ID
         action.start_time = 123456
+        action.cluster = mock.Mock(id=CLUSTER_ID)
+        action.cluster.name = 'fake-cluster'
         self.patchobject(action, 'execute', side_effect=Exception('Boom!'))
         mock_load.return_value = action
 
-        res = action_base.ActionProc(self.ctx, 'ACTION')
+        res = ab.ActionProc(self.ctx, 'ACTION')
         self.assertFalse(res)
 
         mock_load.assert_called_once_with(self.ctx, action_id='ACTION')
