@@ -16,9 +16,9 @@ from oslo_utils import uuidutils
 import six
 
 from senlin.common import exception as exc
-from senlin.db import api as db_api
 from senlin.engine.actions import base as action_base
 from senlin.engine import service
+from senlin.objects import action as ao
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -31,7 +31,7 @@ class ActionTest(base.SenlinTestCase):
         self.eng = service.EngineService('host-a', 'topic-a')
         self.eng.init_tgm()
 
-    @mock.patch.object(db_api, 'action_get')
+    @mock.patch.object(ao.Action, 'get')
     def test_action_find_by_uuid(self, mock_get):
         x_action = mock.Mock()
         mock_get.return_value = x_action
@@ -42,8 +42,8 @@ class ActionTest(base.SenlinTestCase):
         self.assertEqual(x_action, result)
         mock_get.assert_called_once_with(self.ctx, aid, project_safe=True)
 
-    @mock.patch.object(db_api, 'action_get_by_name')
-    @mock.patch.object(db_api, 'action_get')
+    @mock.patch.object(ao.Action, 'get_by_name')
+    @mock.patch.object(ao.Action, 'get')
     def test_action_find_by_uuid_as_name(self, mock_get, mock_get_name):
         x_action = mock.Mock()
         mock_get_name.return_value = x_action
@@ -57,7 +57,7 @@ class ActionTest(base.SenlinTestCase):
         mock_get_name.assert_called_once_with(self.ctx, aid,
                                               project_safe=False)
 
-    @mock.patch.object(db_api, 'action_get_by_name')
+    @mock.patch.object(ao.Action, 'get_by_name')
     def test_action_find_by_name(self, mock_get_name):
         x_action = mock.Mock()
         mock_get_name.return_value = x_action
@@ -68,8 +68,8 @@ class ActionTest(base.SenlinTestCase):
         self.assertEqual(x_action, result)
         mock_get_name.assert_called_once_with(self.ctx, aid, project_safe=True)
 
-    @mock.patch.object(db_api, 'action_get_by_short_id')
-    @mock.patch.object(db_api, 'action_get_by_name')
+    @mock.patch.object(ao.Action, 'get_by_short_id')
+    @mock.patch.object(ao.Action, 'get_by_name')
     def test_action_find_by_shortid(self, mock_get_name, mock_get_shortid):
         x_action = mock.Mock()
         mock_get_shortid.return_value = x_action
@@ -84,11 +84,11 @@ class ActionTest(base.SenlinTestCase):
         mock_get_shortid.assert_called_once_with(self.ctx, aid,
                                                  project_safe=False)
 
-    @mock.patch.object(db_api, 'action_get_by_name')
+    @mock.patch.object(ao.Action, 'get_by_name')
     def test_action_find_not_found(self, mock_get_name):
         mock_get_name.return_value = None
 
-        ex = self.assertRaises(exc.ActionNotFound,
+        ex = self.assertRaises(exc.ResourceNotFound,
                                self.eng.action_find,
                                self.ctx, 'bogus')
         self.assertEqual('The action (bogus) could not be found.',
@@ -200,13 +200,13 @@ class ActionTest(base.SenlinTestCase):
 
     @mock.patch.object(service.EngineService, 'cluster_find')
     def test_action_create_cluster_not_found(self, mock_find):
-        mock_find.side_effect = exc.ClusterNotFound(cluster='C1')
+        mock_find.side_effect = exc.ResourceNotFound(type='cluster', id='C1')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.action_create,
                                self.ctx, 'a1', 'C1', 'OBJECT_ACTION')
 
-        self.assertEqual(exc.ClusterNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         mock_find.assert_called_once_with(self.ctx, 'C1')
 
     @mock.patch.object(action_base.Action, 'load')
@@ -226,12 +226,12 @@ class ActionTest(base.SenlinTestCase):
 
     @mock.patch.object(service.EngineService, 'action_find')
     def test_action_get_not_found(self, mock_find):
-        mock_find.side_effect = exc.ActionNotFound(action='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='action', id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.action_get,
                                self.ctx, 'Bogus')
-        self.assertEqual(exc.ActionNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
     @mock.patch.object(action_base.Action, 'delete')
@@ -253,8 +253,7 @@ class ActionTest(base.SenlinTestCase):
         x_obj = mock.Mock()
         x_obj.id = 'FAKE_ID'
         mock_find.return_value = x_obj
-        ex = exc.ResourceBusyError(resource_type='action',
-                                   resource_id='FAKE_ID')
+        ex = exc.EResourceBusy(type='action', id='FAKE_ID')
         mock_delete.side_effect = ex
 
         ex = self.assertRaises(rpc.ExpectedException,
@@ -262,15 +261,18 @@ class ActionTest(base.SenlinTestCase):
                                self.ctx, 'ACTION')
 
         self.assertEqual(exc.ResourceInUse, ex.exc_info[0])
+        self.assertEqual("The action ACTION cannot be deleted: still in one "
+                         "of WAITING, RUNNING or SUSPENDED state.",
+                         six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'ACTION')
         mock_delete.assert_called_once_with(self.ctx, 'FAKE_ID')
 
     @mock.patch.object(service.EngineService, 'action_find')
     def test_action_delete_not_found(self, mock_find):
-        mock_find.side_effect = exc.ActionNotFound(action='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='action', id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.action_delete,
                                self.ctx, 'Bogus')
 
-        self.assertEqual(exc.ActionNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
