@@ -19,11 +19,12 @@ import six
 from senlin.common import consts
 from senlin.common import exception as exc
 from senlin.common.i18n import _
-from senlin.db.sqlalchemy import api as db_api
 from senlin.engine.actions import base as action_mod
 from senlin.engine import dispatcher
 from senlin.engine import node as node_mod
 from senlin.engine import service
+from senlin.objects import cluster as co
+from senlin.objects import node as no
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -35,7 +36,7 @@ class NodeTest(base.SenlinTestCase):
         self.ctx = utils.dummy_context(project='node_test_project')
         self.eng = service.EngineService('host-a', 'topic-a')
 
-    @mock.patch.object(db_api, 'node_get')
+    @mock.patch.object(no.Node, 'get')
     def test_node_find_by_uuid(self, mock_get):
         x_node = mock.Mock()
         mock_get.return_value = x_node
@@ -46,8 +47,8 @@ class NodeTest(base.SenlinTestCase):
         self.assertEqual(x_node, result)
         mock_get.assert_called_once_with(self.ctx, aid, project_safe=True)
 
-    @mock.patch.object(db_api, 'node_get_by_name')
-    @mock.patch.object(db_api, 'node_get')
+    @mock.patch.object(no.Node, 'get_by_name')
+    @mock.patch.object(no.Node, 'get')
     def test_node_find_by_uuid_as_name(self, mock_get, mock_name):
         mock_get.return_value = None
         x_node = mock.Mock()
@@ -60,7 +61,7 @@ class NodeTest(base.SenlinTestCase):
         mock_get.assert_called_once_with(self.ctx, aid, project_safe=False)
         mock_name.assert_called_once_with(self.ctx, aid, project_safe=False)
 
-    @mock.patch.object(db_api, 'node_get_by_name')
+    @mock.patch.object(no.Node, 'get_by_name')
     def test_node_find_by_name(self, mock_name):
         x_node = mock.Mock()
         mock_name.return_value = x_node
@@ -71,8 +72,8 @@ class NodeTest(base.SenlinTestCase):
         self.assertEqual(x_node, result)
         mock_name.assert_called_once_with(self.ctx, aid, project_safe=True)
 
-    @mock.patch.object(db_api, 'node_get_by_short_id')
-    @mock.patch.object(db_api, 'node_get_by_name')
+    @mock.patch.object(no.Node, 'get_by_short_id')
+    @mock.patch.object(no.Node, 'get_by_name')
     def test_node_find_by_short_id(self, mock_name, mock_shortid):
         mock_name.return_value = None
         x_node = mock.Mock()
@@ -85,11 +86,11 @@ class NodeTest(base.SenlinTestCase):
         mock_name.assert_called_once_with(self.ctx, aid, project_safe=False)
         mock_shortid.assert_called_once_with(self.ctx, aid, project_safe=False)
 
-    @mock.patch.object(db_api, 'node_get_by_short_id')
+    @mock.patch.object(no.Node, 'get_by_short_id')
     def test_node_find_not_found(self, mock_shortid):
         mock_shortid.return_value = None
 
-        ex = self.assertRaises(exc.NodeNotFound,
+        ex = self.assertRaises(exc.ResourceNotFound,
                                self.eng.node_find,
                                self.ctx, 'BOGUS')
         self.assertEqual("The node (BOGUS) could not be found.",
@@ -180,13 +181,14 @@ class NodeTest(base.SenlinTestCase):
 
     @mock.patch.object(service.EngineService, 'cluster_find')
     def test_node_list_cluster_not_found(self, mock_find):
-        mock_find.side_effect = exc.ClusterNotFound(cluster='BOGUS')
+        mock_find.side_effect = exc.ResourceNotFound(type='cluster',
+                                                     id='BOGUS')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_list,
                                self.ctx, cluster_id='BOGUS')
 
-        self.assertEqual(exc.ClusterNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual("The cluster (BOGUS) could not be found.",
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'BOGUS')
@@ -268,11 +270,11 @@ class NodeTest(base.SenlinTestCase):
             name='node_create_NODE_ID',
             cause=action_mod.CAUSE_RPC,
             status=action_mod.Action.READY)
-        notify.assert_called_once_with(action_id='ACTION_ID')
+        notify.assert_called_once_with()
 
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch('senlin.engine.node.Node')
-    @mock.patch.object(db_api, 'cluster_next_index')
+    @mock.patch.object(co.Cluster, 'get_next_index')
     @mock.patch.object(service.EngineService, 'cluster_find')
     @mock.patch.object(service.EngineService, 'profile_find')
     @mock.patch.object(dispatcher, 'start_action')
@@ -307,11 +309,11 @@ class NodeTest(base.SenlinTestCase):
             name='node_create_NODE_ID',
             cause=action_mod.CAUSE_RPC,
             status=action_mod.Action.READY)
-        notify.assert_called_once_with(action_id='ACTION_ID')
+        notify.assert_called_once_with()
 
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch('senlin.engine.node.Node')
-    @mock.patch.object(db_api, 'cluster_next_index')
+    @mock.patch.object(co.Cluster, 'get_next_index')
     @mock.patch.object(service.EngineService, 'cluster_find')
     @mock.patch.object(service.EngineService, 'profile_find')
     @mock.patch.object(dispatcher, 'start_action')
@@ -352,9 +354,9 @@ class NodeTest(base.SenlinTestCase):
             name='node_create_NODE_ID',
             cause=action_mod.CAUSE_RPC,
             status=action_mod.Action.READY)
-        notify.assert_called_once_with(action_id='ACTION_ID')
+        notify.assert_called_once_with()
 
-    @mock.patch.object(db_api, 'node_get_by_name')
+    @mock.patch.object(no.Node, 'get_by_name')
     def test_node_create_name_conflict(self, mock_get):
         cfg.CONF.set_override('name_unique', True, enforce_type=True)
         mock_get.return_value = mock.Mock()
@@ -370,14 +372,15 @@ class NodeTest(base.SenlinTestCase):
 
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_node_create_profile_not_found(self, mock_profile):
-        mock_profile.side_effect = exc.ProfileNotFound(profile='Bogus')
+        mock_profile.side_effect = exc.ResourceNotFound(type='profile',
+                                                        id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_create,
                                self.ctx, 'n-1', 'Bogus')
         self.assertEqual(exc.BadRequest, ex.exc_info[0])
         self.assertEqual("The request is malformed: The specified profile "
-                         "(Bogus) is not found.",
+                         "(Bogus) could not be found.",
                          six.text_type(ex.exc_info[1]))
         mock_profile.assert_called_once_with(self.ctx, 'Bogus')
 
@@ -385,7 +388,8 @@ class NodeTest(base.SenlinTestCase):
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_node_create_cluster_not_found(self, mock_profile, mock_cluster):
         mock_profile.return_value = mock.Mock()
-        mock_cluster.side_effect = exc.ClusterNotFound(cluster='Bogus')
+        mock_cluster.side_effect = exc.ResourceNotFound(type='cluster',
+                                                        id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_create,
@@ -394,7 +398,7 @@ class NodeTest(base.SenlinTestCase):
 
         self.assertEqual(exc.BadRequest, ex.exc_info[0])
         self.assertEqual("The request is malformed: The specified cluster "
-                         "(Bogus) is not found.",
+                         "(Bogus) could not be found.",
                          six.text_type(ex.exc_info[1]))
         mock_profile.assert_called_once_with(self.ctx, 'FAKE_PROFILE')
         mock_cluster.assert_called_once_with(self.ctx, 'Bogus')
@@ -438,7 +442,7 @@ class NodeTest(base.SenlinTestCase):
 
         self.assertEqual({'foo': 'bar'}, result)
         mock_find.assert_called_once_with(self.ctx, 'FAKE_NODE')
-        mock_load.assert_called_once_with(self.ctx, node=x_obj)
+        mock_load.assert_called_once_with(self.ctx, db_node=x_obj)
         x_node.to_dict.assert_called_once_with()
 
     @mock.patch.object(node_mod.Node, 'load')
@@ -456,19 +460,19 @@ class NodeTest(base.SenlinTestCase):
         self.assertEqual({'foo': 'bar', 'details': {'info': 'blahblah'}},
                          result)
         mock_find.assert_called_once_with(self.ctx, 'FAKE_NODE')
-        mock_load.assert_called_once_with(self.ctx, node=x_obj)
+        mock_load.assert_called_once_with(self.ctx, db_node=x_obj)
         x_node.to_dict.assert_called_once_with()
         x_node.get_details.assert_called_once_with(self.ctx)
 
     @mock.patch.object(service.EngineService, 'node_find')
     def test_node_get_node_not_found(self, mock_find):
-        mock_find.side_effect = exc.NodeNotFound(node='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='node', id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_get,
                                self.ctx, 'Bogus')
 
-        self.assertEqual(exc.NodeNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual("The node (Bogus) could not be found.",
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
@@ -485,7 +489,7 @@ class NodeTest(base.SenlinTestCase):
 
         self.assertEqual({'foo': 'bar'}, result)
         mock_find.assert_called_once_with(self.ctx, 'FAKE_NODE')
-        mock_load.assert_called_once_with(self.ctx, node=x_obj)
+        mock_load.assert_called_once_with(self.ctx, db_node=x_obj)
         x_node.to_dict.assert_called_once_with()
         self.assertEqual(0, x_node.get_details.call_count)
 
@@ -520,8 +524,8 @@ class NodeTest(base.SenlinTestCase):
                     'KEY': 'V1',
                 }
             })
-        mock_start.assert_called_once_with(action_id='ACTION_ID')
-        mock_load.assert_called_once_with(self.ctx, node=x_obj)
+        mock_start.assert_called_once_with()
+        mock_load.assert_called_once_with(self.ctx, db_node=x_obj)
 
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(action_mod.Action, 'create')
@@ -564,17 +568,17 @@ class NodeTest(base.SenlinTestCase):
             inputs={
                 'new_profile_id': 'NEW_PROFILE_ID',
             })
-        mock_start.assert_called_once_with(action_id='ACTION_ID')
-        mock_load.assert_called_once_with(self.ctx, node=x_obj)
+        mock_start.assert_called_once_with()
+        mock_load.assert_called_once_with(self.ctx, db_node=x_obj)
 
     @mock.patch.object(service.EngineService, 'node_find')
     def test_node_update_node_not_found(self, mock_find):
-        mock_find.side_effect = exc.NodeNotFound(node='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='node', id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_update, self.ctx, 'Bogus')
 
-        self.assertEqual(exc.NodeNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual('The node (Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
@@ -583,7 +587,8 @@ class NodeTest(base.SenlinTestCase):
     @mock.patch.object(service.EngineService, 'node_find')
     def test_node_update_profile_not_found(self, mock_find, mock_profile):
         mock_find.return_value = mock.Mock()
-        mock_profile.side_effect = exc.ProfileNotFound(profile='Bogus')
+        mock_profile.side_effect = exc.ResourceNotFound(type='profile',
+                                                        id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_update,
@@ -591,7 +596,7 @@ class NodeTest(base.SenlinTestCase):
 
         self.assertEqual(exc.BadRequest, ex.exc_info[0])
         self.assertEqual('The request is malformed: The specified profile '
-                         '(Bogus) is not found.',
+                         '(Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'FAKE_NODE')
         mock_profile.assert_called_once_with(self.ctx, 'Bogus')
@@ -641,7 +646,8 @@ class NodeTest(base.SenlinTestCase):
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch.object(service.EngineService, 'node_find')
     def test_node_delete(self, mock_find, mock_action, mock_start):
-        mock_find.return_value = mock.Mock(id='12345678AB')
+        mock_find.return_value = mock.Mock(id='12345678AB', status='ACTIVE',
+                                           dependents={})
         mock_action.return_value = 'ACTION_ID'
 
         result = self.eng.node_delete(self.ctx, 'FAKE_NODE')
@@ -653,20 +659,48 @@ class NodeTest(base.SenlinTestCase):
             name='node_delete_12345678',
             cause=action_mod.CAUSE_RPC,
             status=action_mod.Action.READY)
-        mock_start.assert_called_once_with(action_id='ACTION_ID')
+        mock_start.assert_called_once_with()
 
     @mock.patch.object(service.EngineService, 'node_find')
     def test_node_delete_node_not_found(self, mock_find):
-        mock_find.side_effect = exc.NodeNotFound(node='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='node', id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_delete, self.ctx, 'Bogus')
 
-        self.assertEqual(exc.NodeNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual('The node (Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
+    @mock.patch.object(service.EngineService, 'node_find')
+    def test_node_delete_improper_status(self, mock_find):
+        for bad_status in [node_mod.Node.CREATING, node_mod.Node.UPDATING,
+                           node_mod.Node.DELETING, node_mod.Node.RECOVERING]:
+            fake_node = mock.Mock(id='12345678AB', status=bad_status)
+            mock_find.return_value = fake_node
+            ex = self.assertRaises(rpc.ExpectedException,
+                                   self.eng.node_delete,
+                                   self.ctx, 'BUSY')
+
+            self.assertEqual(exc.ActionInProgress, ex.exc_info[0])
+            self.assertEqual("The node BUSY is in status %s." % bad_status,
+                             six.text_type(ex.exc_info[1]))
+            # skipping assertion on mock_find
+
+    @mock.patch.object(service.EngineService, 'node_find')
+    def test_node_delete_contain_container(self, mock_find):
+        dependents = {'containers': ['container1']}
+        node = mock.Mock(id='NODE_ID', status='ACTIVE', dependents=dependents)
+        mock_find.return_value = node
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.node_delete,
+                               self.ctx, 'node1')
+        self.assertEqual(exc.ResourceInUse, ex.exc_info[0])
+        self.assertEqual("The node node1 cannot be deleted: still depended "
+                         "by other clusters and/or nodes.",
+                         six.text_type(ex.exc_info[1]))
+
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch.object(service.EngineService, 'node_find')
@@ -685,17 +719,17 @@ class NodeTest(base.SenlinTestCase):
             cause=action_mod.CAUSE_RPC,
             status=action_mod.Action.READY,
             inputs={})
-        mock_start.assert_called_once_with(action_id='ACTION_ID')
+        mock_start.assert_called_once_with()
 
     @mock.patch.object(service.EngineService, 'node_find')
     def test_node_check_not_found(self, mock_find):
-        mock_find.side_effect = exc.NodeNotFound(node='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='node', id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_check,
                                self.ctx, 'Bogus')
 
-        self.assertEqual(exc.NodeNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual('The node (Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
@@ -718,17 +752,17 @@ class NodeTest(base.SenlinTestCase):
             cause=action_mod.CAUSE_RPC,
             status=action_mod.Action.READY,
             inputs={'foo': 'bar'})
-        mock_start.assert_called_once_with(action_id='ACTION_ID')
+        mock_start.assert_called_once_with()
 
     @mock.patch.object(service.EngineService, 'node_find')
     def test_node_recover_not_found(self, mock_find):
-        mock_find.side_effect = exc.NodeNotFound(node='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='node', id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_recover,
                                self.ctx, 'Bogus')
 
-        self.assertEqual(exc.NodeNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual('The node (Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
