@@ -17,10 +17,10 @@ from oslo_utils import uuidutils
 import six
 
 from senlin.common import exception as exc
-from senlin.db.sqlalchemy import api as db_api
 from senlin.engine import environment
 from senlin.engine import service
-from senlin.profiles import base as profile_mod
+from senlin.objects import profile as po
+from senlin.profiles import base as pb
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 from senlin.tests.unit import fakes
@@ -52,7 +52,7 @@ class ProfileTest(base.SenlinTestCase):
             }
         }
 
-    @mock.patch.object(db_api, 'profile_get')
+    @mock.patch.object(po.Profile, 'get')
     def test_profile_find_by_uuid(self, mock_get):
         x_profile = mock.Mock()
         mock_get.return_value = x_profile
@@ -63,8 +63,8 @@ class ProfileTest(base.SenlinTestCase):
         self.assertEqual(x_profile, result)
         mock_get.assert_called_once_with(self.ctx, aid, project_safe=True)
 
-    @mock.patch.object(db_api, 'profile_get_by_name')
-    @mock.patch.object(db_api, 'profile_get')
+    @mock.patch.object(po.Profile, 'get_by_name')
+    @mock.patch.object(po.Profile, 'get')
     def test_profile_find_by_uuid_as_name(self, mock_get, mock_get_name):
         x_profile = mock.Mock()
         mock_get_name.return_value = x_profile
@@ -78,7 +78,7 @@ class ProfileTest(base.SenlinTestCase):
         mock_get_name.assert_called_once_with(self.ctx, aid,
                                               project_safe=False)
 
-    @mock.patch.object(db_api, 'profile_get_by_name')
+    @mock.patch.object(po.Profile, 'get_by_name')
     def test_profile_find_by_name(self, mock_get_name):
         x_profile = mock.Mock()
         mock_get_name.return_value = x_profile
@@ -89,8 +89,8 @@ class ProfileTest(base.SenlinTestCase):
         self.assertEqual(x_profile, result)
         mock_get_name.assert_called_once_with(self.ctx, aid, project_safe=True)
 
-    @mock.patch.object(db_api, 'profile_get_by_short_id')
-    @mock.patch.object(db_api, 'profile_get_by_name')
+    @mock.patch.object(po.Profile, 'get_by_short_id')
+    @mock.patch.object(po.Profile, 'get_by_name')
     def test_profile_find_by_shortid(self, mock_get_name, mock_get_shortid):
         x_profile = mock.Mock()
         mock_get_shortid.return_value = x_profile
@@ -105,11 +105,11 @@ class ProfileTest(base.SenlinTestCase):
         mock_get_shortid.assert_called_once_with(self.ctx, aid,
                                                  project_safe=False)
 
-    @mock.patch.object(db_api, 'profile_get_by_name')
+    @mock.patch.object(po.Profile, 'get_by_name')
     def test_profile_find_not_found(self, mock_get_name):
         mock_get_name.return_value = None
 
-        ex = self.assertRaises(exc.ProfileNotFound,
+        ex = self.assertRaises(exc.ResourceNotFound,
                                self.eng.profile_find,
                                self.ctx, 'Bogus')
 
@@ -118,7 +118,7 @@ class ProfileTest(base.SenlinTestCase):
         mock_get_name.assert_called_once_with(self.ctx, 'Bogus',
                                               project_safe=True)
 
-    @mock.patch.object(profile_mod.Profile, 'load_all')
+    @mock.patch.object(pb.Profile, 'load_all')
     def test_profile_list(self, mock_load):
         x_obj_1 = mock.Mock()
         x_obj_1.to_dict.return_value = {'k': 'v1'}
@@ -133,7 +133,7 @@ class ProfileTest(base.SenlinTestCase):
                                           filters=None, sort=None,
                                           project_safe=True)
 
-    @mock.patch.object(profile_mod.Profile, 'load_all')
+    @mock.patch.object(pb.Profile, 'load_all')
     def test_profile_list_with_params(self, mock_load):
         mock_load.return_value = []
 
@@ -163,7 +163,7 @@ class ProfileTest(base.SenlinTestCase):
                                self.ctx, project_safe='no')
         self.assertEqual(exc.InvalidParameter, ex.exc_info[0])
 
-    @mock.patch.object(profile_mod.Profile, 'load_all')
+    @mock.patch.object(pb.Profile, 'load_all')
     def test_profile_list_with_project_safe(self, mock_load):
         mock_load.return_value = []
 
@@ -213,7 +213,7 @@ class ProfileTest(base.SenlinTestCase):
         self.assertIsNotNone(result['created_at'])
         self.assertIsNotNone(result['id'])
 
-    @mock.patch.object(db_api, 'profile_get_by_name')
+    @mock.patch.object(po.Profile, 'get_by_name')
     def test_profile_create_name_conflict(self, mock_get):
         cfg.CONF.set_override('name_unique', True, enforce_type=True)
         mock_get.return_value = mock.Mock()
@@ -234,8 +234,7 @@ class ProfileTest(base.SenlinTestCase):
         self.assertEqual("The request is malformed: A profile named "
                          "'FAKE_NAME' already exists.",
                          six.text_type(ex.exc_info[1]))
-        mock_get.assert_called_once_with(self.ctx, 'FAKE_NAME',
-                                         project_safe=True)
+        mock_get.assert_called_once_with(self.ctx, 'FAKE_NAME')
 
     def test_profile_create_type_not_found(self):
         # We skip the fakes setup, so we won't get the proper profile type
@@ -252,9 +251,9 @@ class ProfileTest(base.SenlinTestCase):
                                self.eng.profile_create,
                                self.ctx, 'p-2', spec)
 
-        self.assertEqual(exc.BadRequest, ex.exc_info[0])
-        self.assertEqual("The request is malformed: The specified profile "
-                         "type (FakeProfile-1.0) is not found.",
+        self.assertEqual(exc.SpecValidationFailed, ex.exc_info[0])
+        self.assertEqual("The specified profile_type "
+                         "(FakeProfile-1.0) could not be found.",
                          six.text_type(ex.exc_info[1]))
 
     def test_profile_create_invalid_spec(self):
@@ -280,11 +279,52 @@ class ProfileTest(base.SenlinTestCase):
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.profile_create,
                                self.ctx, 'p-2', self.spec)
-        self.assertEqual(exc.BadRequest, ex.exc_info[0])
-        self.assertEqual('The request is malformed: BOOM',
+        self.assertEqual(exc.SpecValidationFailed, ex.exc_info[0])
+        self.assertEqual('BOOM',
                          six.text_type(ex.exc_info[1]))
 
-    @mock.patch.object(profile_mod.Profile, 'load')
+    def test_profile_validate_pass(self):
+        self._setup_fakes()
+
+        expected_resp = {
+            'created_at': None,
+            'domain': '',
+            'id': None,
+            'metadata': None,
+            'name': 'validated_profile',
+            'project': 'profile_test_project',
+            'type': 'TestProfile-1.0',
+            'updated_at': None,
+            'user': 'test_user_id',
+            'spec': {
+                'type': 'TestProfile',
+                'version': '1.0',
+                'properties': {
+                    'INT': 1,
+                    'STR': 'str',
+                    'LIST': ['v1', 'v2'],
+                    'MAP': {'KEY1': 1, 'KEY2': 'v2'},
+                }
+            }
+        }
+
+        resp = self.eng.profile_validate(self.ctx, self.spec)
+        self.assertEqual(expected_resp, resp)
+
+    def test_profile_validate_failed(self):
+        self._setup_fakes()
+
+        mock_do_validate = self.patchobject(fakes.TestProfile, 'do_validate')
+        mock_do_validate.side_effect = exc.InvalidSpec(message='BOOM')
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.profile_validate,
+                               self.ctx, self.spec)
+        self.assertEqual(exc.SpecValidationFailed, ex.exc_info[0])
+        self.assertEqual('BOOM',
+                         six.text_type(ex.exc_info[1]))
+
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_get(self, mock_find, mock_load):
         x_obj = mock.Mock()
@@ -301,17 +341,18 @@ class ProfileTest(base.SenlinTestCase):
 
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_get_not_found(self, mock_find):
-        mock_find.side_effect = exc.ProfileNotFound(profile='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='profile',
+                                                     id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.profile_get, self.ctx, 'Bogus')
 
-        self.assertEqual(exc.ProfileNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual('The profile (Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
-    @mock.patch.object(profile_mod.Profile, 'load')
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_update(self, mock_find, mock_load):
         x_obj = mock.Mock()
@@ -335,18 +376,19 @@ class ProfileTest(base.SenlinTestCase):
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_update_not_found(self, mock_find):
 
-        mock_find.side_effect = exc.ProfileNotFound(profile='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='profile',
+                                                     id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.profile_update,
                                self.ctx, 'Bogus', name='NEW_NAME')
 
-        self.assertEqual(exc.ProfileNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual('The profile (Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
-    @mock.patch.object(profile_mod.Profile, 'load')
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_update_no_change(self, mock_find, mock_load):
         x_obj = mock.Mock()
@@ -356,16 +398,16 @@ class ProfileTest(base.SenlinTestCase):
         x_profile.to_dict.return_value = {'foo': 'bar'}
         mock_load.return_value = x_profile
 
-        result = self.eng.profile_update(self.ctx, 'FAKE_PROFILE',
-                                         name='OLD_NAME')
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.profile_update,
+                               self.ctx, 'FAKE_PROFILE')
 
-        self.assertEqual({'foo': 'bar'}, result)
-        mock_find.assert_called_once_with(self.ctx, 'FAKE_PROFILE')
-        mock_load.assert_called_once_with(self.ctx, profile=x_obj)
-        self.assertEqual(0, x_profile.store.call_count)
-        self.assertEqual('OLD_NAME', x_profile.name)
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual(
+            'The request is malformed: No property needs an update.',
+            six.text_type(ex.exc_info[1]))
 
-    @mock.patch.object(profile_mod.Profile, 'delete')
+    @mock.patch.object(pb.Profile, 'delete')
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_delete(self, mock_find, mock_delete):
         x_obj = mock.Mock(id='PROFILE_ID')
@@ -380,23 +422,23 @@ class ProfileTest(base.SenlinTestCase):
 
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_delete_not_found(self, mock_find):
-        mock_find.side_effect = exc.ProfileNotFound(profile='Bogus')
+        mock_find.side_effect = exc.ResourceNotFound(type='profile',
+                                                     id='Bogus')
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.profile_delete, self.ctx, 'Bogus')
 
-        self.assertEqual(exc.ProfileNotFound, ex.exc_info[0])
+        self.assertEqual(exc.ResourceNotFound, ex.exc_info[0])
         self.assertEqual('The profile (Bogus) could not be found.',
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
-    @mock.patch.object(profile_mod.Profile, 'delete')
+    @mock.patch.object(pb.Profile, 'delete')
     @mock.patch.object(service.EngineService, 'profile_find')
     def test_profile_delete_profile_in_use(self, mock_find, mock_delete):
         x_obj = mock.Mock(id='PROFILE_ID')
         mock_find.return_value = x_obj
-        err = exc.ResourceBusyError(resource_type='profile',
-                                    resource_id='PROFILE_ID')
+        err = exc.EResourceBusy(type='profile', id='PROFILE_ID')
         mock_delete.side_effect = err
 
         ex = self.assertRaises(rpc.ExpectedException,
@@ -404,7 +446,8 @@ class ProfileTest(base.SenlinTestCase):
                                self.ctx, 'FAKE_PROFILE')
 
         self.assertEqual(exc.ResourceInUse, ex.exc_info[0])
-        self.assertEqual('The profile (PROFILE_ID) is still in use.',
+        self.assertEqual("The profile FAKE_PROFILE cannot be deleted: still "
+                         "referenced by some clusters and/or nodes.",
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'FAKE_PROFILE')
         mock_delete.assert_called_once_with(self.ctx, 'PROFILE_ID')
