@@ -10,13 +10,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
+
 import mock
+import six
 
 from senlin.common import consts
 from senlin.common import context
-from senlin.common import exception
+from senlin.common import exception as exc
 from senlin.common import scaleutils
-from senlin.db import api as db_api
+from senlin.objects import cluster as co
+from senlin.objects import cluster_policy as cpo
+from senlin.objects import node as no
 from senlin.policies import affinity_policy as ap
 from senlin.policies import base as pb
 from senlin.tests.unit.common import base
@@ -44,33 +49,63 @@ class TestAffinityPolicy(base.SenlinTestCase):
         self.assertFalse(policy.enable_drs)
         self.assertIsNone(policy._novaclient)
 
-    @mock.patch("senlin.drivers.base.SenlinDriver")
-    def test_nova(self, mock_driver):
-        cluster = mock.Mock()
+    @mock.patch.object(pb.Policy, 'validate')
+    def test_validate_okay(self, mock_base_validate):
+        new_spec = copy.deepcopy(self.spec)
+        new_spec['properties']['availability_zone'] = 'NEWAZ'
+        policy = ap.AffinityPolicy('test-policy', new_spec)
+        nc = mock.Mock()
+        nc.validate_azs.return_value = ['NEWAZ']
+        policy._novaclient = nc
+        ctx = mock.Mock(user='U1', project='P1')
+
+        res = policy.validate(ctx, True)
+
+        self.assertTrue(res)
+        mock_base_validate.assert_called_once_with(ctx, True)
+        nc.validate_azs.assert_called_once_with(['NEWAZ'])
+
+    @mock.patch.object(pb.Policy, 'validate')
+    def test_validate_no_validate_props(self, mock_base_validate):
         policy = ap.AffinityPolicy('test-policy', self.spec)
-        mock_params = mock.Mock()
-        mock_build = self.patchobject(policy, '_build_conn_params',
-                                      return_value=mock_params)
-        x_driver = mock.Mock()
-        mock_driver.return_value = x_driver
+        ctx = mock.Mock(user='U1', project='P1')
 
-        result = policy.nova(cluster)
+        res = policy.validate(ctx, False)
 
-        x_nova = x_driver.compute.return_value
-        self.assertEqual(x_nova, result)
-        self.assertEqual(x_nova, policy._novaclient)
-        mock_build.assert_called_once_with(cluster)
-        x_driver.compute.assert_called_once_with(mock_params)
+        self.assertTrue(res)
+        mock_base_validate.assert_called_once_with(ctx, False)
 
-    def test_nova_already_initialized(self):
-        cluster = mock.Mock()
+    @mock.patch.object(pb.Policy, 'validate')
+    def test_validate_az_not_specified(self, mock_base_validate):
         policy = ap.AffinityPolicy('test-policy', self.spec)
-        x_nova = mock.Mock()
-        policy._novaclient = x_nova
+        nc = mock.Mock()
+        policy._novaclient = nc
+        ctx = mock.Mock(user='U1', project='P1')
 
-        result = policy.nova(cluster)
+        res = policy.validate(ctx, True)
 
-        self.assertEqual(x_nova, result)
+        self.assertTrue(res)
+        mock_base_validate.assert_called_once_with(ctx, True)
+        self.assertEqual(0, nc.validate_azs.call_count)
+
+    @mock.patch.object(pb.Policy, 'validate')
+    def test_validate_az_not_found(self, mock_base_validate):
+        new_spec = copy.deepcopy(self.spec)
+        new_spec['properties']['availability_zone'] = 'NEWAZ'
+        policy = ap.AffinityPolicy('test-policy', new_spec)
+        nc = mock.Mock()
+        nc.validate_azs.return_value = []  # this means not found
+        policy._novaclient = nc
+        ctx = mock.Mock(user='U1', project='P1')
+
+        ex = self.assertRaises(exc.InvalidSpec,
+                               policy.validate,
+                               ctx, True)
+
+        mock_base_validate.assert_called_once_with(ctx, True)
+        nc.validate_azs.assert_called_once_with(['NEWAZ'])
+        self.assertEqual("The specified availability_zone 'NEWAZ' could not "
+                         "be found.", six.text_type(ex))
 
     def test_attach_using_profile_hints(self):
         x_profile = mock.Mock()
@@ -80,11 +115,11 @@ class TestAffinityPolicy(base.SenlinTestCase):
                 'group': 'KONGFOO',
             }
         }
-        cluster = mock.Mock(id='CLUSTER_ID')
-        cluster.rt = {'profile': x_profile}
+        cluster = mock.Mock(id='CLUSTER_ID', user='UU', project='PP',
+                            rt={'profile': x_profile})
         x_group = mock.Mock(id='GROUP_ID', policies=[u'anti-affinity'])
         x_nova = mock.Mock()
-        x_nova.find_server_group.return_value = x_group
+        x_nova.server_group_find.return_value = x_group
 
         policy = ap.AffinityPolicy('test-policy', self.spec)
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -99,8 +134,8 @@ class TestAffinityPolicy(base.SenlinTestCase):
         self.assertEqual(x_data, data)
         self.assertTrue(res)
 
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.find_server_group.assert_called_once_with('KONGFOO', True)
+        mock_nova.assert_called_once_with('UU', 'PP')
+        x_nova.server_group_find.assert_called_once_with('KONGFOO', True)
         mock_build.assert_called_once_with({
             'servergroup_id': 'GROUP_ID',
             'inherited_group': True
@@ -111,11 +146,11 @@ class TestAffinityPolicy(base.SenlinTestCase):
         x_profile = mock.Mock()
         x_profile.type = 'os.nova.server-1.0'
         x_profile.spec = {'foo': 'bar'}
-        cluster = mock.Mock(id='CLUSTER_ID')
-        cluster.rt = {'profile': x_profile}
+        cluster = mock.Mock(id='CLUSTER_ID', user='UU', project='PP',
+                            rt={'profile': x_profile})
         x_group = mock.Mock(id='GROUP_ID', policies=['anti-affinity'])
         x_nova = mock.Mock()
-        x_nova.find_server_group.return_value = x_group
+        x_nova.server_group_find.return_value = x_group
 
         policy = ap.AffinityPolicy('test-policy', self.spec)
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -130,8 +165,8 @@ class TestAffinityPolicy(base.SenlinTestCase):
         self.assertTrue(res)
         self.assertEqual(x_data, data)
 
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.find_server_group.assert_called_once_with('KONGFU', True)
+        mock_nova.assert_called_once_with('UU', 'PP')
+        x_nova.server_group_find.assert_called_once_with('KONGFU', True)
         mock_build.assert_called_once_with({
             'servergroup_id': 'GROUP_ID',
             'inherited_group': True
@@ -142,12 +177,12 @@ class TestAffinityPolicy(base.SenlinTestCase):
         x_profile = mock.Mock()
         x_profile.spec = {'foo': 'bar'}
         x_profile.type = 'os.nova.server-1.0'
-        cluster = mock.Mock(id='CLUSTER_ID')
-        cluster.rt = {'profile': x_profile}
+        cluster = mock.Mock(id='CLUSTER_ID', user='USER', project='PROJ',
+                            rt={'profile': x_profile})
         x_group = mock.Mock(id='GROUP_ID')
         x_nova = mock.Mock()
-        x_nova.find_server_group.return_value = None
-        x_nova.create_server_group.return_value = x_group
+        x_nova.server_group_find.return_value = None
+        x_nova.server_group_create.return_value = x_group
 
         policy = ap.AffinityPolicy('test-policy', self.spec)
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -162,9 +197,9 @@ class TestAffinityPolicy(base.SenlinTestCase):
         self.assertTrue(res)
         self.assertEqual(x_data, data)
 
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.find_server_group.assert_called_once_with('KONGFU', True)
-        x_nova.create_server_group.assert_called_once_with(
+        mock_nova.assert_called_once_with('USER', 'PROJ')
+        x_nova.server_group_find.assert_called_once_with('KONGFU', True)
+        x_nova.server_group_create.assert_called_once_with(
             name='KONGFU',
             policies=[policy.ANTI_AFFINITY])
         mock_build.assert_called_once_with({
@@ -176,11 +211,11 @@ class TestAffinityPolicy(base.SenlinTestCase):
         x_profile = mock.Mock()
         x_profile.spec = {'foo': 'bar'}
         x_profile.type = 'os.nova.server-1.0'
-        cluster = mock.Mock(id='CLUSTER_ID')
-        cluster.rt = {'profile': x_profile}
+        cluster = mock.Mock(id='CLUSTER_ID', user='USER', project='PROJ',
+                            rt={'profile': x_profile})
         x_group = mock.Mock(id='GROUP_ID')
         x_nova = mock.Mock()
-        x_nova.create_server_group.return_value = x_group
+        x_nova.server_group_create.return_value = x_group
 
         policy = ap.AffinityPolicy('test-policy', self.spec)
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -195,8 +230,8 @@ class TestAffinityPolicy(base.SenlinTestCase):
         self.assertTrue(res)
         self.assertEqual(x_data, data)
 
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.create_server_group.assert_called_once_with(
+        mock_nova.assert_called_once_with('USER', 'PROJ')
+        x_nova.server_group_create.assert_called_once_with(
             name=mock.ANY,
             policies=[policy.ANTI_AFFINITY])
         mock_build.assert_called_once_with({
@@ -221,11 +256,11 @@ class TestAffinityPolicy(base.SenlinTestCase):
         x_profile = mock.Mock()
         x_profile.type = 'os.nova.server-1.0'
         x_profile.spec = {'foo': 'bar'}
-        cluster = mock.Mock(id='CLUSTER_ID')
-        cluster.rt = {'profile': x_profile}
+        cluster = mock.Mock(id='CLUSTER_ID', user='USER', project='PROJ',
+                            rt={'profile': x_profile})
         x_nova = mock.Mock()
-        err = exception.InternalError(code=500, message='Boom')
-        x_nova.find_server_group.side_effect = err
+        err = exc.InternalError(code=500, message='Boom')
+        x_nova.server_group_find.side_effect = err
 
         policy = ap.AffinityPolicy('test-policy', self.spec)
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -237,19 +272,19 @@ class TestAffinityPolicy(base.SenlinTestCase):
         self.assertFalse(res)
         self.assertEqual("Failed in retrieving servergroup 'KONGFU'.", data)
 
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.find_server_group.assert_called_once_with('KONGFU', True)
+        mock_nova.assert_called_once_with('USER', 'PROJ')
+        x_nova.server_group_find.assert_called_once_with('KONGFU', True)
 
     def test_attach_policies_not_match(self):
         self.spec['properties']['servergroup']['name'] = 'KONGFU'
         x_profile = mock.Mock()
         x_profile.type = 'os.nova.server-1.0'
         x_profile.spec = {'foo': 'bar'}
-        cluster = mock.Mock(id='CLUSTER_ID')
-        cluster.rt = {'profile': x_profile}
+        cluster = mock.Mock(id='CLUSTER_ID', user='U1', project='P1',
+                            rt={'profile': x_profile})
         x_group = mock.Mock(id='GROUP_ID', policies=['affinity'])
         x_nova = mock.Mock()
-        x_nova.find_server_group.return_value = x_group
+        x_nova.server_group_find.return_value = x_group
 
         policy = ap.AffinityPolicy('test-policy', self.spec)
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -263,19 +298,19 @@ class TestAffinityPolicy(base.SenlinTestCase):
                          "that of the existing servergroup (affinity).",
                          data)
 
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.find_server_group.assert_called_once_with('KONGFU', True)
+        mock_nova.assert_called_once_with('U1', 'P1')
+        x_nova.server_group_find.assert_called_once_with('KONGFU', True)
 
     def test_attach_failed_creating_server_group(self):
         self.spec['properties']['servergroup']['name'] = 'KONGFU'
         x_profile = mock.Mock()
         x_profile.type = 'os.nova.server-1.0'
         x_profile.spec = {'foo': 'bar'}
-        cluster = mock.Mock(id='CLUSTER_ID')
-        cluster.rt = {'profile': x_profile}
+        cluster = mock.Mock(id='CLUSTER_ID', user='U1', project='P1',
+                            rt={'profile': x_profile})
         x_nova = mock.Mock()
-        x_nova.find_server_group.return_value = None
-        x_nova.create_server_group.side_effect = Exception()
+        x_nova.server_group_find.return_value = None
+        x_nova.server_group_create.side_effect = Exception()
 
         policy = ap.AffinityPolicy('test-policy', self.spec)
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -287,13 +322,13 @@ class TestAffinityPolicy(base.SenlinTestCase):
         self.assertEqual('Failed in creating servergroup.', data)
         self.assertFalse(res)
 
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.find_server_group.assert_called_once_with('KONGFU', True)
-        x_nova.create_server_group.assert_called_once_with(
+        mock_nova.assert_called_once_with('U1', 'P1')
+        x_nova.server_group_find.assert_called_once_with('KONGFU', True)
+        x_nova.server_group_create.assert_called_once_with(
             name=mock.ANY,
             policies=[policy.ANTI_AFFINITY])
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     @mock.patch.object(context, 'get_admin_context')
     def test_detach_inherited(self, mock_context, mock_cp):
         cluster = mock.Mock(id='CLUSTER_ID')
@@ -321,10 +356,10 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_cp.assert_called_once_with(x_ctx, 'CLUSTER_ID', 'POLICY_ID')
         mock_extract.assert_called_once_with(x_binding.data)
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     @mock.patch.object(context, 'get_admin_context')
     def test_detach_not_inherited(self, mock_context, mock_cp):
-        cluster = mock.Mock(id='CLUSTER_ID')
+        cluster = mock.Mock(id='CLUSTER_ID', user='USER', project='PROJECT')
         x_ctx = mock.Mock()
         mock_context.return_value = x_ctx
         x_binding = mock.Mock()
@@ -338,7 +373,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_extract = self.patchobject(policy, '_extract_policy_data',
                                         return_value=policy_data)
         x_nova = mock.Mock()
-        x_nova.delete_server_group.return_value = None
+        x_nova.server_group_delete.return_value = None
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
 
         # do it
@@ -351,10 +386,10 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_context.assert_called_once_with()
         mock_cp.assert_called_once_with(x_ctx, 'CLUSTER_ID', 'POLICY_ID')
         mock_extract.assert_called_once_with(x_binding.data)
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.delete_server_group.assert_called_once_with('SERVERGROUP_ID')
+        mock_nova.assert_called_once_with('USER', 'PROJECT')
+        x_nova.server_group_delete.assert_called_once_with('SERVERGROUP_ID')
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     @mock.patch.object(context, 'get_admin_context')
     def test_detach_binding_not_found(self, mock_context, mock_cp):
         cluster = mock.Mock(id='CLUSTER_ID')
@@ -376,7 +411,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_context.assert_called_once_with()
         mock_cp.assert_called_once_with(x_ctx, 'CLUSTER_ID', 'POLICY_ID')
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     @mock.patch.object(context, 'get_admin_context')
     def test_detach_binding_data_empty(self, mock_context, mock_cp):
         cluster = mock.Mock(id='CLUSTER_ID')
@@ -398,7 +433,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_context.assert_called_once_with()
         mock_cp.assert_called_once_with(x_ctx, 'CLUSTER_ID', 'POLICY_ID')
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     @mock.patch.object(context, 'get_admin_context')
     def test_detach_policy_data_empty(self, mock_context, mock_cp):
         cluster = mock.Mock(id='CLUSTER_ID')
@@ -422,10 +457,10 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_cp.assert_called_once_with(x_ctx, 'CLUSTER_ID', 'POLICY_ID')
         mock_extract.assert_called_once_with({'foo': 'bar'})
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     @mock.patch.object(context, 'get_admin_context')
     def test_detach_failing_delete_sg(self, mock_context, mock_cp):
-        cluster = mock.Mock(id='CLUSTER_ID')
+        cluster = mock.Mock(id='CLUSTER_ID', user='USER', project='PROJ')
         x_ctx = mock.Mock()
         mock_context.return_value = x_ctx
         x_binding = mock.Mock(data={'foo': 'bar'})
@@ -439,7 +474,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_extract = self.patchobject(policy, '_extract_policy_data',
                                         return_value=policy_data)
         x_nova = mock.Mock()
-        x_nova.delete_server_group.side_effect = Exception()
+        x_nova.server_group_delete.side_effect = Exception()
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
 
         # do it
@@ -452,10 +487,10 @@ class TestAffinityPolicy(base.SenlinTestCase):
         mock_context.assert_called_once_with()
         mock_cp.assert_called_once_with(x_ctx, 'CLUSTER_ID', 'POLICY_ID')
         mock_extract.assert_called_once_with({'foo': 'bar'})
-        mock_nova.assert_called_once_with(cluster)
-        x_nova.delete_server_group.assert_called_once_with('SERVERGROUP_ID')
+        mock_nova.assert_called_once_with('USER', 'PROJ')
+        x_nova.server_group_delete.assert_called_once_with('SERVERGROUP_ID')
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     def test_pre_op(self, mock_cp):
         x_action = mock.Mock()
         x_action.data = {'creation': {'count': 2}}
@@ -497,7 +532,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
             x_action.data)
         x_action.store.assert_called_once_with(x_action.context)
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     def test_pre_op_use_scaleout_input(self, mock_cp):
         x_action = mock.Mock()
         x_action.data = {}
@@ -538,10 +573,48 @@ class TestAffinityPolicy(base.SenlinTestCase):
             x_action.data)
         x_action.store.assert_called_once_with(x_action.context)
 
-    @mock.patch.object(db_api, 'cluster_get')
-    @mock.patch.object(db_api, 'cluster_policy_get')
-    def test_pre_op_use_resize_params(self, mock_cp, mock_cluster):
-        def fake_parse_func(action, cluster):
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
+    def test_pre_op_for_node_create(self, mock_cp):
+        x_action = mock.Mock()
+        x_action.data = {}
+        x_action.action = consts.NODE_CREATE
+        x_binding = mock.Mock()
+        mock_cp.return_value = x_binding
+
+        policy_data = {
+            'servergroup_id': 'SERVERGROUP_ID',
+            'inherited_group': False,
+        }
+        policy = ap.AffinityPolicy('test-policy', self.spec)
+        policy.id = 'POLICY_ID'
+        mock_extract = self.patchobject(policy, '_extract_policy_data',
+                                        return_value=policy_data)
+
+        # do it
+        policy.pre_op('CLUSTER_ID', x_action)
+
+        mock_cp.assert_called_once_with(x_action.context, 'CLUSTER_ID',
+                                        'POLICY_ID')
+        mock_extract.assert_called_once_with(x_binding.data)
+        self.assertEqual(
+            {
+                'placement': {
+                    'count': 1,
+                    'placements': [
+                        {
+                            'servergroup': 'SERVERGROUP_ID'
+                        }
+                    ]
+                }
+            },
+            x_action.data)
+        x_action.store.assert_called_once_with(x_action.context)
+
+    @mock.patch.object(no.Node, 'count_by_cluster')
+    @mock.patch.object(co.Cluster, 'get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
+    def test_pre_op_use_resize_params(self, mock_cp, mock_cluster, mock_count):
+        def fake_parse_func(action, cluster, current):
             action.data = {
                 'creation': {
                     'count': 2
@@ -557,6 +630,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         }
         x_cluster = mock.Mock()
         mock_cluster.return_value = x_cluster
+        mock_count.return_value = 10
         mock_parse = self.patchobject(scaleutils, 'parse_resize_params',
                                       side_effect=fake_parse_func)
 
@@ -575,7 +649,8 @@ class TestAffinityPolicy(base.SenlinTestCase):
         # do it
         policy.pre_op('CLUSTER_ID', x_action)
 
-        mock_parse.assert_called_once_with(x_action, x_cluster)
+        mock_count.assert_called_once_with(x_action.context, 'CLUSTER_ID')
+        mock_parse.assert_called_once_with(x_action, x_cluster, 10)
         mock_cluster.assert_called_once_with(x_action.context, 'CLUSTER_ID')
         mock_cp.assert_called_once_with(x_action.context, 'CLUSTER_ID',
                                         'POLICY_ID')
@@ -600,10 +675,11 @@ class TestAffinityPolicy(base.SenlinTestCase):
             x_action.data)
         x_action.store.assert_called_once_with(x_action.context)
 
-    @mock.patch.object(db_api, 'cluster_get')
-    @mock.patch.object(db_api, 'cluster_policy_get')
-    def test_pre_op_resize_shrinking(self, mock_cp, mock_cluster):
-        def fake_parse_func(action, cluster):
+    @mock.patch.object(no.Node, 'count_by_cluster')
+    @mock.patch.object(co.Cluster, 'get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
+    def test_pre_op_resize_shrinking(self, mock_cp, mock_cluster, mock_count):
+        def fake_parse_func(action, cluster, current):
             action.data = {
                 'deletion': {
                     'count': 2
@@ -619,6 +695,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         }
         x_cluster = mock.Mock()
         mock_cluster.return_value = x_cluster
+        mock_count.return_value = 5
         mock_parse = self.patchobject(scaleutils, 'parse_resize_params',
                                       side_effect=fake_parse_func)
         policy = ap.AffinityPolicy('test-policy', self.spec)
@@ -628,12 +705,13 @@ class TestAffinityPolicy(base.SenlinTestCase):
         # do it
         policy.pre_op('CLUSTER_ID', x_action)
 
-        mock_parse.assert_called_once_with(x_action, x_cluster)
+        mock_count.assert_called_once_with(x_action.context, 'CLUSTER_ID')
+        mock_parse.assert_called_once_with(x_action, x_cluster, 5)
         mock_cluster.assert_called_once_with(x_action.context, 'CLUSTER_ID')
         self.assertEqual(0, mock_cp.call_count)
         self.assertEqual(0, mock_extract.call_count)
 
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     def test_pre_op_with_zone_name(self, mock_cp):
         self.spec['properties']['availability_zone'] = 'BLUE_ZONE'
         x_action = mock.Mock()
@@ -678,8 +756,8 @@ class TestAffinityPolicy(base.SenlinTestCase):
             x_action.data)
         x_action.store.assert_called_once_with(x_action.context)
 
-    @mock.patch.object(db_api, 'cluster_get')
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(co.Cluster, 'get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     def test_pre_op_with_drs_enabled(self, mock_cp, mock_cluster):
         self.spec['properties']['enable_drs_extension'] = True
         x_action = mock.Mock()
@@ -695,7 +773,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         policy.id = 'POLICY_ID'
         mock_extract = self.patchobject(policy, '_extract_policy_data',
                                         return_value=policy_data)
-        x_cluster = mock.Mock()
+        x_cluster = mock.Mock(user='USER', project='PROJ')
         mock_cluster.return_value = x_cluster
         x_nova = mock.Mock()
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -718,7 +796,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
                                         'POLICY_ID')
         mock_extract.assert_called_once_with(x_binding.data)
         mock_cluster.assert_called_once_with(x_action.context, 'CLUSTER_ID')
-        mock_nova.assert_called_once_with(x_cluster)
+        mock_nova.assert_called_once_with('USER', 'PROJ')
         x_nova.hypervisor_list.assert_called_once_with()
         x_nova.hypervisor_get.assert_called_once_with('HV_2')
         self.assertEqual(
@@ -743,8 +821,8 @@ class TestAffinityPolicy(base.SenlinTestCase):
             x_action.data)
         x_action.store.assert_called_once_with(x_action.context)
 
-    @mock.patch.object(db_api, 'cluster_get')
-    @mock.patch.object(db_api, 'cluster_policy_get')
+    @mock.patch.object(co.Cluster, 'get')
+    @mock.patch.object(cpo.ClusterPolicy, 'get')
     def test_pre_op_with_drs_enabled_no_match(self, mock_cp, mock_cluster):
         self.spec['properties']['enable_drs_extension'] = True
         x_action = mock.Mock()
@@ -760,7 +838,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
         policy.id = 'POLICY_ID'
         mock_extract = self.patchobject(policy, '_extract_policy_data',
                                         return_value=policy_data)
-        x_cluster = mock.Mock()
+        x_cluster = mock.Mock(user='USER', project='PROJ')
         mock_cluster.return_value = x_cluster
         x_nova = mock.Mock()
         mock_nova = self.patchobject(policy, 'nova', return_value=x_nova)
@@ -777,7 +855,7 @@ class TestAffinityPolicy(base.SenlinTestCase):
                                         'POLICY_ID')
         mock_extract.assert_called_once_with(x_binding.data)
         mock_cluster.assert_called_once_with(x_action.context, 'CLUSTER_ID')
-        mock_nova.assert_called_once_with(x_cluster)
+        mock_nova.assert_called_once_with('USER', 'PROJ')
         self.assertEqual(
             {
                 'creation': {
