@@ -10,16 +10,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import logging
-
+import datetime
+import mock
+from oslo_log import log as logging
+from oslo_utils import timeutils
 import requests
-from requests import exceptions
 import six
 
 from oslo_config import cfg
 
 from senlin.common import exception
 from senlin.common import utils
+from senlin.objects import service as service_obj
 from senlin.tests.unit.common import base
 
 
@@ -122,9 +124,6 @@ class Response(object):
 
 
 class UrlFetchTest(base.SenlinTestCase):
-    def setUp(self):
-        super(UrlFetchTest, self).setUp()
-
     def test_file_scheme_default_behaviour(self):
         self.assertRaises(utils.URLFetchError,
                           utils.url_fetch, 'file:///etc/profile')
@@ -162,13 +161,15 @@ class UrlFetchTest(base.SenlinTestCase):
     def test_http_error(self):
         url = 'http://example.com/somedata'
 
-        self.patchobject(requests, 'get', side_effect=exceptions.HTTPError())
+        self.patchobject(requests, 'get',
+                         side_effect=requests.exceptions.HTTPError())
         self.assertRaises(utils.URLFetchError, utils.url_fetch, url)
 
     def test_non_exist_url(self):
         url = 'http://non-exist.com/somedata'
 
-        self.patchobject(requests, 'get', side_effect=exceptions.Timeout())
+        self.patchobject(requests, 'get',
+                         side_effect=requests.exceptions.Timeout())
         self.assertRaises(utils.URLFetchError, utils.url_fetch, url)
 
     def test_garbage(self):
@@ -252,3 +253,60 @@ class TestParseLevelValues(base.SenlinTestCase):
     def test_with_only_invalid_values(self):
         res = utils.parse_level_values(['warn'])
         self.assertIsNone(res)
+
+
+class TestGetPathParser(base.SenlinTestCase):
+
+    def test_normal(self):
+        res = utils.get_path_parser('foo.bar')
+        self.assertIsNotNone(res)
+
+    def test_bad_path(self):
+        err = self.assertRaises(exception.BadRequest,
+                                utils.get_path_parser,
+                                '^foo.bar')
+        self.assertEqual("The request is malformed: Invalid attribute path - "
+                         "Unexpected character: ^.",
+                         six.text_type(err))
+
+
+class EngineDeathTest(base.SenlinTestCase):
+
+    def setUp(self):
+        super(EngineDeathTest, self).setUp()
+        self.ctx = mock.Mock()
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_engine_is_none(self, mock_service):
+        mock_service.return_value = None
+        self.assertTrue(utils.is_engine_dead(self.ctx, 'fake_engine_id'))
+        mock_service.assert_called_once_with(self.ctx, 'fake_engine_id')
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_engine_is_dead(self, mock_service):
+        delta = datetime.timedelta(seconds=3 * cfg.CONF.periodic_interval)
+        update_time = timeutils.utcnow(True) - delta
+        mock_service.return_value = mock.Mock(updated_at=update_time)
+
+        res = utils.is_engine_dead(self.ctx, 'fake_engine_id')
+
+        self.assertTrue(res)
+        mock_service.assert_called_once_with(self.ctx, 'fake_engine_id')
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_engine_is_alive(self, mock_svc):
+        mock_svc.return_value = mock.Mock(updated_at=timeutils.utcnow(True))
+
+        res = utils.is_engine_dead(self.ctx, 'fake_engine_id')
+
+        self.assertFalse(res)
+        mock_svc.assert_called_once_with(self.ctx, 'fake_engine_id')
+
+    @mock.patch.object(service_obj.Service, 'get')
+    def test_use_specified_duration(self, mock_svc):
+        mock_svc.return_value = mock.Mock(updated_at=timeutils.utcnow(True))
+
+        res = utils.is_engine_dead(self.ctx, 'fake_engine_id', 10000)
+
+        self.assertFalse(res)
+        mock_svc.assert_called_once_with(self.ctx, 'fake_engine_id')
