@@ -161,7 +161,7 @@ class EngineRpcAPITestCase(base.SenlinTestCase):
 
     def test_profile_update(self):
         default_args = {
-            'profile_id': 'aaaa-bbbb-cccc',
+            'identity': 'aaaa-bbbb-cccc',
             'name': mock.ANY,
             'metadata': mock.ANY,
         }
@@ -173,6 +173,10 @@ class EngineRpcAPITestCase(base.SenlinTestCase):
     def test_profile_delete_call(self):
         self._test_engine_api('profile_delete', 'call', identity='a-profile')
 
+    def test_profile_validate(self):
+        self._test_engine_api('profile_validate', 'call',
+                              spec=mock.ANY)
+
     def test_policy_type_list(self):
         self._test_engine_api('policy_type_list', 'call')
 
@@ -213,6 +217,9 @@ class EngineRpcAPITestCase(base.SenlinTestCase):
     def test_policy_delete_call(self):
         self._test_engine_api('policy_delete', 'call', identity='a-policy')
 
+    def test_policy_validate(self):
+        self._test_engine_api('policy_validate', 'call', spec=mock.ANY)
+
     def test_cluster_list(self):
         default_args = {
             'limit': mock.ANY,
@@ -275,6 +282,11 @@ class EngineRpcAPITestCase(base.SenlinTestCase):
     def test_cluster_delete_call(self):
         self._test_engine_api('cluster_delete', 'call', identity='a-cluster')
 
+    def test_cluster_collect(self):
+        self._test_engine_api('cluster_collect', 'call', identity='a-cluster',
+                              path='astring', project_safe=True,
+                              version='1.1')
+
     def test_cluster_check(self):
         self._test_engine_api('cluster_check', 'call', identity='a-cluster',
                               params=None)
